`default_nettype none
`timescale 1ns / 10ps

module wb_dualarbiter(
    input wire clk_i, rst_i,
    input wire [31:0] wbm0_adr_i,
    input wire [31:0] wbm1_adr_i, 
    input wire [31:0] wbm2_adr_i, 
    input wire [31:0] wbm3_adr_i, 
    input wire [31:0] wbm4_adr_i, 
		      
    input wire 	      cyc0_i,cyc1_i,cyc2_i,cyc3_i,cyc4_i,
    output wire       gnt0_o,gnt1_o,gnt2_o,gnt3_o, gnt4_o,
    output reg [4:0]  sels,
    output reg [1:0]  sel_num,
    output wire [1:0] gnt_num_o);


   reg [1:0] 	      gnt_num;

   reg [2:0] 	      grantstate;

   localparam [2:0]   GRANT1 = 3'd0;
   localparam [2:0]   GRANT2 = 3'd1;
   localparam [2:0]   GRANT3 = 3'd2;
   localparam [2:0]   GRANT4 = 3'd3;
   localparam [2:0]   GRANT0AND1 = 3'd4;
   localparam [2:0]   GRANT0AND2 = 3'd5;
   localparam [2:0]   GRANT0AND3 = 3'd6;
   localparam [2:0]   GRANT0AND4 = 3'd7;

   parameter [4:0]    BASEADDR = 5'h0;


   wire 		 wbm1_islocal,wbm2_islocal,wbm3_islocal,wbm4_islocal;

   defparam 		 localcheck1.LOCALMASK = 32'hf8000000;
   defparam 		 localcheck2.LOCALMASK = 32'hf8000000;
   defparam 		 localcheck3.LOCALMASK = 32'hf8000000;
   defparam 		 localcheck4.LOCALMASK = 32'hf8000000;
   defparam 		 localcheck1.LOCALADDR = {BASEADDR, 27'h0};
   defparam 		 localcheck2.LOCALADDR = {BASEADDR, 27'h0};
   defparam 		 localcheck3.LOCALADDR = {BASEADDR, 27'h0};
   defparam 		 localcheck4.LOCALADDR = {BASEADDR, 27'h0};
   wb_check_local_access localcheck1(.addr(wbm1_adr_i),.islocal(wbm1_islocal));
   wb_check_local_access localcheck2(.addr(wbm2_adr_i),.islocal(wbm2_islocal));
   wb_check_local_access localcheck3(.addr(wbm3_adr_i),.islocal(wbm3_islocal));
   wb_check_local_access localcheck4(.addr(wbm4_adr_i),.islocal(wbm4_islocal));


   wire [3:0] 	     s0_sels,s1_sels,s2_sels,s3_sels,s4_sels;
   wire [1:0] 	     s0_num, s1_num, s2_num, s3_num, s4_num;

   wb_dualselgen selgen0(.addr_i(wbm0_adr_i),.selgen_o(s0_sels),.selnum_o(s0_num));
   wb_dualselgen selgen1(.addr_i(wbm1_adr_i),.selgen_o(s1_sels),.selnum_o(s1_num));
   wb_dualselgen selgen2(.addr_i(wbm2_adr_i),.selgen_o(s2_sels),.selnum_o(s2_num));
   wb_dualselgen selgen3(.addr_i(wbm3_adr_i),.selgen_o(s3_sels),.selnum_o(s3_num));
   wb_dualselgen selgen4(.addr_i(wbm4_adr_i),.selgen_o(s4_sels),.selnum_o(s4_num));


   // Note that I do not think that this arbiter is completely fair at all times.
   
   always @(posedge clk_i) begin
      if(rst_i) begin
	 grantstate <= GRANT1;
      end else begin
	 case(grantstate)
	   GRANT1: 
	     if (cyc1_i && !wbm1_islocal) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc1_i) begin
		grantstate <= GRANT1;
		sels <= { s1_sels, 1'b0};
		sel_num <= s1_num;
	     end else if (cyc2_i && !wbm2_islocal) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc2_i) begin
		grantstate <= GRANT2;
		sels <= { s2_sels, 1'b0};
		sel_num <= s2_num;
	     end else if (cyc3_i && !wbm3_islocal) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc3_i) begin
		grantstate <= GRANT3;
		sels <= { s3_sels, 1'b0};
		sel_num <= s3_num;
	     end else if (cyc4_i && !wbm4_islocal) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc4_i) begin
		grantstate <= GRANT4;
		sels <= { s4_sels, 1'b0};
		sel_num <= s4_num;
	     end else if (cyc0_i) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b0};
	     end else begin
		sels <= 0; // Noone wants the bus at the moment
	     end
	   
	   GRANT2:
	     if (cyc2_i && !wbm2_islocal) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc2_i) begin
		grantstate <= GRANT2;
		sels <= { s2_sels, 1'b0};
		sel_num <= s2_num;
	     end else if (cyc3_i && !wbm3_islocal) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc3_i) begin
		grantstate <= GRANT3;
		sels <= { s3_sels, 1'b0};
		sel_num <= s3_num;
	     end else if (cyc4_i && !wbm4_islocal) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc4_i) begin
		grantstate <= GRANT4;
		sels <= { s4_sels, 1'b0};
		sel_num <= s4_num;
	     end else if (cyc1_i && !wbm1_islocal) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc1_i) begin
		grantstate <= GRANT1;
		sels <= { s1_sels, 1'b0};
		sel_num <= s1_num;
	     end else if (cyc0_i) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b0};
	     end else begin
		sels <= 0; // Noone wants the bus at the moment
	     end


	   GRANT3:
	     if (cyc3_i && !wbm3_islocal) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc3_i) begin
		grantstate <= GRANT3;
		sels <= { s3_sels, 1'b0};
		sel_num <= s3_num;
	     end else if (cyc4_i && !wbm4_islocal) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc4_i) begin
		grantstate <= GRANT4;
		sels <= { s4_sels, 1'b0};
		sel_num <= s4_num;
	     end else if (cyc1_i && !wbm1_islocal) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc1_i) begin
		grantstate <= GRANT1;
		sels <= { s1_sels, 1'b0};
		sel_num <= s1_num;
	     end else if (cyc2_i && !wbm2_islocal) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc2_i) begin
		grantstate <= GRANT2;
		sels <= { s2_sels, 1'b0};
		sel_num <= s2_num;
	     end else if (cyc0_i) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b0};
	     end else begin
		sels <= 0; // Noone wants the bus at the moment
	     end


	   GRANT4:
	     if (cyc4_i && !wbm4_islocal) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc4_i) begin
		grantstate <= GRANT4;
		sels <= { s4_sels, 1'b0};
		sel_num <= s4_num;
	     end else if (cyc1_i && !wbm1_islocal) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc1_i) begin
		grantstate <= GRANT1;
		sels <= { s1_sels, 1'b0};
		sel_num <= s1_num;
	     end else if (cyc2_i && !wbm2_islocal) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc2_i) begin
		grantstate <= GRANT2;
		sels <= { s2_sels, 1'b0};
		sel_num <= s2_num;
	     end else if (cyc3_i && !wbm3_islocal) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc3_i) begin
		grantstate <= GRANT3;
		sels <= { s3_sels, 1'b0};
		sel_num <= s3_num;
	     end else if (cyc0_i) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b0};
	     end else begin
		sels <= 0; // Noone wants the bus at the moment
	     end


	   GRANT0AND1:  begin
	     sels <= { s0_sels, 1'b0};
	     if (cyc1_i && !wbm1_islocal) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc0_i) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b0};
		sel_num <= s0_num;
	     end else if (cyc2_i && !wbm2_islocal) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc2_i) begin
		grantstate <= GRANT2;
		sels <= { s2_sels, 1'b0};
		sel_num <= s2_num;
	     end else if (cyc3_i && !wbm3_islocal) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc3_i) begin
		grantstate <= GRANT3;
		sels <= { s3_sels, 1'b0};
		sel_num <= s3_num;
	     end else if (cyc4_i && !wbm4_islocal) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc4_i) begin
		grantstate <= GRANT4;
		sels <= { s4_sels, 1'b0};
		sel_num <= s4_num;
	     end else if (cyc1_i) begin
		grantstate <= GRANT1;
		sels <= { s1_sels, 1'b0};
	     end else begin
		sels <= 0; // Noone wants the bus at the moment
	     end
	   end // case: GRANT0AND1
	   
	   
	   GRANT0AND2: begin
	     sels <= { s0_sels, 1'b0};
	     if (cyc2_i && !wbm2_islocal) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc0_i) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b0};
		sel_num <= s0_num;
	     end else if (cyc3_i && !wbm3_islocal) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc3_i) begin
		grantstate <= GRANT3;
		sels <= { s3_sels, 1'b0};
		sel_num <= s3_num;
	     end else if (cyc4_i && !wbm4_islocal) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc4_i) begin
		grantstate <= GRANT4;
		sels <= { s4_sels, 1'b0};
		sel_num <= s4_num;
	     end else if (cyc1_i && !wbm1_islocal) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc1_i) begin
		grantstate <= GRANT1;
		sels <= { s1_sels, 1'b0};
		sel_num <= s1_num;
	     end else if (cyc2_i) begin
		grantstate <= GRANT2;
		sels <= { s2_sels, 1'b0};
	     end else begin
		sels <= 0; // Noone wants the bus at the moment
	     end
	   end // case: GRANT0AND2

	   GRANT0AND3: begin
	     sels <= { s0_sels, 1'b0};
	     if (cyc3_i && !wbm3_islocal) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc0_i) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b0};
		sel_num <= s0_num;
	     end else if (cyc4_i && !wbm4_islocal) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc4_i) begin
		grantstate <= GRANT4;
		sels <= { s4_sels, 1'b0};
		sel_num <= s4_num;
	     end else if (cyc1_i && !wbm1_islocal) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc1_i) begin
		grantstate <= GRANT1;
		sels <= { s1_sels, 1'b0};
		sel_num <= s1_num;
	     end else if (cyc2_i && !wbm2_islocal) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc2_i) begin
		grantstate <= GRANT2;
		sels <= { s2_sels, 1'b0};
		sel_num <= s2_num;
	     end else if (cyc3_i) begin
		grantstate <= GRANT3;
		sels <= { s3_sels, 1'b0};
	     end else begin
		sels <= 0; // Noone wants the bus at the moment
	     end
	   end // case: GRANT0AND3

	   GRANT0AND4: begin
	     sels <= { s0_sels, 1'b0};
	     if (cyc4_i && !wbm4_islocal) begin
		grantstate <= GRANT0AND4;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc0_i) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b0};
		sel_num <= s0_num;
	     end else if (cyc1_i && !wbm1_islocal) begin
		grantstate <= GRANT0AND1;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc1_i) begin
		grantstate <= GRANT1;
		sels <= { s1_sels, 1'b0};
		sel_num <= s1_num;
	     end else if (cyc2_i && !wbm2_islocal) begin
		grantstate <= GRANT0AND2;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc2_i) begin
		grantstate <= GRANT2;
		sels <= { s2_sels, 1'b0};
		sel_num <= s2_num;
	     end else if (cyc3_i && !wbm3_islocal) begin
		grantstate <= GRANT0AND3;
		sels <= { s0_sels, 1'b1};
		sel_num <= s0_num;
	     end else if (cyc3_i) begin
		grantstate <= GRANT3;
		sels <= { s3_sels, 1'b0};
		sel_num <= s3_num;
	     end else if (cyc4_i) begin
		grantstate <= GRANT4;
		sels <= { s4_sels, 1'b0};
	     end else begin
		sels <= 0; // Noone wants the bus at the moment
	     end
	   end // case: GRANT0AND4
	 endcase // case(grantstate)
      end
   end

   always @(*) begin
      case(grantstate)
	GRANT1: gnt_num = 2'd0;
	GRANT2: gnt_num = 2'd1;
	GRANT3: gnt_num = 2'd2;
	GRANT4: gnt_num = 2'd3;
	GRANT0AND1: gnt_num = 2'd0;
	GRANT0AND2: gnt_num = 2'd1;
	GRANT0AND3: gnt_num = 2'd2;
	GRANT0AND4: gnt_num = 2'd3;
      endcase // case(grantstate)
   end

   assign 	gnt_num_o = grantstate;

   assign 	gnt0_o = (grantstate == GRANT0AND1) || (grantstate == GRANT0AND2) || (grantstate == GRANT0AND3) || (grantstate == GRANT0AND4);
   assign 	gnt1_o = (grantstate == GRANT1) || (grantstate == GRANT0AND1);
   assign 	gnt2_o = (grantstate == GRANT2) || (grantstate == GRANT0AND2);
   assign 	gnt3_o = (grantstate == GRANT3) || (grantstate == GRANT0AND3);
   assign 	gnt4_o = (grantstate == GRANT4) || (grantstate == GRANT0AND4);
   
endmodule // wb_dualarbiter
