`default_nettype none
`timescale 1ns / 10ps
  // This is a Wishbone bus that can handle two masters simultaneously under
  // certain conditions:
  // If wbm0 wants to talk to wbs1-4 while at the same time
  // one of wbm1-4 wants to talk to wbs0, they can do so at the same time.
  // At any other time, only one master may communicate on the bus.

module wb_dualbus(
	      input wire clk_i,
	      input wire rst_i,

	      
    // Wishbone slave port 0
    input wire           wbs0_stb_i,
    input wire 		 wbs0_cyc_i,
    input wire 		 wbs0_cab_i,
    input wire 		 wbs0_we_i,
    input wire [31:0] 	 wbs0_adr_i,
    input wire [31:0] 	 wbs0_dat_i,
    input wire [3:0] 	 wbs0_sel_i,
    input wire [4:0]     wbs0_size_i,		 
    output wire [31:0] 	 wbs0_dat_o,
    output wire  	 wbs0_ack_o,
    output wire  	 wbs0_rty_o,
    output wire  	 wbs0_err_o,

    // Wishbone slave port 1
    input wire           wbs1_stb_i,
    input wire 		 wbs1_cyc_i,
    input wire 		 wbs1_cab_i,
    input wire 		 wbs1_we_i,
    input wire [31:0] 	 wbs1_adr_i,
    input wire [31:0] 	 wbs1_dat_i,
    input wire [3:0] 	 wbs1_sel_i,
    input wire [4:0]     wbs1_size_i,
    output wire [31:0] 	 wbs1_dat_o,
    output wire  	 wbs1_ack_o,
    output wire  	 wbs1_rty_o,
    output wire  	 wbs1_err_o,

    // Wishbone slave port 2
    input wire           wbs2_stb_i,
    input wire 		 wbs2_cyc_i,
    input wire 		 wbs2_cab_i,
    input wire 		 wbs2_we_i,
    input wire [31:0] 	 wbs2_adr_i,
    input wire [31:0] 	 wbs2_dat_i,
    input wire [3:0] 	 wbs2_sel_i,
    input wire [4:0]     wbs2_size_i,
    output wire [31:0] 	 wbs2_dat_o,
    output wire  	 wbs2_ack_o,
    output wire  	 wbs2_rty_o,
    output wire  	 wbs2_err_o,

    // Wishbone slave port 3
    input wire           wbs3_stb_i,
    input wire 		 wbs3_cyc_i,
    input wire 		 wbs3_cab_i,
    input wire 		 wbs3_we_i,
    input wire [31:0] 	 wbs3_adr_i,
    input wire [31:0] 	 wbs3_dat_i,
    input wire [3:0] 	 wbs3_sel_i,
    input wire [4:0]     wbs3_size_i,
    output wire [31:0] 	 wbs3_dat_o,
    output wire  	 wbs3_ack_o,
    output wire  	 wbs3_rty_o,
    output wire  	 wbs3_err_o,

    // Wishbone slave port 4
    input wire           wbs4_stb_i,
    input wire 		 wbs4_cyc_i,
    input wire 		 wbs4_cab_i,
    input wire 		 wbs4_we_i,
    input wire [31:0] 	 wbs4_adr_i,
    input wire [31:0] 	 wbs4_dat_i,
    input wire [3:0] 	 wbs4_sel_i,
    input wire [4:0]     wbs4_size_i,
    output wire [31:0] 	 wbs4_dat_o,
    output wire  	 wbs4_ack_o,
    output wire  	 wbs4_rty_o,
    output wire  	 wbs4_err_o,
		  

    // Wishbone master  port 0
    output wire 	 wbm0_stb_o,
    output wire 	 wbm0_cyc_o,
    output wire 	 wbm0_cab_o,
    output wire 	 wbm0_we_o,
    output wire [31:0] 	 wbm0_adr_o,
    output wire [31:0] 	 wbm0_dat_o,
    output wire [3:0] 	 wbm0_sel_o,
    output wire [4:0]    wbm0_size_o,
    input wire [31:0] 	 wbm0_dat_i,
    input wire  	 wbm0_ack_i,
    input wire  	 wbm0_rty_i,
    input wire  	 wbm0_err_i,

    // Wishbone master  port 1
    output wire 	 wbm1_stb_o,
    output wire 	 wbm1_cyc_o,
    output wire 	 wbm1_cab_o,
    output wire 	 wbm1_we_o,
    output wire [31:0] 	 wbm1_adr_o,
    output wire [31:0] 	 wbm1_dat_o,
    output wire [3:0] 	 wbm1_sel_o,
    output wire [4:0]    wbm1_size_o,
    input wire [31:0] 	 wbm1_dat_i,
    input wire  	 wbm1_ack_i,
    input wire  	 wbm1_rty_i,
    input wire  	 wbm1_err_i,
	      
    // Wishbone master  port 2
    output wire 	 wbm2_stb_o,
    output wire 	 wbm2_cyc_o,
    output wire 	 wbm2_cab_o,
    output wire 	 wbm2_we_o,
    output wire [31:0] 	 wbm2_adr_o,
    output wire [31:0] 	 wbm2_dat_o,
    output wire [3:0] 	 wbm2_sel_o,
    output wire [4:0]    wbm2_size_o,
    input wire [31:0] 	 wbm2_dat_i,
    input wire  	 wbm2_ack_i,
    input wire  	 wbm2_rty_i,
    input wire  	 wbm2_err_i,

    // Wishbone master  port 3
    output wire 	 wbm3_stb_o,
    output wire 	 wbm3_cyc_o,
    output wire 	 wbm3_cab_o,
    output wire 	 wbm3_we_o,
    output wire [31:0] 	 wbm3_adr_o,
    output wire [31:0] 	 wbm3_dat_o,
    output wire [3:0] 	 wbm3_sel_o,
    output wire [4:0]    wbm3_size_o,
    input wire [31:0] 	 wbm3_dat_i,
    input wire  	 wbm3_ack_i,
    input wire  	 wbm3_rty_i,
    input wire  	 wbm3_err_i,

    // Wishbone master  port 4
    output wire 	 wbm4_stb_o,
    output wire 	 wbm4_cyc_o,
    output wire 	 wbm4_cab_o,
    output wire 	 wbm4_we_o,
    output wire [31:0] 	 wbm4_adr_o,
    output wire [31:0] 	 wbm4_dat_o,
    output wire [3:0] 	 wbm4_sel_o,
    output wire [4:0]    wbm4_size_o,
    input wire [31:0] 	 wbm4_dat_i,
    input wire  	 wbm4_ack_i,
    input wire  	 wbm4_rty_i,
    input wire  	 wbm4_err_i
	      );


   wire 		 gnt0,gnt1,gnt2,gnt3, gnt4;
   wire [1:0] 		 gnt_num;


   parameter [4:0] 	 BASEADDR = 5'h0;

   defparam 		 arb.BASEADDR = BASEADDR;
   
   
   wire 		 sel0,sel1,sel2,sel3, sel4;
   wire [1:0] 		 slave_num;
   
   wb_dualarbiter arb(.clk_i(clk_i),
		      .rst_i(rst_i),
		      .wbm0_adr_i(wbs0_adr_i),
		      .wbm1_adr_i(wbs1_adr_i),
		      .wbm2_adr_i(wbs2_adr_i),
		      .wbm3_adr_i(wbs3_adr_i),
		      .wbm4_adr_i(wbs4_adr_i),
		      .cyc0_i(wbs0_cyc_i),
		      .cyc1_i(wbs1_cyc_i),
		      .cyc2_i(wbs2_cyc_i),
		      .cyc3_i(wbs3_cyc_i),
		      .cyc4_i(wbs4_cyc_i),
		      .gnt_num_o(gnt_num),
		      .gnt0_o(gnt0),
		      .gnt1_o(gnt1),
		      .gnt2_o(gnt2),
		      .gnt3_o(gnt3),
		      .gnt4_o(gnt4),
		      .sels({sel4,sel3,sel2,sel1,sel0}),
		      .sel_num(slave_num));

   

   wire 		 m_stb;
   wire 		 m_cyc;
   wire 		 m_cab;
   wire 		 m_we;
   wire [31:0] 		 m_adr;
   wire [31:0] 		 m_dat;
   wire [3:0] 		 m_sel;
   wire [4:0] 		 m_size;

   wire 		 nm_stb;
   wire 		 nm_cyc;
   wire 		 nm_cab;
   wire 		 nm_we;
   wire [31:0] 		 nm_adr;
   wire [31:0] 		 nm_dat;
   wire [3:0] 		 nm_sel;
   wire [4:0] 		 nm_size;
   
   wire 		 s_ack;
   wire 		 s_rty;
   wire 		 s_err;
   wire [31:0] 		 s_dat;

   wb_dualmastermux mmux(.sel_i(gnt_num),
			 .nocsel_i(gnt0),
		     .master0({wbs0_stb_i,wbs0_cyc_i,wbs0_cab_i,wbs0_we_i,wbs0_adr_i,wbs0_dat_i,wbs0_sel_i,wbs0_size_i}),
		     .master1({wbs1_stb_i,wbs1_cyc_i,wbs1_cab_i,wbs1_we_i,wbs1_adr_i,wbs1_dat_i,wbs1_sel_i,wbs1_size_i}),
		     .master2({wbs2_stb_i,wbs2_cyc_i,wbs2_cab_i,wbs2_we_i,wbs2_adr_i,wbs2_dat_i,wbs2_sel_i,wbs2_size_i}),
		     .master3({wbs3_stb_i,wbs3_cyc_i,wbs3_cab_i,wbs3_we_i,wbs3_adr_i,wbs3_dat_i,wbs3_sel_i,wbs3_size_i}),
		     .master4({wbs4_stb_i,wbs4_cyc_i,wbs4_cab_i,wbs4_we_i,wbs4_adr_i,wbs4_dat_i,wbs4_sel_i,wbs4_size_i}),
		     .nocmux_o({nm_stb,nm_cyc,nm_cab,nm_we,nm_adr,nm_dat,nm_sel,nm_size}),
		     .mux_o({m_stb,m_cyc,m_cab,m_we,m_adr,m_dat,m_sel,m_size}));


//   defparam 		 addrdec.baseaddr = BASEADDR;


   


   
   
   
//    wb_dualslavesel addrdec(.addr_i(m_adr),
// 			   .nocaddr_i(nm_adr),
// 			   .sel_o(slave_num),
// 			   .sel0_o(sel0),
// 			   .sel1_o(sel1),
// 			   .sel2_o(sel2),
// 			   .sel3_o(sel3),
// 			   .sel4_o(sel4),
// 			   .ack(),
// 			   .nomatch_o(),
// 			   .clk_i(clk_i),
// 			   .rst_i(rst_i),
// 			   .cyc(m_cyc)
// 			   );


   wire 		 wbs0_ack, wbs0_rty, wbs0_err;
   

   wb_dualslavemux smux(
			.sel_i(slave_num),
			.nocsel_i(gnt0),
			.slave0({wbm0_ack_i,wbm0_rty_i,wbm0_err_i,wbm0_dat_i}),
			.slave1({wbm1_ack_i,wbm1_rty_i,wbm1_err_i,wbm1_dat_i}),
			.slave2({wbm2_ack_i,wbm2_rty_i,wbm2_err_i,wbm2_dat_i}),
			.slave3({wbm3_ack_i,wbm3_rty_i,wbm3_err_i,wbm3_dat_i}),
			.slave4({wbm4_ack_i,wbm4_rty_i,wbm4_err_i,wbm4_dat_i}),
			.mux_o({s_ack,s_rty,s_err,s_dat}),
			.nocmux_o({wbs0_ack, wbs0_rty, wbs0_err, wbs0_dat_o}));



				    


   assign 		 wbm0_dat_o = nm_dat;
   assign 		 wbm0_adr_o = nm_adr;
   assign 		 wbm0_we_o  = nm_we;
   assign 		 wbm0_cab_o = nm_cab;
   assign 		 wbm0_sel_o = nm_sel;
   assign 		 wbm0_size_o = nm_size;

   assign 		 wbm1_dat_o = m_dat;
   assign 		 wbm1_adr_o = m_adr;
   assign 		 wbm1_we_o  = m_we;
   assign 		 wbm1_cab_o = m_cab;
   assign 		 wbm1_sel_o = m_sel;
   assign 		 wbm1_size_o = m_size;

   assign 		 wbm2_dat_o = m_dat;
   assign 		 wbm2_adr_o = m_adr;
   assign 		 wbm2_we_o  = m_we;
   assign 		 wbm2_cab_o = m_cab;
   assign 		 wbm2_sel_o = m_sel;
   assign 		 wbm2_size_o = m_size;

   assign 		 wbm3_dat_o = m_dat;
   assign 		 wbm3_adr_o = m_adr;
   assign 		 wbm3_we_o  = m_we;
   assign 		 wbm3_cab_o = m_cab;
   assign 		 wbm3_sel_o = m_sel;
   assign 		 wbm3_size_o = m_size;

   assign 		 wbm0_cyc_o = nm_cyc && sel0;
   assign 		 wbm0_stb_o = nm_stb && sel0;

   assign 		 wbm1_cyc_o = m_cyc && sel1;
   assign 		 wbm1_stb_o = m_stb && sel1;

   assign 		 wbm2_cyc_o = m_cyc && sel2;
   assign 		 wbm2_stb_o = m_stb && sel2;

   assign 		 wbm3_cyc_o = m_cyc && sel3;
   assign 		 wbm3_stb_o = m_stb && sel3;

   assign 		 wbm4_cyc_o = m_cyc && sel4;
   assign 		 wbm4_stb_o = m_stb && sel4;


   assign 		 wbs0_ack_o = wbs0_ack && gnt0;
   assign 		 wbs0_rty_o = wbs0_rty && gnt0;
   assign 		 wbs0_err_o = wbs0_err && gnt0;
   


   assign 		 wbs1_dat_o = s_dat;
   assign 		 wbs1_ack_o = s_ack && gnt1;
   assign 		 wbs1_rty_o = s_rty && gnt1;
   assign 		 wbs1_err_o = s_err && gnt1;

   assign 		 wbs2_dat_o = s_dat;
   assign 		 wbs2_ack_o = s_ack && gnt2;
   assign 		 wbs2_rty_o = s_rty && gnt2;
   assign 		 wbs2_err_o = s_err && gnt2;

   assign 		 wbs3_dat_o = s_dat;
   assign 		 wbs3_ack_o = s_ack && gnt3;
   assign 		 wbs3_rty_o = s_rty && gnt3;
   assign 		 wbs3_err_o = s_err && gnt3;
   
   assign 		 wbs4_dat_o = s_dat;
   assign 		 wbs4_ack_o = s_ack && gnt4;
   assign 		 wbs4_rty_o = s_rty && gnt4;
   assign 		 wbs4_err_o = s_err && gnt4;


   
   wb_monitor mon0(.clk_i		(clk_i),
		   .rst_i		(rst_i),	
		   .gnt_i               (gnt0),
		   .ack_i		(wbs0_ack_o),
		   .rty_i		(wbs0_rty_o),
		   .err_i		(wbs0_err_o),
		   .stb_i		(wbs0_stb_i),
		   .cyc_i		(wbs0_cyc_i));

   wb_monitor mon1(.clk_i		(clk_i),
		   .rst_i		(rst_i),
		   .gnt_i               (gnt1),
		   .ack_i		(wbs1_ack_o),
		   .rty_i		(wbs1_rty_o),
		   .err_i		(wbs1_err_o),
		   .stb_i		(wbs1_stb_i),
		   .cyc_i		(wbs1_cyc_i));

   wb_monitor mon2(.clk_i		(clk_i),
		   .rst_i		(rst_i),
		   .gnt_i               (gnt2),
		   .ack_i		(wbs2_ack_o),
		   .rty_i		(wbs2_rty_o),
		   .err_i		(wbs2_err_o),
		   .stb_i		(wbs2_stb_i),
		   .cyc_i		(wbs2_cyc_i));

   wb_monitor mon3(.clk_i		(clk_i),
		   .rst_i		(rst_i),
		   .gnt_i               (gnt3),
		   .ack_i		(wbs3_ack_o),
		   .rty_i		(wbs3_rty_o),
		   .err_i		(wbs3_err_o),
		   .stb_i		(wbs3_stb_i),
		   .cyc_i		(wbs3_cyc_i));

   wb_monitor mon4(.clk_i		(clk_i),
		   .rst_i		(rst_i),
		   .gnt_i               (gnt4),
		   .ack_i		(wbs4_ack_o),
		   .rty_i		(wbs4_rty_o),
		   .err_i		(wbs4_err_o),
		   .stb_i		(wbs4_stb_i),
		   .cyc_i		(wbs4_cyc_i));
   
   
   
endmodule // wb_dualbus

