`timescale 1ns / 10ps
// Simple round robin arbiter.
// If two requests arrive simultaneously to an idle switch,
// Asel has priority over Bsel which has priority over Csel

module arbiter(input wire clk,
    input wire rst,

    input wire Asel,
    input wire Bsel,
    input wire Csel,
    input wire Dsel,
    input wire oktosend,

    input wire Alast,
    input wire Blast,
    input wire Clast,
    input wire Dlast,

    output reg override,
    output reg Aoverride,
    output reg Boverride,
    output reg Coverride,
    output reg Doverride

	       );

   reg 	       Adone,Bdone,Cdone,Ddone;

   wire        onlyA, onlyB, onlyC,onlyD;
   assign      onlyA =  Asel && !Bsel && !Csel && !Dsel;
   assign      onlyB = !Asel &&  Bsel && !Csel && !Dsel;
   assign      onlyC = !Asel && !Bsel &&  Csel && !Dsel;
   assign      onlyD = !Asel && !Bsel && !Csel &&  Dsel;



   always @(posedge clk) begin
      
      if(Asel && (!Adone || onlyA) && !override) begin
	 Aoverride <= 1;
      end

      if(Bsel && (!Bdone || onlyB) && (!Asel || Adone) && !override) begin
	 Boverride <= 1;
      end

      if(Csel && (!Cdone || onlyC) && (!Asel || Adone) && (!Bsel || Bdone) && !override) begin
	 Coverride <= 1;
      end
      
      if(Dsel && (!Ddone || onlyD) && (!Csel || Cdone) && (!Asel || Adone) && (!Bsel || Bdone) && !override) begin
	 Doverride <= 1;
      end
      
      
      if((Aoverride || onlyA) && Alast && oktosend) begin
	 Adone <= 1;
	 Aoverride <= 0;
      end

      if((Boverride || onlyB) && Blast && oktosend) begin
	 Bdone <= 1;
	 Boverride <= 0;
      end

      if((Coverride || onlyC) && Clast && oktosend) begin
	 Cdone <= 1;
	 Coverride <= 0;
      end

      if((Doverride || onlyD) && Dlast && oktosend) begin
	 Ddone <= 1;
	 Doverride <= 0;
      end
      
      if(!(~{Bdone,Cdone,Ddone} & {Bsel,Csel,Dsel})) begin
	 Adone <= 0;
      end

      if(!(~{Adone,Cdone,Ddone} & {Asel,Csel,Dsel})) begin
	 Bdone <= 0;
      end

      if(!(~{Adone,Bdone,Ddone} & {Asel,Bsel,Ddone})) begin
	 Cdone <= 0;
      end

      if(!(~{Adone,Bdone,Cdone} & {Asel,Bsel,Cdone})) begin
	 Ddone <= 0;
      end
      

      if(rst) begin
	 Adone <= 0;
	 Bdone <= 0;
	 Cdone <= 0;
	 Ddone <= 0;
	 
	 Aoverride <= 0;
	 Boverride <= 0;
	 Coverride <= 0;
	 Doverride <= 0;
      end
      
   end // always @ (posedge clk)


   always @(*) begin
      override = Aoverride || Boverride || Coverride || Doverride;
   end


   // For sanity checking
   reg [2:0] count;
   always @(posedge clk) begin
      count = 0;
      if(Aoverride) count = count + 1;
      if(Boverride) count = count + 1;
      if(Coverride) count = count + 1;
      if(Doverride) count = count + 1;
      if(count > 1) begin
	 $display("%m: Fatal error, more than one override active at the same time!");
	 $stop;
      end
   end
   

endmodule // arbiter
