`timescale 1ns / 10ps
`include "noc_pkt.vh"

// The output mux for a local connection
// This is a bit smaller than the regular outputmux since
// there is no sense in including some of the NoC control signals
// (like next route) here

module localoutputmux(input wire clk,
    input wire rst,
    input wire oktosend,
    output reg send,

    input wire [`DATAWIDTH+`SENDBIT-1:0] A,B,C,D,
    output wire [`DATAWIDTH+`SENDBIT-1:0] Q,

    input wire 				  Asel,
    input wire 				  Bsel,
    input wire 				  Csel,
    input wire 				  Dsel,

    input wire 				  Aoverride,
    input wire 				  Boverride,
    input wire 				  Coverride,
    input wire 				  Doverride,
    input wire 				  override);

   wire 				  muxsel;
   assign 				  muxsel = ((Dsel || Csel) && !Boverride && !Aoverride);
   

   wire 				  sendit1;
   wire 				  sendit2;

   assign 				  sendit1 = (Asel && Aoverride) || (Bsel && Boverride) || (Dsel && Doverride);
   assign 				  sendit2 = (Csel && Coverride) || (!override && (( Asel && !Bsel && !Csel && !Dsel) || 
											  (!Asel &&  Bsel && !Csel && !Dsel) || 
											  (!Asel && !Bsel &&  Csel && !Dsel) ||
											  (!Asel && !Bsel && !Csel &&  Dsel) ));
   
   always @(posedge clk) begin
      send <= 0;
      if(oktosend) begin
	 send <= sendit1 || sendit2;
      end
      // Might want to add a reset signal to "send"
   end

   genvar i;
   
   generate
      for(i=0; i < `SENDBIT+`DATAWIDTH; i = i + 1) begin:MUX
	 muxbit mux(.clk(clk),.rst(rst),.a(A[i]),.b(B[i]),.c(C[i]),.d(D[i]),.Asel(Asel),.Csel(Csel),.Boverride(Boverride),.Doverride(Doverride),.muxsel(muxsel),.q(Q[i]));
      end
   endgenerate
   
endmodule // outputmux
