`timescale 1ns / 10ps
`default_nettype none
`include "noc_pkt.vh"

  
module node4(input wire  clk,
	    input wire  rst,
	    
	    input wire  [`DATAWIDTH+`CTRLWIDTH-1:0] west_i,
	    output wire [`DATAWIDTH+`CTRLWIDTH-1:0] west_o,
	    
	    input wire  [`DATAWIDTH+`CTRLWIDTH-1:0] east_i,
	    output wire [`DATAWIDTH+`CTRLWIDTH-1:0] east_o,
	    
	    input wire  [`DATAWIDTH+`CTRLWIDTH-1:0] north_i,
	    output wire [`DATAWIDTH+`CTRLWIDTH-1:0] north_o,
	    
	    input wire  [`DATAWIDTH+`CTRLWIDTH-1:0] south_i,
	    output wire [`DATAWIDTH+`CTRLWIDTH-1:0] south_o


);

   wire [`DATAWIDTH - 1:0]     west_data_i  = west_i[ `DATAWIDTH - 1:0];
   wire [`CTRLWIDTH - 1:0]     west_ctrl_i  = west_i[ `DATAWIDTH +`CTRLWIDTH - 1:`DATAWIDTH];
   wire [`DATAWIDTH - 1:0]     east_data_i  = east_i[ `DATAWIDTH - 1:0];
   wire [`CTRLWIDTH - 1:0]     east_ctrl_i  = east_i[ `DATAWIDTH +`CTRLWIDTH - 1:`DATAWIDTH];
   wire [`DATAWIDTH - 1:0]     north_data_i = north_i[`DATAWIDTH - 1:0];
   wire [`CTRLWIDTH - 1:0]     north_ctrl_i = north_i[`DATAWIDTH +`CTRLWIDTH - 1:`DATAWIDTH];
   wire [`DATAWIDTH - 1:0]     south_data_i = south_i[`DATAWIDTH - 1:0];
   wire [`CTRLWIDTH - 1:0]     south_ctrl_i = south_i[`DATAWIDTH +`CTRLWIDTH - 1:`DATAWIDTH];
   
   wire 	  west_stb;
   wire 	  west_avail;
   wire [`DATAWIDTH - 1:0] west_fifodata;
   wire [`CTRLWIDTH - 1-1:0] west_fifoctrl;
   

   wire 	  east_stb;
   wire 	  east_avail;
   wire [`DATAWIDTH - 1:0] east_fifodata;
   wire [`CTRLWIDTH - 1-1:0] east_fifoctrl;

   wire 	  north_stb;
   wire 	  north_avail;
   wire [`DATAWIDTH - 1:0] north_fifodata;
   wire [`CTRLWIDTH - 1-1:0] north_fifoctrl;

   wire 	  south_stb;
   wire 	  south_avail;
   wire [`DATAWIDTH - 1:0] south_fifodata;
   wire [`CTRLWIDTH - 1-1:0] south_fifoctrl;

   

   wire 	  east_to_west;
   wire 	  east_to_north;
   wire 	  east_to_south;
   wire 	  north_to_west;
   wire 	  north_to_east;
   wire 	  north_to_south;
   wire 	  south_to_west;
   wire 	  south_to_east;
   wire 	  south_to_north;
   wire 	  west_to_east;
   wire 	  west_to_north;
   wire 	  west_to_south;

   
   // Packet status signals

   wire 	  west_lastpart;
   wire 	  east_lastpart;
   wire 	  south_lastpart;
   wire 	  north_lastpart;

// Override signals...
   wire 	  east_to_west_override;
   wire 	  east_to_south_override;
   wire 	  east_to_north_override;
   wire 	  west_to_east_override;
   wire 	  west_to_south_override;
   wire 	  west_to_north_override;
   wire 	  south_to_west_override;
   wire 	  south_to_east_override;
   wire 	  south_to_north_override;

   wire 	  north_to_west_override;
   wire 	  north_to_east_override;
   wire 	  north_to_south_override;

   wire 	  east_override;
   wire 	  west_override;
   wire 	  south_override;
   wire 	  north_override;



// Read signals
   wire 	  west_rd;
   wire 	  east_rd;
   wire 	  south_rd;
   wire 	  north_rd;

   reg 		  west_oktosend;
   reg 		  east_oktosend;
   reg 		  north_oktosend;
   reg 		  south_oktosend;

   wire [3:0] west_to_east_nextsel;
   wire [3:0] west_to_north_nextsel;
   wire [3:0] west_to_south_nextsel;
   wire [3:0] east_to_west_nextsel;
   wire [3:0] east_to_north_nextsel;
   wire [3:0] east_to_south_nextsel;
   wire [3:0] south_to_east_nextsel;
   wire [3:0] south_to_north_nextsel;
   wire [3:0] south_to_west_nextsel;
   wire [3:0] north_to_east_nextsel;
   wire [3:0] north_to_west_nextsel;
   wire [3:0] north_to_south_nextsel;
   
//   wire 		  west_oktosend1;
   wire 		  east_oktosend1;
   wire 		  north_oktosend1;
   wire 		  south_oktosend1;
   wire 		  west_oktosend2;
   wire 		  north_oktosend2;
   wire 		  south_oktosend2;
   wire 		  west_oktosend3;
   wire 		  east_oktosend3;
//   wire 		  north_oktosend3;
   wire 		  south_oktosend3;
   wire 		  west_oktosend4;
   wire 		  east_oktosend4;
   wire 		  north_oktosend4;
//   wire 		  south_oktosend4;

   //////////////////////////////////////////////////////////////////////
   // Route parameters
   //////////////////////////////////////////////////////////////////////

   parameter 		  UNIFIED_ROUTELOOKUP = 1;

   defparam 		  westfifo.UNIFIED_ROUTELOOKUP = UNIFIED_ROUTELOOKUP;
   defparam 		  eastfifo.UNIFIED_ROUTELOOKUP = UNIFIED_ROUTELOOKUP;
   defparam 		  northfifo.UNIFIED_ROUTELOOKUP = UNIFIED_ROUTELOOKUP;
   defparam 		  southfifo.UNIFIED_ROUTELOOKUP = UNIFIED_ROUTELOOKUP;
   
   parameter [31:0] 	  AROUTE = 32'h00000800;
   parameter [31:0] 	  BROUTE = 32'h00040000;
   parameter [31:0] 	  CROUTE = 32'h03000000;
   parameter [31:0] 	  DROUTE = 32'h00100000;
   
	 parameter [31:0] WESTROUTE = 32'h00000002;
	 parameter [31:0] EASTROUTE = 32'h0000000c;
	 parameter [31:0] NORTHROUTE = 32'h00000030;
	 parameter [31:0] SOUTHROUTE = 32'hffffffc0;
	 
	 parameter [31:0] WEST_WESTROUTE = 32'h00000002;
	 parameter [31:0] WEST_NORTHROUTE = 32'h00000030;
	 parameter [31:0] WEST_SOUTHROUTE = 32'hffffffc0;
	 parameter [31:0] WEST_LOCALROUTE = 32'hffffffc0;
	 
	 parameter [31:0] EAST_EASTROUTE = 32'h0000000c;
	 parameter [31:0] EAST_NORTHROUTE = 32'h00000030;
	 parameter [31:0] EAST_SOUTHROUTE = 32'hffffffc0;
	 parameter [31:0] EAST_LOCALROUTE = 32'hffffffc0;
	 
	 parameter [31:0] SOUTH_WESTROUTE = 32'h00000002;
	 parameter [31:0] SOUTH_EASTROUTE = 32'h0000000c;
	 parameter [31:0] SOUTH_SOUTHROUTE = 32'hffffffc0;
	 parameter [31:0] SOUTH_LOCALROUTE = 32'hffffffc0;
	 
	 parameter [31:0] NORTH_WESTROUTE = 32'h00000002;
	 parameter [31:0] NORTH_EASTROUTE = 32'h0000000c;
	 parameter [31:0] NORTH_NORTHROUTE = 32'h00000030;
	 parameter [31:0] NORTH_LOCALROUTE = 32'h00000030;

	 
	 defparam 	  westfifo.A_AROUTE = EAST_NORTHROUTE;
	 defparam 	  westfifo.A_BROUTE = EAST_EASTROUTE;
	 defparam 	  westfifo.A_CROUTE = EAST_SOUTHROUTE;
	 defparam 	  westfifo.A_DROUTE = EAST_LOCALROUTE;

	 defparam 	  westfifo.B_AROUTE = NORTH_WESTROUTE;
	 defparam 	  westfifo.B_BROUTE = NORTH_NORTHROUTE;
	 defparam 	  westfifo.B_CROUTE = NORTH_EASTROUTE;
	 defparam 	  westfifo.B_DROUTE = NORTH_LOCALROUTE;

	 defparam 	  westfifo.C_AROUTE = SOUTH_WESTROUTE;
	 defparam 	  westfifo.C_BROUTE = SOUTH_EASTROUTE;
	 defparam 	  westfifo.C_CROUTE = SOUTH_SOUTHROUTE;
	 defparam 	  westfifo.C_DROUTE = SOUTH_LOCALROUTE;
	 
	 defparam 	  eastfifo.A_AROUTE = WEST_WESTROUTE;
	 defparam 	  eastfifo.A_BROUTE = WEST_NORTHROUTE;
	 defparam 	  eastfifo.A_CROUTE = WEST_SOUTHROUTE;
	 defparam 	  eastfifo.A_DROUTE = WEST_LOCALROUTE;

	 defparam 	  eastfifo.B_AROUTE = NORTH_WESTROUTE;
	 defparam 	  eastfifo.B_BROUTE = NORTH_NORTHROUTE;
	 defparam 	  eastfifo.B_CROUTE = NORTH_EASTROUTE;
	 defparam 	  eastfifo.B_DROUTE = NORTH_LOCALROUTE;

	 defparam 	  eastfifo.C_AROUTE = SOUTH_WESTROUTE;
	 defparam 	  eastfifo.C_BROUTE = SOUTH_EASTROUTE;
	 defparam 	  eastfifo.C_CROUTE = SOUTH_SOUTHROUTE;
	 defparam 	  eastfifo.C_DROUTE = SOUTH_LOCALROUTE;

	 defparam 	  northfifo.A_AROUTE = WEST_WESTROUTE;
	 defparam 	  northfifo.A_BROUTE = WEST_NORTHROUTE;
	 defparam 	  northfifo.A_CROUTE = WEST_SOUTHROUTE;
	 defparam 	  northfifo.A_DROUTE = WEST_LOCALROUTE;

	 defparam 	  northfifo.B_AROUTE = EAST_NORTHROUTE;
	 defparam 	  northfifo.B_BROUTE = EAST_EASTROUTE;
	 defparam 	  northfifo.B_CROUTE = EAST_SOUTHROUTE;
	 defparam 	  northfifo.B_DROUTE = EAST_LOCALROUTE;

	 defparam 	  northfifo.C_AROUTE = SOUTH_WESTROUTE;
	 defparam 	  northfifo.C_BROUTE = SOUTH_EASTROUTE;
	 defparam 	  northfifo.C_CROUTE = SOUTH_SOUTHROUTE;
	 defparam 	  northfifo.C_DROUTE = SOUTH_LOCALROUTE;

	 defparam 	  southfifo.A_AROUTE = WEST_WESTROUTE;
	 defparam 	  southfifo.A_BROUTE = WEST_NORTHROUTE;
	 defparam 	  southfifo.A_CROUTE = WEST_SOUTHROUTE;
	 defparam 	  southfifo.A_DROUTE = WEST_LOCALROUTE;

	 defparam 	  southfifo.B_AROUTE = EAST_NORTHROUTE;
	 defparam 	  southfifo.B_BROUTE = EAST_EASTROUTE;
	 defparam 	  southfifo.B_CROUTE = EAST_SOUTHROUTE;
	 defparam 	  southfifo.B_DROUTE = EAST_LOCALROUTE;

	 defparam 	  southfifo.C_AROUTE = NORTH_WESTROUTE;
	 defparam 	  southfifo.C_BROUTE = NORTH_NORTHROUTE;
	 defparam 	  southfifo.C_CROUTE = NORTH_EASTROUTE;
	 defparam 	  southfifo.C_DROUTE = NORTH_LOCALROUTE;




	 defparam 	  westfifo.AROUTE = AROUTE;
	 defparam 	  westfifo.BROUTE = BROUTE;
	 defparam 	  westfifo.CROUTE = CROUTE;
	 defparam 	  westfifo.DROUTE = DROUTE;

	 defparam 	  eastfifo.AROUTE = AROUTE;
	 defparam 	  eastfifo.BROUTE = BROUTE;
	 defparam 	  eastfifo.CROUTE = CROUTE;
	 defparam 	  eastfifo.DROUTE = DROUTE;

	 defparam 	  northfifo.AROUTE = AROUTE;
	 defparam 	  northfifo.BROUTE = BROUTE;
	 defparam 	  northfifo.CROUTE = CROUTE;
	 defparam 	  northfifo.DROUTE = DROUTE;

	 defparam 	  southfifo.AROUTE = AROUTE;
	 defparam 	  southfifo.BROUTE = BROUTE;
	 defparam 	  southfifo.CROUTE = CROUTE;
	 defparam 	  southfifo.DROUTE = DROUTE;

	 
	 
	 
   
	 

   

      FD okfdeast1 (.C(clk),.D(east_i[`DATAWIDTH+`SENDOKBIT]),.Q(east_oktosend1));
      FD okfdnorth1 (.C(clk),.D(north_i[`DATAWIDTH+`SENDOKBIT]),.Q(north_oktosend1));
      FD okfdsouth1 (.C(clk),.D(south_i[`DATAWIDTH+`SENDOKBIT]),.Q(south_oktosend1));
      FD okfdwest2 (.C(clk),.D(west_i[`DATAWIDTH+`SENDOKBIT]),.Q(west_oktosend2));
      FD okfdnorth2 (.C(clk),.D(north_i[`DATAWIDTH+`SENDOKBIT]),.Q(north_oktosend2));
      FD okfdsouth2 (.C(clk),.D(south_i[`DATAWIDTH+`SENDOKBIT]),.Q(south_oktosend2));
      FD okfdwest3 (.C(clk),.D(west_i[`DATAWIDTH+`SENDOKBIT]),.Q(west_oktosend3));
      FD okfdeast3 (.C(clk),.D(east_i[`DATAWIDTH+`SENDOKBIT]),.Q(east_oktosend3));
      FD okfdsouth3 (.C(clk),.D(south_i[`DATAWIDTH+`SENDOKBIT]),.Q(south_oktosend3));
      FD okfdwest4 (.C(clk),.D(west_i[`DATAWIDTH+`SENDOKBIT]),.Q(west_oktosend4));
      FD okfdeast4 (.C(clk),.D(east_i[`DATAWIDTH+`SENDOKBIT]),.Q(east_oktosend4)); 
      FD okfdnorth4 (.C(clk),.D(north_i[`DATAWIDTH+`SENDOKBIT]),.Q(north_oktosend4));
   
   
   

   always @(posedge clk) begin
      west_oktosend <=  west_i[`DATAWIDTH+`SENDOKBIT];
      east_oktosend <=  east_i[`DATAWIDTH+`SENDOKBIT];
      south_oktosend <= south_i[`DATAWIDTH+`SENDOKBIT];
      north_oktosend <= north_i[`DATAWIDTH+`SENDOKBIT];

   end

   

   //////////////////////////////////////////////////////////////////////
   // West part
   //////////////////////////////////////////////////////////////////////


   assign 	  west_stb = west_ctrl_i[`SENDBIT];
   
   


   // Clockwise assignment of routes...

   
   
   parameter 	  WESTLOWWATERMARK = 4'h9;
   defparam 	  westfifo.LOWWATERMARK = WESTLOWWATERMARK;

    srlfifo westfifo(.clk		(clk),
					.rst		(rst),
					.rd	(west_rd),

					.sendok	(west_o[`DATAWIDTH+`SENDOKBIT]),
					.avail		(west_avail),
					.q		(west_fifodata),
					.controlq	(west_fifoctrl),
					// Inputs
					.d		(west_data_i),
					.control        (west_ctrl_i),
					.we		(west_stb),
					.toA            (west_to_east_nextsel),
					.toB            (west_to_north_nextsel),
					.toC            (west_to_south_nextsel),
					
					.Asel(west_to_north),
					.Bsel(west_to_east),
					.Csel(west_to_south),
					.Dsel());

   assign 	  west_lastpart = west_fifoctrl[`LASTBIT];


    readenable westread(
		       // Outputs
		       .rd		(west_rd),
		       // Inputs
		       .nolocal_rd	(west_avail),
		       .A_Asel		(west_to_east),
		       .A_Bsel		(north_to_east),
		       .A_Csel		(south_to_east),
		       .A_Dsel		(1'b0),
		       .B_Asel		(west_to_south),
		       .B_Bsel		(east_to_south),
		       .B_Csel		(north_to_south),
		       .B_Dsel		(1'b0),
		       .C_Asel		(west_to_north),
		       .C_Bsel		(east_to_north),
		       .C_Csel		(south_to_north),
		       .C_Dsel		(1'b0),
		       .D_Asel (1'b0),
		       .D_Bsel (1'b0),
		       .D_Csel (1'b0),
		       .D_Dsel (1'b0),
		       .Aoktosend	(east_oktosend1),
		       .Boktosend	(south_oktosend1),
		       .Coktosend	(north_oktosend1),
		       .Doktosend	(1'b0),
		       .Aoverride	(west_to_east_override),
		       .Boverride	(west_to_south_override),
		       .Coverride	(west_to_north_override),
 		       .Doverride());
   
   
    outputmux west_mux(.clk(clk),
		      .rst(rst),
		      .oktosend(west_oktosend),
		      .send(west_o[`DATAWIDTH+`SENDBIT]),

		      .A({east_to_west_nextsel,  east_fifoctrl[`LASTBIT:0],east_fifodata}),
		      .B({south_to_west_nextsel, south_fifoctrl[`LASTBIT:0],south_fifodata}),
		      .C({north_to_west_nextsel, north_fifoctrl[`LASTBIT:0],north_fifodata}),
		      .D(0),
		      .Asel(east_to_west),
		      .Bsel(south_to_west),
		      .Csel(north_to_west),
					   .Dsel(1'b0),
		      .Aoverride(east_to_west_override),
		      .Boverride(south_to_west_override),
		      .Coverride(north_to_west_override),
					   .Doverride(1'b0),
		      .override(west_override),
		      .Q(west_o[`DATAWIDTH+`SENDBIT - 1:0])
		      );


    arbiter west_arbiter(
			.clk		(clk),
			.rst		(rst),

			.override	(west_override),
			.Aoverride	(east_to_west_override),
			.Boverride	(south_to_west_override),
			.Coverride	(north_to_west_override),
			.Doverride(),


			.Asel		(east_to_west),
			.Bsel		(south_to_west),
			.Csel		(north_to_west),
		        .Dsel (1'b0),
			.oktosend	(west_oktosend),
			.Alast		(east_lastpart),
			.Blast		(south_lastpart),
			.Clast		(north_lastpart),
			.Dlast(1'b0)
			);
   
   


   //////////////////////////////////////////////////////////////////////
   // East part
   //////////////////////////////////////////////////////////////////////   

   assign 	  east_stb = east_ctrl_i[`SENDBIT];

   parameter 	  EASTLOWWATERMARK = 4'h9;
   defparam 	  eastfifo.LOWWATERMARK = EASTLOWWATERMARK;
    srlfifo eastfifo(.clk		(clk),
					.rst		(rst),
					.rd(east_rd),
					
					.sendok	(east_o[`DATAWIDTH+`SENDOKBIT]),
					.avail		(east_avail),
					.q		(east_fifodata),
					.controlq		(east_fifoctrl),
					// Inputs
					.d		(east_data_i),
					.control	(east_ctrl_i),
					.we		(east_stb),
					.toA            (east_to_west_nextsel),
					.toB            (east_to_north_nextsel),
					.toC            (east_to_south_nextsel),
					.Asel(east_to_west),
					.Bsel(east_to_north),
					.Csel(east_to_south),
					 .Dsel());
   

   assign 	  east_lastpart = east_fifoctrl[`LASTBIT];
   
       readenable eastread(
		       // Outputs
		       .rd		(east_rd),
		       // Inputs
		       .nolocal_rd	(east_avail),
		       .A_Asel		(east_to_west),
		       .A_Bsel		(north_to_west),
		       .A_Csel		(south_to_west),
		       .A_Dsel		(1'b0),
		       .B_Asel		(east_to_south),
		       .B_Bsel		(west_to_south),
		       .B_Csel		(north_to_south),
		       .B_Dsel		(1'b0),
		       .C_Asel		(east_to_north),
		       .C_Bsel		(west_to_north),
		       .C_Csel		(south_to_north),
		       .C_Dsel		(1'b0),
		       .D_Asel (1'b0),
		       .D_Bsel (1'b0),
		       .D_Csel (1'b0),
		       .D_Dsel (1'b0),
		       .Aoktosend	(west_oktosend2),
		       .Boktosend	(south_oktosend2),
		       .Coktosend	(north_oktosend2),
		       .Doktosend	(1'b0),
		       .Aoverride	(east_to_west_override),
		       .Boverride	(east_to_south_override),
		       .Coverride	(east_to_north_override),
		       .Doverride(1'b0));


    outputmux east_mux(.clk(clk),
		      .rst(rst),
		      .oktosend(east_oktosend),
		      .send(east_o[`DATAWIDTH+`SENDBIT]),

		      .A({west_to_east_nextsel,  west_fifoctrl[`LASTBIT:0],west_fifodata}),
		      .B({south_to_east_nextsel, south_fifoctrl[`LASTBIT:0],south_fifodata}),
		      .C({north_to_east_nextsel, north_fifoctrl[`LASTBIT:0],north_fifodata}),
		      .D(0),
		      .Asel(west_to_east),
		      .Bsel(south_to_east),
		      .Csel(north_to_east),
		      .Dsel(1'b0),
		      .Aoverride(west_to_east_override),
		      .Boverride(south_to_east_override),
		      .Coverride(north_to_east_override),
		      .Doverride(1'b0),
		      .override(east_override),
		      .Q(east_o[`DATAWIDTH+`SENDBIT - 1:0])
		      );


       arbiter east_arbiter(
			.clk		(clk),
			.rst		(rst),

			.override	(east_override),
			.Aoverride	(west_to_east_override),
			.Boverride	(south_to_east_override),
			.Coverride	(north_to_east_override),
			.Doverride	(),


			.Asel		(west_to_east),
			.Bsel		(south_to_east),
			.Csel		(north_to_east),
			.Dsel		(1'b0),
			.oktosend	(east_oktosend),
			.Alast		(west_lastpart),
			.Blast		(south_lastpart),
			.Clast		(north_lastpart),
			.Dlast		(1'b0)
			);
   
   
   

   //////////////////////////////////////////////////////////////////////
   // North part
   //////////////////////////////////////////////////////////////////////   

   assign 	  north_stb = north_ctrl_i[`SENDBIT];


   parameter 	  NORTHLOWWATERMARK = 4'h9;
   defparam 	  northfifo.LOWWATERMARK = NORTHLOWWATERMARK;

    srlfifo northfifo(.clk		(clk),
					 .rst		(rst),
					 .rd(north_rd),
					 
					 .sendok	(north_o[`DATAWIDTH+`SENDOKBIT]),
					 .avail		(north_avail),
					 .q		(north_fifodata),
					 .controlq		(north_fifoctrl),
					 // Inputs
					 .d		(north_data_i),
					 .control	(north_ctrl_i),
					 .we		(north_stb),
					 .toA            (north_to_west_nextsel),
					 .toB            (north_to_east_nextsel),
					 .toC            (north_to_south_nextsel),
					 .Asel(north_to_west),
					 .Bsel(north_to_east),
					 .Csel(north_to_south),
					 .Dsel());
   
   assign 	  north_lastpart = north_fifoctrl[`LASTBIT];
   
       readenable northread(
		       // Outputs
		       .rd		(north_rd),
		       // Inputs
		       .nolocal_rd	(north_avail),
		       .A_Asel		(north_to_west),
		       .A_Bsel		(east_to_west),
		       .A_Csel		(south_to_west),
		       .A_Dsel		(1'b0),
		       .B_Asel		(north_to_south),
		       .B_Bsel		(west_to_south),
		       .B_Csel		(east_to_south),
		       .B_Dsel		(1'b0),
		       .C_Asel		(north_to_east),
		       .C_Bsel		(west_to_east),
		       .C_Csel		(south_to_east),
		       .C_Dsel		(1'b0),
		       .D_Asel (1'b0),
		       .D_Bsel (1'b0),
		       .D_Csel (1'b0),
		       .D_Dsel (1'b0),
		       .Aoktosend	(west_oktosend3),
		       .Boktosend	(south_oktosend3),
		       .Coktosend	(east_oktosend3),
		       .Doktosend	(1'b0),
		       .Aoverride	(north_to_west_override),
		       .Boverride	(north_to_south_override),
		       .Coverride	(north_to_east_override),
		       .Doverride(1'b0));



    outputmux north_mux(.clk(clk),
		       .rst(rst),
		      .oktosend(north_oktosend),
		      .send(north_o[`DATAWIDTH+`SENDBIT]),

		      .A({west_to_north_nextsel,  west_fifoctrl[`LASTBIT:0],west_fifodata}),
		      .B({south_to_north_nextsel, south_fifoctrl[`LASTBIT:0],south_fifodata}),
		      .C({east_to_north_nextsel, east_fifoctrl[`LASTBIT:0],east_fifodata}),
		      .D(0),
		      .Asel(west_to_north),
		      .Bsel(south_to_north),
		      .Csel(east_to_north),
					    .Dsel(1'b0),
		      .Aoverride(west_to_north_override),
		      .Boverride(south_to_north_override),
		      .Coverride(east_to_north_override),
					    .Doverride(1'b0),
		      .override(north_override),
		      .Q(north_o[`DATAWIDTH+`SENDBIT - 1:0])
		      );


       arbiter north_arbiter(
			.clk		(clk),
			.rst		(rst),

			.override	(north_override),
			.Aoverride	(west_to_north_override),
			.Boverride	(south_to_north_override),
			.Coverride	(east_to_north_override),
			.Doverride	(),


			.Asel		(west_to_north),
			.Bsel		(south_to_north),
			.Csel		(east_to_north),
			.Dsel		(1'b0),
			.oktosend	(north_oktosend),
			.Alast		(west_lastpart),
			.Blast		(south_lastpart),
			.Clast		(east_lastpart),
			.Dlast		(1'b0)
			);

   //////////////////////////////////////////////////////////////////////
   // South part
   //////////////////////////////////////////////////////////////////////   

   assign 	  south_stb = south_ctrl_i[`SENDBIT];


   parameter 	  SOUTHLOWWATERMARK = 4'h9;
   defparam 	  southfifo.LOWWATERMARK = SOUTHLOWWATERMARK;


    srlfifo southfifo(.clk		(clk),
					 .rst		(rst),
					 .rd(south_rd),
					 
					 .sendok	(south_o[`DATAWIDTH+`SENDOKBIT]),
					 .avail		(south_avail),
					 .q		(south_fifodata),
					  .controlq	(south_fifoctrl),
					 // Inputs
					 .d		(south_data_i),
					 .control	(south_ctrl_i),
					 .we		(south_stb),
					 .toA            (south_to_west_nextsel),
					 .toB            (south_to_east_nextsel),
					 .toC            (south_to_north_nextsel),
					 .Asel(south_to_west),
					 .Bsel(south_to_north),
					  .Csel(south_to_east),
					  .Dsel());
   
    assign 	  south_lastpart = south_fifoctrl[`LASTBIT];
    

       readenable southread(
		       // Outputs
		       .rd		(south_rd),
		       // Inputs
		       .nolocal_rd	(south_avail),
		       .A_Asel		(south_to_west),
		       .A_Bsel		(east_to_west),
		       .A_Csel		(north_to_west),
		       .A_Dsel		(1'b0),
		       .B_Asel		(south_to_north),
		       .B_Bsel		(west_to_north),
		       .B_Csel		(east_to_north),
		       .B_Dsel		(1'b0),
		       .C_Asel		(south_to_east),
		       .C_Bsel		(west_to_east),
		       .C_Csel		(north_to_east),
		       .C_Dsel		(1'b0),
		       .D_Asel (1'b0),
		       .D_Bsel (1'b0),
		       .D_Csel (1'b0),
		       .D_Dsel (1'b0),
		       .Aoktosend	(west_oktosend4),
		       .Boktosend	(north_oktosend4),
		       .Coktosend	(east_oktosend4),
		       .Doktosend	(1'b0),
		       .Aoverride	(south_to_west_override),
		       .Boverride	(south_to_north_override),
		       .Coverride	(south_to_east_override),
		       .Doverride	(1'b0));
   

    outputmux south_mux(.clk(clk),
		       .rst(rst),
		      .oktosend(south_oktosend),
		      .send(south_o[`DATAWIDTH+`SENDBIT]),

		      .A({west_to_south_nextsel,  west_fifoctrl[`LASTBIT:0],west_fifodata}),
		      .B({north_to_south_nextsel, north_fifoctrl[`LASTBIT:0],north_fifodata}),
		      .C({east_to_south_nextsel,  east_fifoctrl[`LASTBIT:0],east_fifodata}),
		      .D(0),
		      .Asel(west_to_south),
		      .Bsel(north_to_south),
		      .Csel(east_to_south),
					   .Dsel(1'b0),
		      .Aoverride(west_to_south_override),
		      .Boverride(north_to_south_override),
		      .Coverride(east_to_south_override),
					   .Doverride(1'b0),
		      .override(south_override),
		      .Q(south_o[`DATAWIDTH+`SENDBIT - 1:0])
		      );


       arbiter south_arbiter(
			.clk		(clk),
			.rst		(rst),

			.override	(south_override),
			.Aoverride	(west_to_south_override),
			.Boverride	(north_to_south_override),
			.Coverride	(east_to_south_override),
			.Doverride	(),

			.Asel		(west_to_south),
			.Bsel		(north_to_south),
			.Csel		(east_to_south),
			.Dsel		(1'b0),
			.oktosend	(south_oktosend),
			.Alast		(west_lastpart),
			.Blast		(north_lastpart),
			.Clast		(east_lastpart),
			.Dlast		(1'b0)
			);

   
endmodule
