`timescale 1ns / 10ps
// Generate the read enable for the input FIFO combinatorially from
// all incoming select signal and the state of all output port
// arbiters
//
// This is one of the most critical paths.  It can almost fit into one
// CLB for a node with four input ports but it gets a bit worse for a
// five input switch.

module readenable(
    input wire A_Asel,
    input wire A_Bsel,
    input wire A_Csel,
    input wire A_Dsel,

    input wire B_Asel,
    input wire B_Bsel,
    input wire B_Csel,
    input wire B_Dsel,

    input wire C_Asel,
    input wire C_Bsel,
    input wire C_Csel,
    input wire C_Dsel,

    input wire D_Asel,
    input wire D_Bsel,
    input wire D_Csel,
    input wire D_Dsel,

    input wire Aoktosend,
    input wire Boktosend,
    input wire Coktosend,
    input wire Doktosend,

    input wire Aoverride,
    input wire Boverride,
    input wire Coverride,
    input wire Doverride,

    input wire nolocal_rd,

    output wire rd);

   wire 	rd1;
   wire 	rd2;
   wire 	rd3;
   wire 	rd4;
   wire 	rd5;
   wire 	rd6;


   // ABCok
   // 0000    0
   // 0001    0
   // 0010    0
   // 0011    0
   // 0100    0
   // 0101    0
   // 0110    0
   // 0111    0
   // 1000    0
   // 1001    1
   // 1010    0
   // 1011    0
   // 1100    0
   // 1101    0
   // 1110    0
   // 1111    0
   // INIT = 16'b0000001000000000; = 16'h0200;

   // 0000    0
   // 0001    0
   // 0010    0
   // 0011    1
   // 0100    0
   // 0101    0
   // 0110    0
   // 0111    1
   // 1000    0
   // 1001    0
   // 1010    0
   // 1011    1
   // 1100    1
   // 1101    1
   // 1110    1
   // 1111    1
   // INIT = 16'b1111100010001000; = 16'hf888;


   defparam 	slice1.INIT = 32'h00000200;
   defparam 	slice2.INIT = 32'h0200;
   defparam 	slice3.INIT = 32'h0200;
   defparam 	slice4.INIT = 32'h0200;

   wire 	rd_d;
   
   SLICE5 slice1 (.I0(Aoktosend),.I1(A_Csel),.I2(A_Bsel),.I3(A_Asel),.I4(A_Dsel),.O(rd1));
   SLICE5 slice2 (.I0(Boktosend),.I1(B_Csel),.I2(B_Bsel),.I3(B_Asel),.I4(B_Dsel),.O(rd2));
   SLICE5 slice3 (.I0(Coktosend),.I1(C_Csel),.I2(C_Bsel),.I3(C_Asel),.I4(C_Dsel),.O(rd3));
   SLICE5 slice4 (.I0(Doktosend),.I1(D_Csel),.I2(D_Bsel),.I3(D_Asel),.I4(D_Dsel),.O(rd_d));
   
   defparam 	lut5.INIT = 16'hf888;
   defparam 	lut6.INIT = 16'hf888;
   LUT4 lut5 (.I0(Aoverride),.I1(Aoktosend),.I2(Boverride),.I3(Boktosend),.O(rd4));
   LUT4 lut6 (.I0(Coverride),.I1(Coktosend),.I2(Doverride),.I3(Doktosend),.O(rd5));

   
   

   defparam 	lut7.INIT = 16'hfffe;
   LUT4 lut7 (.I0(rd1),.I1(rd2),.I2(rd3),.I3(rd5),.O(rd6));

   defparam 	lut8.INIT = 16'b1111111011111111;
   LUT4 lut8(.I0(rd6),.I1(rd_d),.I2(rd4),.I3(nolocal_rd),.O(rd));
   
endmodule // readenable

