`timescale 1ns / 10ps
`default_nettype none
`include "noc_pkt.vh"
// TODO: Merge this and srlfifo.v
  

module srlfifo(input wire clk,
    input wire rst,

    input wire [`CTRLWIDTH-1:0] control,
    input wire [`DATAWIDTH-1:0] d,
  
    input wire 			we,
    output reg 			sendok,
  
    output wire 		avail,
    output wire [`DATAWIDTH-1:0] q,
    output wire [`LASTBIT:0] 	 controlq,


    input wire 			 rd,
    output wire 		 Asel,Bsel,Csel,Dsel,

    output wire [3:0] 		 toA,
    output wire [3:0] 		 toB,
    output wire [3:0] 		 toC
		 );


   wire [3:0] 			 addrptr;
   reg [3:0] 			 new_addrptr;
   wire [`DATAWIDTH-1:0] 	 srlq;
   wire [`DESTWIDTH+`NEXTHOPWIDTH+1-1:0] controlsrlq;
   
   wire 				 avail_in_ff;
   wire 				 avail_int;
   
   wire 				 fdce_sel;
   

   assign 				 avail_int = addrptr != 4'hf;

   parameter 				 LOWWATERMARK = 4'h9; // Default watermark for the default NoC switch architecture
   // with no pipeline stages between NoC switches.
   always @(posedge clk) begin
      sendok <= (addrptr == 4'hf) || (addrptr < LOWWATERMARK);
   end



   wire 	      extread;
   assign 	      extread = rd;

   FDS addrptr0 (.C(clk),.S(rst),.D(new_addrptr[0]),.Q(addrptr[0]));
   FDS addrptr1 (.C(clk),.S(rst),.D(new_addrptr[1]),.Q(addrptr[1]));
   FDS addrptr2 (.C(clk),.S(rst),.D(new_addrptr[2]),.Q(addrptr[2]));
   FDS addrptr3 (.C(clk),.S(rst),.D(new_addrptr[3]),.Q(addrptr[3]));
   

   reg [3:0] 	      maxval = 0;

   always @(posedge clk) begin
      if(avail_int) 

	//	if(maxval < addrptr) begin
	//	   $display("%m: New maxval: %d",addrptr);
	//	   maxval <= addrptr;
	//	end

	if(avail && !(Asel || Bsel || Csel || Dsel)) begin
	   $display("%m: Error, no sel signal active!");
	   $stop;
	end
   end
   

   always @* begin
      new_addrptr = addrptr;
      if(we) begin
	 //	 $display("%t: %m: Got data %x",$time,d);
	 if(!((extread) && avail_int))  new_addrptr = addrptr + 1;
      end else if((extread) && avail_int) begin
	 //	    if(addrptr != 4'hf) begin
	 new_addrptr = addrptr - 1;
	 //	    end
      end

   end // always @ (posedge clk)

   assign fdce_sel = extread;

   reg 	  avail_flag;
   always @* begin
      if(extread) begin
	 avail_flag = avail_int;
      end else begin
	 avail_flag = avail_in_ff;
      end
   end

   always @(posedge clk) begin
      if(extread && avail_in_ff) begin
	 //	 $display("%m: reading from FIFO: read value %x (destinations are %x)",q,{Asel,Bsel,Csel});
      end
   end

   always @(posedge clk) begin
      if(addrptr == 4'he) begin
	 if(new_addrptr == 4'hf) begin
	    $display("%m: Fifo overflow, aborting simulation!!");
	    $stop;
	 end
      end
   end


   FDR availfd (.C(clk),.R(rst),.D(avail_flag),.Q(avail_in_ff));
   assign avail = avail_in_ff;

   genvar i;

   generate
      for(i=0; i < `DATAWIDTH;i = i + 1) begin:FLIPFLOPS
	 FDE fd0(.C(clk),.CE(fdce_sel),.D(srlq[i]),.Q(q[i]));
      end
   endgenerate

   FDE fd36(.C(clk),.CE(fdce_sel),.D(controlsrlq[0]),.Q(controlq[0]));
   FDE fd37(.C(clk),.CE(fdce_sel),.D(controlsrlq[1]),.Q(controlq[1]));
   FDE fd38(.C(clk),.CE(fdce_sel),.D(controlsrlq[2]),.Q(controlq[2]));
   FDE fd39(.C(clk),.CE(fdce_sel),.D(controlsrlq[3]),.Q(controlq[3]));
   FDE fd40(.C(clk),.CE(fdce_sel),.D(controlsrlq[4]),.Q(controlq[4]));
   FDE fd41(.C(clk),.CE(fdce_sel),.D(controlsrlq[5]),.Q(controlq[5]));

   wire   Asel_int,Bsel_int,Csel_int,Dsel_int;

   assign Asel_int = controlsrlq[`NEXTHOPBIT+0] && avail_int;
   assign Bsel_int = controlsrlq[`NEXTHOPBIT+1] && avail_int;
   assign Csel_int = controlsrlq[`NEXTHOPBIT+2] && avail_int;
   assign Dsel_int = controlsrlq[`NEXTHOPBIT+3] && avail_int;
   
   
   FDE fd42(.C(clk),.CE(fdce_sel),.D(Asel_int),.Q(Asel));
   FDE fd43(.C(clk),.CE(fdce_sel),.D(Bsel_int),.Q(Bsel));
   FDE fd44(.C(clk),.CE(fdce_sel),.D(Csel_int),.Q(Csel));
   FDE fd45(.C(clk),.CE(fdce_sel),.D(Dsel_int),.Q(Dsel));


   
   parameter UNIFIED_ROUTELOOKUP = 1;
   
   parameter [31:0] A_AROUTE = 32'h00000000;
   parameter [31:0] A_BROUTE = 32'h00000000;
   parameter [31:0] A_CROUTE = 32'h00000000;
   parameter [31:0] A_DROUTE = 32'h00000000;
   parameter [31:0] B_AROUTE = 32'h00000000;
   parameter [31:0] B_BROUTE = 32'h00000000;
   parameter [31:0] B_CROUTE = 32'h00000000;
   parameter [31:0] B_DROUTE = 32'h00000000;
   parameter [31:0] C_AROUTE = 32'h00000000;
   parameter [31:0] C_BROUTE = 32'h00000000;
   parameter [31:0] C_CROUTE = 32'h00000000;
   parameter [31:0] C_DROUTE = 32'h00000000;

   parameter [31:0] AROUTE = 32'h00000000;
   parameter [31:0] BROUTE = 32'h00000000;
   parameter [31:0] CROUTE = 32'h00000000;
   parameter [31:0] DROUTE = 32'h00000000;


   generate
      // This is retained here because we will need something like this for
      // adaptive routing.
      if(UNIFIED_ROUTELOOKUP == 0) begin: FOO
	 defparam 	    route_toA_A.ROUTE = A_AROUTE;
	 defparam 	    route_toA_B.ROUTE = A_BROUTE;
	 defparam 	    route_toA_C.ROUTE = A_CROUTE;
	 defparam 	    route_toA_D.ROUTE = A_DROUTE;
	 defparam 	    route_toB_A.ROUTE = B_AROUTE;
	 defparam 	    route_toB_B.ROUTE = B_BROUTE;
	 defparam 	    route_toB_C.ROUTE = B_CROUTE;
	 defparam 	    route_toB_D.ROUTE = B_DROUTE;
	 defparam 	    route_toC_A.ROUTE = C_AROUTE;
	 defparam 	    route_toC_B.ROUTE = C_BROUTE;
	 defparam 	    route_toC_C.ROUTE = C_CROUTE;
	 defparam 	    route_toC_D.ROUTE = C_DROUTE;

	 nextroute route_toA_A(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toA[0]));
	 nextroute route_toA_B(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toA[1]));
	 nextroute route_toA_C(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toA[2]));
	 nextroute route_toA_D(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toA[3]));
	 nextroute route_toB_A(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toB[0]));
	 nextroute route_toB_B(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toB[1]));
	 nextroute route_toB_C(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toB[2]));
	 nextroute route_toB_D(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toB[3]));
	 nextroute route_toC_A(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toC[0]));
	 nextroute route_toC_B(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toC[1]));
	 nextroute route_toC_C(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toC[2]));
	 nextroute route_toC_D(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(srlq[`DESTWIDTH-1:0]),.sel(toC[3]));
      end else begin:BAR // if (ROUTELOOKUP_FOR_EACH_BLOCK)
	 defparam 	    route_a.ROUTE = AROUTE;
	 defparam 	    route_b.ROUTE = BROUTE;
	 defparam 	    route_c.ROUTE = CROUTE;
	 defparam 	    route_d.ROUTE = DROUTE;
	 nextroute route_a(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(controlsrlq[`DESTWIDTH-1:0]),.sel(toA[0]));
	 nextroute route_b(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(controlsrlq[`DESTWIDTH-1:0]),.sel(toA[1]));
	 nextroute route_c(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(controlsrlq[`DESTWIDTH-1:0]),.sel(toA[2]));
	 nextroute route_d(.clk(clk),.rst(rst),.ce(fdce_sel),.destaddr(controlsrlq[`DESTWIDTH-1:0]),.sel(toA[3]));
	 assign 	    toB = toA;
	 assign 	    toC = toA;
      end // else: !if(ROUTELOOKUP_FOR_EACH_BLOCK)
   endgenerate

   generate
      for(i=0; i < `DATAWIDTH; i = i + 1) begin:FIFO
	 SRL16E srl00(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[i]),.D(d[i]));
      end
   endgenerate
   
   SRL16E srl36(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 0]),.D(control[ 0]));
   SRL16E srl37(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 1]),.D(control[ 1]));
   SRL16E srl38(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 2]),.D(control[ 2]));
   SRL16E srl39(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 3]),.D(control[ 3]));
   SRL16E srl40(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 4]),.D(control[ 4]));
   SRL16E srl41(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 5]),.D(control[ 5]));
   SRL16E srl42(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 6]),.D(control[ 6]));
   SRL16E srl43(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 7]),.D(control[ 7]));
   SRL16E srl44(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 8]),.D(control[ 8]));
   SRL16E srl45(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(controlsrlq[ 9]),.D(control[ 9]));

endmodule // srlfifo

