`timescale 1ns / 10ps
`default_nettype none

module serial_wb_parport( 

   input wire        clk_i,rst_i,


   output reg [31:0] parport_o,
   
   output reg [7:0]  data_o,
   input wire [31:0]  parport_i,
   
   input wire         writestrobe_i,
   input wire [7:0] 	 data_i,
   input wire [1:0] 	 address_i,
   input wire         readstrobe_i,

   output reg	 parport_writestrobe_o,
   output reg	 parport_readstrobe_o

			  );
   



   reg [23:0]	 outputreg_r;
   reg [23:0]	 inputreg_r;



   always @* begin
      data_o = parport_i[7:0];
      case(address_i)
	 2'b01: data_o = inputreg_r[7:0];
	 2'b10: data_o = inputreg_r[15:8];
	 2'b11: data_o = inputreg_r[23:16];
      endcase // case(address_i)

      
      parport_readstrobe_o = 0;
      if((address_i == 2'b00) && readstrobe_i) begin
	 parport_readstrobe_o = 1;
      end
   end

   always @(posedge clk_i) begin
      parport_writestrobe_o <= 0;
      if(rst_i) begin
	 parport_o <= 0;
	 outputreg_r <= 0;
	 parport_writestrobe_o <= 0;
      end else if(writestrobe_i) begin
	 parport_writestrobe_o <= 0;
	 case(address_i)
	   2'b00: outputreg_r[7:0] <= data_i;
	   2'b01: outputreg_r[15:8] <= data_i;
	   2'b10: outputreg_r[23:16] <= data_i;
	   2'b11: begin
	      parport_o <= {data_i[7:0],outputreg_r[23:0]};
	      parport_writestrobe_o <= 1;
	   end
	   
	 endcase // case(address_i)
      end else if(readstrobe_i) begin
	 if(address_i == 2'b00) begin
	    inputreg_r <= parport_i[31:8];
	 end
      end
   end

endmodule // serial_wb_parport
