`timescale 1ns / 10ps
`default_nettype none

module serial_wb_tb;

   wire [7:0] led;
   reg       clk;
   reg       rst;
   wire       tx_o;
   reg       rx;

   wire [31:0] parport_i;
   wire [31:0] parport_o;
   assign      parport_i = 32'hdeadbeef;
   

   serial_wb_top mcu(
		     // Outputs
		     .tx_o		(tx_o),
		     .parport_o		(parport_o[31:0]),
		     .parport_readstrobe_o(),
		     .parport_writestrobe_o(),
		     .wbm_dat_o		(),
		     .wbm_adr_o		(),
		     .wbm_cyc_o		(),
		     .wbm_lock_o	(),
		     .wbm_sel_o		(),
		     .wbm_stb_o		(),
		     .wbm_we_o		(),
		     // Inputs
		     .clk_i		(clk),
		     .rst_i		(rst),
		     .rx_i		(rx),
		     .parport_i		(parport_i[31:0]),
		     .wbm_dat_i		(32'h0),
		     .wbm_ack_i		(1'b0),
		     .wbm_err_i		(1'b0),
		     .wbm_rty_i		(1'b0));
   
   always begin
      clk = 0;
      #50;
      clk = 1;
      #50;
   end

   parameter BAUDRATE = 868;

   
      
   task wishbone_write;
      input [63:0] addr;
      input [63:0] data;
      integer 	    i;
      begin
	 putch("m");
	 putch(" ");
	 for(i = 7; i >= 0; i = i - 1) begin
	    putch(addr[63:56]);
	    addr = addr << 8;
	 end
	 putch(" ");
	 for(i = 7; i >= 0; i = i - 1) begin
	    putch(data[63:56]);
	    data = data << 8;
	 end
      repeat(6000) @(posedge clk);
      end
   endtask // wishbone_write

   task parport_write;
      input [63:0] data;
      integer 	    i;
      begin
	 putch("o");
	 putch(" ");
	 for(i = 7; i >= 0; i = i - 1) begin
	    putch(data[63:56]);
	    data = data << 8;
	 end
      end
   endtask // parport_write
      
   task putch;
      input  [7:0] char;
      begin
	 $display("Trying to write out '%c'",char);
	 @(posedge clk);

	 rx <= 0;
	 repeat(BAUDRATE) @(posedge clk);

	 rx <= char[0];
	 repeat(BAUDRATE) @(posedge clk);

	 rx <= char[1];
	 repeat(BAUDRATE) @(posedge clk);

	 rx <= char[2];
	 repeat(BAUDRATE) @(posedge clk);

	 rx <= char[3];
	 repeat(BAUDRATE) @(posedge clk);

	 rx <= char[4];
	 repeat(BAUDRATE) @(posedge clk);

	 rx <= char[5];
	 repeat(BAUDRATE) @(posedge clk);

	 rx <= char[6];
	 repeat(BAUDRATE) @(posedge clk);

	 rx <= char[7];
	 repeat(BAUDRATE) @(posedge clk);

	 // stopbit
	 rx <= 1;
	 repeat(BAUDRATE) @(posedge clk);
      end
   endtask // putch


   
   initial begin
      rx = 1;
      rst = 1;
      repeat(100) @(posedge clk);
      rst <= 0;
      repeat(250000) @(posedge clk);
      $display("Writing to parport via rs232");
      parport_write("f00f115a");
      $display("Command finished");
      repeat(50000) @(posedge clk);
      if(parport_o === 32'hf00f115a) begin
	 $display("Success!");
	 $stop;
      end else begin
	 $display("Failure, parport_o is %x, expected f00f115a",parport_o);
	 $stop;
      end
   end

endmodule // tb_system
