// Testbench for a mesh with 2 x 2

`timescale 1ns / 10ps
`default_nettype none
module noc(
    input wire clk, rst,
    input wire [31:0] confbus_i,
    input wire [11:0] confbus_addr_i,
    input wire 	      confbus_we_i,
    output wire [31:0] confbus_o
	   );
   wire [47:0] link_0_to_1;
   wire [47:0] link_1_to_0;
   wire [47:0] link_0_to_2;
   wire [47:0] link_2_to_0;
   wire [47:0] port0_o;
   wire [47:0] port0_i;
   wire [47:0] port1_o;
   wire [47:0] port1_i;
   wire [47:0] port2_o;
   wire [47:0] port2_i;
   node5 node0(
           .clk(clk), .rst(rst),

           .west_o        (port0_o),
           .west_i        (port0_i),
           .north_o        (port1_o),
           .north_i        (port1_i),
           .south_o        (link_0_to_2),
           .south_i        (link_2_to_0),
           .east_o        (link_0_to_1),
           .east_i        (link_1_to_0),
           .local_o        (port2_o),
              .local_i        (port2_i));
   wire [47:0] link_1_to_3;
   wire [47:0] link_3_to_1;
   wire [47:0] port3_o;
   wire [47:0] port3_i;
   wire [47:0] port4_o;
   wire [47:0] port4_i;
   wire [47:0] port5_o;
   wire [47:0] port5_i;
   node5 node1(
           .clk(clk), .rst(rst),

           .west_o        (link_1_to_0),
           .west_i        (link_0_to_1),
           .north_o        (port3_o),
           .north_i        (port3_i),
           .south_o        (link_1_to_3),
           .south_i        (link_3_to_1),
           .east_o        (port4_o),
           .east_i        (port4_i),
           .local_o        (port5_o),
              .local_i        (port5_i));
   wire [47:0] link_2_to_3;
   wire [47:0] link_3_to_2;
   wire [47:0] port6_o;
   wire [47:0] port6_i;
   wire [47:0] port7_o;
   wire [47:0] port7_i;
   wire [47:0] port8_o;
   wire [47:0] port8_i;
   node5 node2(
           .clk(clk), .rst(rst),

           .west_o        (port6_o),
           .west_i        (port6_i),
           .north_o        (link_2_to_0),
           .north_i        (link_0_to_2),
           .south_o        (port7_o),
           .south_i        (port7_i),
           .east_o        (link_2_to_3),
           .east_i        (link_3_to_2),
           .local_o        (port8_o),
              .local_i        (port8_i));
   wire [47:0] port9_o;
   wire [47:0] port9_i;
   wire [47:0] port10_o;
   wire [47:0] port10_i;
   wire [47:0] port11_o;
   wire [47:0] port11_i;
   node5 node3(
           .clk(clk), .rst(rst),

           .west_o        (link_3_to_2),
           .west_i        (link_2_to_3),
           .north_o        (link_3_to_1),
           .north_i        (link_1_to_3),
           .south_o        (port9_o),
           .south_i        (port9_i),
           .east_o        (port10_o),
           .east_i        (port10_i),
           .local_o        (port11_o),
              .local_i        (port11_i));
   localparam [31:0] ROUTE_NONE = 32'h00000000;
   localparam [31:0] ROUTE_0 = 32'h00000001;
   localparam [31:0] ROUTE_1 = 32'h00000002;
   localparam [31:0] ROUTE_2 = 32'h00000004;
   localparam [31:0] ROUTE_3 = 32'h00000008;
   localparam [31:0] ROUTE_4 = 32'h00000010;
   localparam [31:0] ROUTE_5 = 32'h00000020;
   localparam [31:0] ROUTE_6 = 32'h00000040;
   localparam [31:0] ROUTE_7 = 32'h00000080;
   localparam [31:0] ROUTE_8 = 32'h00000100;
   localparam [31:0] ROUTE_9 = 32'h00000200;
   localparam [31:0] ROUTE_10 = 32'h00000400;
   localparam [31:0] ROUTE_11 = 32'h00000800;

    defparam node0.UNIFIED_ROUTELOOKUP = 1;
    defparam node0.AROUTE = ROUTE_NONE | ROUTE_3 | ROUTE_6;
    defparam node0.BROUTE = ROUTE_NONE | ROUTE_4;
    defparam node0.CROUTE = ROUTE_NONE | ROUTE_7 | ROUTE_9 | ROUTE_10 | ROUTE_11;
    defparam node0.DROUTE = ROUTE_NONE | ROUTE_5 | ROUTE_8;


    defparam node1.UNIFIED_ROUTELOOKUP = 1;
    defparam node1.AROUTE = ROUTE_NONE | ROUTE_0;
    defparam node1.BROUTE = ROUTE_NONE | ROUTE_1 | ROUTE_10;
    defparam node1.CROUTE = ROUTE_NONE | ROUTE_6 | ROUTE_7 | ROUTE_8 | ROUTE_9;
    defparam node1.DROUTE = ROUTE_NONE | ROUTE_2 | ROUTE_11;


    defparam node2.UNIFIED_ROUTELOOKUP = 1;
    defparam node2.AROUTE = ROUTE_NONE | ROUTE_0 | ROUTE_3 | ROUTE_4 | ROUTE_5;
    defparam node2.BROUTE = ROUTE_NONE | ROUTE_1 | ROUTE_10;
    defparam node2.CROUTE = ROUTE_NONE | ROUTE_9;
    defparam node2.DROUTE = ROUTE_NONE | ROUTE_2 | ROUTE_7 | ROUTE_11;


    defparam node3.UNIFIED_ROUTELOOKUP = 1;
    defparam node3.AROUTE = ROUTE_NONE | ROUTE_6;
    defparam node3.BROUTE = ROUTE_NONE | ROUTE_0 | ROUTE_1 | ROUTE_2 | ROUTE_3;
    defparam node3.CROUTE = ROUTE_NONE | ROUTE_4 | ROUTE_7;
    defparam node3.DROUTE = ROUTE_NONE | ROUTE_5 | ROUTE_8 | ROUTE_9;

    wire [31:0] confbus0 = confbus_i;
    wire [11:0] confbus_addr0 = confbus_addr_i;
    wire        confbus_we0 = confbus_we_i;

   defparam    if0.THISNODEADDRESS = 5'd0;
   // Port 0 is connected to node at (0,0) nodeport is 4
   defparam    if0.NUM_DESTS = 11;
   defparam    if0.NEXTHOPNODES = { 5'd1, 5'd2, 5'd3, 5'd4, 5'd5, 5'd6, 5'd7, 5'd8, 5'd9, 5'd10, 5'd11};
   defparam    if0.NEXTHOPDESTS = { 4'b0001, 4'b1000, 4'b0010, 4'b0010, 4'b0010, 4'b0100, 4'b0100, 4'b0100, 4'b0010, 4'b0010, 4'b0010 };
   defparam    if0.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if0.ADDRESS = { 32'h08000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if0.DESTS = { 4'b0001, 5'd1, 4'b1000, 5'd2, 4'b0010, 5'd3, 4'b0010, 5'd4, 4'b0010, 5'd5, 4'b0100, 5'd6, 4'b0100, 5'd7, 4'b0100, 5'd8, 4'b0010, 5'd9, 4'b0010, 5'd10, 4'b0010, 5'd11 };

   wire [31:0] 		 m0_dat_i;
   wire [31:0] 		 m0_dat_o;
   wire [31:0] 		 m0_adr;
   wire [3:0] 		 m0_sel;
   wire 		 m0_ack;
   wire 		 m0_rty;
   wire 		 m0_err;
   wire 		 m0_we;
   wire 		 m0_cyc;
   wire 		 m0_stb;
   wire 		 m0_cab;
   wire [4:0]		 m0_size;

   wire [31:0] 		 s0_dat_i;
   wire [31:0] 		 s0_dat_o;
   wire [31:0] 		 s0_adr;
   wire [3:0] 		 s0_sel;
   wire 		 s0_ack;
   wire 		 s0_rty;
   wire 		 s0_err;
   wire 		 s0_we;
   wire 		 s0_cyc;
   wire 		 s0_stb;
   wire 		 s0_cab;
   wire [4:0]		 s0_size;

   wb_if if0(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port0_i),
	     .soclink_i			(port0_o),

	     // Slave port
	     .wbs_dat_o			(m0_dat_i),
	     .wbs_ack_o			(m0_ack),
	     .wbs_rty_o			(m0_rty),
	     .wbs_err_o			(m0_err),
	     .wbs_adr_i			(m0_adr),
	     .wbs_dat_i			(m0_dat_o),
	     .wbs_stb_i			(m0_stb),
	     .wbs_cyc_i			(m0_cyc),
	     .wbs_cab_i			(m0_cab),
	     .wbs_we_i			(m0_we),
	     .wbs_sel_i			(m0_sel),
             .wbs_size_i                (m0_size),

	     // Master port
	     .wbm_dat_i			(s0_dat_i),
	     .wbm_ack_i			(s0_ack),
	     .wbm_rty_i			(s0_rty),
	     .wbm_err_i			(s0_err),
	     .wbm_stb_o			(s0_stb),
	     .wbm_cyc_o			(s0_cyc),
	     .wbm_cab_o			(s0_cab),
	     .wbm_we_o			(s0_we),
	     .wbm_adr_o			(s0_adr),
	     .wbm_dat_o			(s0_dat_o),
	     .wbm_sel_o			(s0_sel));



   wire [31:0] confbus1;
   wire [11:0] confbus_addr1;
   wire        confbus_we1;
   defparam busnode0.CONFADDR0 = 7'd0;
   defparam busnode0.CONFADDR1 = 7'd1;
   defparam busnode0.CONFADDR2 = 7'd2;
   defparam busnode0.BASEADDR = 5'd0;
   wb_bus_node busnode0(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s0_dat_i),
		   .wbs_ack_o		(s0_ack),
		   .wbs_rty_o		(s0_rty),
		   .wbs_err_o		(s0_err),
		   // Inputs
		   .wbs_stb_i		(s0_stb),
		   .wbs_cyc_i		(s0_cyc),
		   .wbs_cab_i		(s0_cab),
		   .wbs_we_i		(s0_we),
		   .wbs_adr_i		(s0_adr),
		   .wbs_dat_i		(s0_dat_o),
		   .wbs_sel_i		(s0_sel),

		     // Outputs
		     .wbm_stb_o		(m0_stb),
		     .wbm_cyc_o		(m0_cyc),
		     .wbm_cab_o		(m0_cab),
		     .wbm_we_o		(m0_we),
		     .wbm_adr_o		(m0_adr),
		     .wbm_dat_o		(m0_dat_o),
		     .wbm_sel_o		(m0_sel),
                     .wbm_size_o        (m0_size),
		     // Inputs
		     .wbm_dat_i		(m0_dat_i),
		     .wbm_ack_i		(m0_ack),
		     .wbm_rty_i		(m0_rty),
		     .wbm_err_i		(m0_err),
                     
                     .confbus_i         (confbus0),
                     .confbus_we_i         (confbus_we0),
                     .confbus_addr_i         (confbus_addr0),

                     .confbus_o         (confbus1),
                     .confbus_we_o         (confbus_we1),
                     .confbus_addr_o         (confbus_addr1)

);
   defparam    if1.THISNODEADDRESS = 5'd1;
   // Port 1 is connected to node at (0,0) nodeport is 3
   defparam    if1.NUM_DESTS = 11;
   defparam    if1.NEXTHOPNODES = { 5'd0, 5'd2, 5'd3, 5'd4, 5'd5, 5'd6, 5'd7, 5'd8, 5'd9, 5'd10, 5'd11};
   defparam    if1.NEXTHOPDESTS = { 4'b0001, 4'b1000, 4'b0010, 4'b0010, 4'b0010, 4'b0100, 4'b0100, 4'b0100, 4'b0010, 4'b0010, 4'b0010 };
   defparam    if1.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if1.ADDRESS = { 32'h00000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if1.DESTS = { 4'b0001, 5'd0, 4'b1000, 5'd2, 4'b0010, 5'd3, 4'b0010, 5'd4, 4'b0010, 5'd5, 4'b0100, 5'd6, 4'b0100, 5'd7, 4'b0100, 5'd8, 4'b0010, 5'd9, 4'b0010, 5'd10, 4'b0010, 5'd11 };

   wire [31:0] 		 m1_dat_i;
   wire [31:0] 		 m1_dat_o;
   wire [31:0] 		 m1_adr;
   wire [3:0] 		 m1_sel;
   wire 		 m1_ack;
   wire 		 m1_rty;
   wire 		 m1_err;
   wire 		 m1_we;
   wire 		 m1_cyc;
   wire 		 m1_stb;
   wire 		 m1_cab;
   wire [4:0]		 m1_size;

   wire [31:0] 		 s1_dat_i;
   wire [31:0] 		 s1_dat_o;
   wire [31:0] 		 s1_adr;
   wire [3:0] 		 s1_sel;
   wire 		 s1_ack;
   wire 		 s1_rty;
   wire 		 s1_err;
   wire 		 s1_we;
   wire 		 s1_cyc;
   wire 		 s1_stb;
   wire 		 s1_cab;
   wire [4:0]		 s1_size;

   wb_if if1(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port1_i),
	     .soclink_i			(port1_o),

	     // Slave port
	     .wbs_dat_o			(m1_dat_i),
	     .wbs_ack_o			(m1_ack),
	     .wbs_rty_o			(m1_rty),
	     .wbs_err_o			(m1_err),
	     .wbs_adr_i			(m1_adr),
	     .wbs_dat_i			(m1_dat_o),
	     .wbs_stb_i			(m1_stb),
	     .wbs_cyc_i			(m1_cyc),
	     .wbs_cab_i			(m1_cab),
	     .wbs_we_i			(m1_we),
	     .wbs_sel_i			(m1_sel),
             .wbs_size_i                (m1_size),

	     // Master port
	     .wbm_dat_i			(s1_dat_i),
	     .wbm_ack_i			(s1_ack),
	     .wbm_rty_i			(s1_rty),
	     .wbm_err_i			(s1_err),
	     .wbm_stb_o			(s1_stb),
	     .wbm_cyc_o			(s1_cyc),
	     .wbm_cab_o			(s1_cab),
	     .wbm_we_o			(s1_we),
	     .wbm_adr_o			(s1_adr),
	     .wbm_dat_o			(s1_dat_o),
	     .wbm_sel_o			(s1_sel));



   wire [31:0] confbus2;
   wire [11:0] confbus_addr2;
   wire        confbus_we2;
   defparam busnode1.CONFADDR0 = 7'd3;
   defparam busnode1.CONFADDR1 = 7'd4;
   defparam busnode1.CONFADDR2 = 7'd5;
   defparam busnode1.BASEADDR = 5'd1;
   wb_bus_node busnode1(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s1_dat_i),
		   .wbs_ack_o		(s1_ack),
		   .wbs_rty_o		(s1_rty),
		   .wbs_err_o		(s1_err),
		   // Inputs
		   .wbs_stb_i		(s1_stb),
		   .wbs_cyc_i		(s1_cyc),
		   .wbs_cab_i		(s1_cab),
		   .wbs_we_i		(s1_we),
		   .wbs_adr_i		(s1_adr),
		   .wbs_dat_i		(s1_dat_o),
		   .wbs_sel_i		(s1_sel),

		     // Outputs
		     .wbm_stb_o		(m1_stb),
		     .wbm_cyc_o		(m1_cyc),
		     .wbm_cab_o		(m1_cab),
		     .wbm_we_o		(m1_we),
		     .wbm_adr_o		(m1_adr),
		     .wbm_dat_o		(m1_dat_o),
		     .wbm_sel_o		(m1_sel),
                     .wbm_size_o        (m1_size),
		     // Inputs
		     .wbm_dat_i		(m1_dat_i),
		     .wbm_ack_i		(m1_ack),
		     .wbm_rty_i		(m1_rty),
		     .wbm_err_i		(m1_err),
                     
                     .confbus_i         (confbus1),
                     .confbus_we_i         (confbus_we1),
                     .confbus_addr_i         (confbus_addr1),

                     .confbus_o         (confbus2),
                     .confbus_we_o         (confbus_we2),
                     .confbus_addr_o         (confbus_addr2)

);
   defparam    if2.THISNODEADDRESS = 5'd2;
   // Port 2 is connected to node at (0,0) nodeport is 0
   defparam    if2.NUM_DESTS = 11;
   defparam    if2.NEXTHOPNODES = { 5'd0, 5'd1, 5'd3, 5'd4, 5'd5, 5'd6, 5'd7, 5'd8, 5'd9, 5'd10, 5'd11};
   defparam    if2.NEXTHOPDESTS = { 4'b0001, 4'b0010, 4'b0100, 4'b0100, 4'b0100, 4'b1000, 4'b1000, 4'b1000, 4'b0100, 4'b0100, 4'b0100 };
   defparam    if2.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if2.ADDRESS = { 32'h00000000, 32'h08000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if2.DESTS = { 4'b0001, 5'd0, 4'b0010, 5'd1, 4'b0100, 5'd3, 4'b0100, 5'd4, 4'b0100, 5'd5, 4'b1000, 5'd6, 4'b1000, 5'd7, 4'b1000, 5'd8, 4'b0100, 5'd9, 4'b0100, 5'd10, 4'b0100, 5'd11 };

   wire [31:0] 		 m2_dat_i;
   wire [31:0] 		 m2_dat_o;
   wire [31:0] 		 m2_adr;
   wire [3:0] 		 m2_sel;
   wire 		 m2_ack;
   wire 		 m2_rty;
   wire 		 m2_err;
   wire 		 m2_we;
   wire 		 m2_cyc;
   wire 		 m2_stb;
   wire 		 m2_cab;
   wire [4:0]		 m2_size;

   wire [31:0] 		 s2_dat_i;
   wire [31:0] 		 s2_dat_o;
   wire [31:0] 		 s2_adr;
   wire [3:0] 		 s2_sel;
   wire 		 s2_ack;
   wire 		 s2_rty;
   wire 		 s2_err;
   wire 		 s2_we;
   wire 		 s2_cyc;
   wire 		 s2_stb;
   wire 		 s2_cab;
   wire [4:0]		 s2_size;

   wb_if if2(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port2_i),
	     .soclink_i			(port2_o),

	     // Slave port
	     .wbs_dat_o			(m2_dat_i),
	     .wbs_ack_o			(m2_ack),
	     .wbs_rty_o			(m2_rty),
	     .wbs_err_o			(m2_err),
	     .wbs_adr_i			(m2_adr),
	     .wbs_dat_i			(m2_dat_o),
	     .wbs_stb_i			(m2_stb),
	     .wbs_cyc_i			(m2_cyc),
	     .wbs_cab_i			(m2_cab),
	     .wbs_we_i			(m2_we),
	     .wbs_sel_i			(m2_sel),
             .wbs_size_i                (m2_size),

	     // Master port
	     .wbm_dat_i			(s2_dat_i),
	     .wbm_ack_i			(s2_ack),
	     .wbm_rty_i			(s2_rty),
	     .wbm_err_i			(s2_err),
	     .wbm_stb_o			(s2_stb),
	     .wbm_cyc_o			(s2_cyc),
	     .wbm_cab_o			(s2_cab),
	     .wbm_we_o			(s2_we),
	     .wbm_adr_o			(s2_adr),
	     .wbm_dat_o			(s2_dat_o),
	     .wbm_sel_o			(s2_sel));



   wire [31:0] confbus3;
   wire [11:0] confbus_addr3;
   wire        confbus_we3;
   defparam busnode2.CONFADDR0 = 7'd6;
   defparam busnode2.CONFADDR1 = 7'd7;
   defparam busnode2.CONFADDR2 = 7'd8;
   defparam busnode2.BASEADDR = 5'd2;
   wb_bus_node busnode2(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s2_dat_i),
		   .wbs_ack_o		(s2_ack),
		   .wbs_rty_o		(s2_rty),
		   .wbs_err_o		(s2_err),
		   // Inputs
		   .wbs_stb_i		(s2_stb),
		   .wbs_cyc_i		(s2_cyc),
		   .wbs_cab_i		(s2_cab),
		   .wbs_we_i		(s2_we),
		   .wbs_adr_i		(s2_adr),
		   .wbs_dat_i		(s2_dat_o),
		   .wbs_sel_i		(s2_sel),

		     // Outputs
		     .wbm_stb_o		(m2_stb),
		     .wbm_cyc_o		(m2_cyc),
		     .wbm_cab_o		(m2_cab),
		     .wbm_we_o		(m2_we),
		     .wbm_adr_o		(m2_adr),
		     .wbm_dat_o		(m2_dat_o),
		     .wbm_sel_o		(m2_sel),
                     .wbm_size_o        (m2_size),
		     // Inputs
		     .wbm_dat_i		(m2_dat_i),
		     .wbm_ack_i		(m2_ack),
		     .wbm_rty_i		(m2_rty),
		     .wbm_err_i		(m2_err),
                     
                     .confbus_i         (confbus2),
                     .confbus_we_i         (confbus_we2),
                     .confbus_addr_i         (confbus_addr2),

                     .confbus_o         (confbus3),
                     .confbus_we_o         (confbus_we3),
                     .confbus_addr_o         (confbus_addr3)

);
   defparam    if3.THISNODEADDRESS = 5'd3;
   // Port 3 is connected to node at (1,0) nodeport is 3
   defparam    if3.NUM_DESTS = 11;
   defparam    if3.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd4, 5'd5, 5'd6, 5'd7, 5'd8, 5'd9, 5'd10, 5'd11};
   defparam    if3.NEXTHOPDESTS = { 4'b0001, 4'b0001, 4'b0001, 4'b0010, 4'b1000, 4'b0001, 4'b0001, 4'b0001, 4'b0100, 4'b0100, 4'b0100 };
   defparam    if3.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if3.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if3.DESTS = { 4'b0001, 5'd0, 4'b0001, 5'd1, 4'b0001, 5'd2, 4'b0010, 5'd4, 4'b1000, 5'd5, 4'b0001, 5'd6, 4'b0001, 5'd7, 4'b0001, 5'd8, 4'b0100, 5'd9, 4'b0100, 5'd10, 4'b0100, 5'd11 };

   wire [31:0] 		 m3_dat_i;
   wire [31:0] 		 m3_dat_o;
   wire [31:0] 		 m3_adr;
   wire [3:0] 		 m3_sel;
   wire 		 m3_ack;
   wire 		 m3_rty;
   wire 		 m3_err;
   wire 		 m3_we;
   wire 		 m3_cyc;
   wire 		 m3_stb;
   wire 		 m3_cab;
   wire [4:0]		 m3_size;

   wire [31:0] 		 s3_dat_i;
   wire [31:0] 		 s3_dat_o;
   wire [31:0] 		 s3_adr;
   wire [3:0] 		 s3_sel;
   wire 		 s3_ack;
   wire 		 s3_rty;
   wire 		 s3_err;
   wire 		 s3_we;
   wire 		 s3_cyc;
   wire 		 s3_stb;
   wire 		 s3_cab;
   wire [4:0]		 s3_size;

   wb_if if3(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port3_i),
	     .soclink_i			(port3_o),

	     // Slave port
	     .wbs_dat_o			(m3_dat_i),
	     .wbs_ack_o			(m3_ack),
	     .wbs_rty_o			(m3_rty),
	     .wbs_err_o			(m3_err),
	     .wbs_adr_i			(m3_adr),
	     .wbs_dat_i			(m3_dat_o),
	     .wbs_stb_i			(m3_stb),
	     .wbs_cyc_i			(m3_cyc),
	     .wbs_cab_i			(m3_cab),
	     .wbs_we_i			(m3_we),
	     .wbs_sel_i			(m3_sel),
             .wbs_size_i                (m3_size),

	     // Master port
	     .wbm_dat_i			(s3_dat_i),
	     .wbm_ack_i			(s3_ack),
	     .wbm_rty_i			(s3_rty),
	     .wbm_err_i			(s3_err),
	     .wbm_stb_o			(s3_stb),
	     .wbm_cyc_o			(s3_cyc),
	     .wbm_cab_o			(s3_cab),
	     .wbm_we_o			(s3_we),
	     .wbm_adr_o			(s3_adr),
	     .wbm_dat_o			(s3_dat_o),
	     .wbm_sel_o			(s3_sel));



   wire [31:0] confbus4;
   wire [11:0] confbus_addr4;
   wire        confbus_we4;
   defparam busnode3.CONFADDR0 = 7'd9;
   defparam busnode3.CONFADDR1 = 7'd10;
   defparam busnode3.CONFADDR2 = 7'd11;
   defparam busnode3.BASEADDR = 5'd3;
   wb_bus_node busnode3(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s3_dat_i),
		   .wbs_ack_o		(s3_ack),
		   .wbs_rty_o		(s3_rty),
		   .wbs_err_o		(s3_err),
		   // Inputs
		   .wbs_stb_i		(s3_stb),
		   .wbs_cyc_i		(s3_cyc),
		   .wbs_cab_i		(s3_cab),
		   .wbs_we_i		(s3_we),
		   .wbs_adr_i		(s3_adr),
		   .wbs_dat_i		(s3_dat_o),
		   .wbs_sel_i		(s3_sel),

		     // Outputs
		     .wbm_stb_o		(m3_stb),
		     .wbm_cyc_o		(m3_cyc),
		     .wbm_cab_o		(m3_cab),
		     .wbm_we_o		(m3_we),
		     .wbm_adr_o		(m3_adr),
		     .wbm_dat_o		(m3_dat_o),
		     .wbm_sel_o		(m3_sel),
                     .wbm_size_o        (m3_size),
		     // Inputs
		     .wbm_dat_i		(m3_dat_i),
		     .wbm_ack_i		(m3_ack),
		     .wbm_rty_i		(m3_rty),
		     .wbm_err_i		(m3_err),
                     
                     .confbus_i         (confbus3),
                     .confbus_we_i         (confbus_we3),
                     .confbus_addr_i         (confbus_addr3),

                     .confbus_o         (confbus4),
                     .confbus_we_o         (confbus_we4),
                     .confbus_addr_o         (confbus_addr4)

);
   defparam    if4.THISNODEADDRESS = 5'd4;
   // Port 4 is connected to node at (1,0) nodeport is 2
   defparam    if4.NUM_DESTS = 11;
   defparam    if4.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd3, 5'd5, 5'd6, 5'd7, 5'd8, 5'd9, 5'd10, 5'd11};
   defparam    if4.NEXTHOPDESTS = { 4'b0001, 4'b0001, 4'b0001, 4'b0010, 4'b1000, 4'b0001, 4'b0001, 4'b0001, 4'b0100, 4'b0100, 4'b0100 };
   defparam    if4.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if4.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h18000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if4.DESTS = { 4'b0001, 5'd0, 4'b0001, 5'd1, 4'b0001, 5'd2, 4'b0010, 5'd3, 4'b1000, 5'd5, 4'b0001, 5'd6, 4'b0001, 5'd7, 4'b0001, 5'd8, 4'b0100, 5'd9, 4'b0100, 5'd10, 4'b0100, 5'd11 };

   wire [31:0] 		 m4_dat_i;
   wire [31:0] 		 m4_dat_o;
   wire [31:0] 		 m4_adr;
   wire [3:0] 		 m4_sel;
   wire 		 m4_ack;
   wire 		 m4_rty;
   wire 		 m4_err;
   wire 		 m4_we;
   wire 		 m4_cyc;
   wire 		 m4_stb;
   wire 		 m4_cab;
   wire [4:0]		 m4_size;

   wire [31:0] 		 s4_dat_i;
   wire [31:0] 		 s4_dat_o;
   wire [31:0] 		 s4_adr;
   wire [3:0] 		 s4_sel;
   wire 		 s4_ack;
   wire 		 s4_rty;
   wire 		 s4_err;
   wire 		 s4_we;
   wire 		 s4_cyc;
   wire 		 s4_stb;
   wire 		 s4_cab;
   wire [4:0]		 s4_size;

   wb_if if4(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port4_i),
	     .soclink_i			(port4_o),

	     // Slave port
	     .wbs_dat_o			(m4_dat_i),
	     .wbs_ack_o			(m4_ack),
	     .wbs_rty_o			(m4_rty),
	     .wbs_err_o			(m4_err),
	     .wbs_adr_i			(m4_adr),
	     .wbs_dat_i			(m4_dat_o),
	     .wbs_stb_i			(m4_stb),
	     .wbs_cyc_i			(m4_cyc),
	     .wbs_cab_i			(m4_cab),
	     .wbs_we_i			(m4_we),
	     .wbs_sel_i			(m4_sel),
             .wbs_size_i                (m4_size),

	     // Master port
	     .wbm_dat_i			(s4_dat_i),
	     .wbm_ack_i			(s4_ack),
	     .wbm_rty_i			(s4_rty),
	     .wbm_err_i			(s4_err),
	     .wbm_stb_o			(s4_stb),
	     .wbm_cyc_o			(s4_cyc),
	     .wbm_cab_o			(s4_cab),
	     .wbm_we_o			(s4_we),
	     .wbm_adr_o			(s4_adr),
	     .wbm_dat_o			(s4_dat_o),
	     .wbm_sel_o			(s4_sel));



   wire [31:0] confbus5;
   wire [11:0] confbus_addr5;
   wire        confbus_we5;
   defparam busnode4.CONFADDR0 = 7'd12;
   defparam busnode4.CONFADDR1 = 7'd13;
   defparam busnode4.CONFADDR2 = 7'd14;
   defparam busnode4.BASEADDR = 5'd4;
   wb_bus_node busnode4(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s4_dat_i),
		   .wbs_ack_o		(s4_ack),
		   .wbs_rty_o		(s4_rty),
		   .wbs_err_o		(s4_err),
		   // Inputs
		   .wbs_stb_i		(s4_stb),
		   .wbs_cyc_i		(s4_cyc),
		   .wbs_cab_i		(s4_cab),
		   .wbs_we_i		(s4_we),
		   .wbs_adr_i		(s4_adr),
		   .wbs_dat_i		(s4_dat_o),
		   .wbs_sel_i		(s4_sel),

		     // Outputs
		     .wbm_stb_o		(m4_stb),
		     .wbm_cyc_o		(m4_cyc),
		     .wbm_cab_o		(m4_cab),
		     .wbm_we_o		(m4_we),
		     .wbm_adr_o		(m4_adr),
		     .wbm_dat_o		(m4_dat_o),
		     .wbm_sel_o		(m4_sel),
                     .wbm_size_o        (m4_size),
		     // Inputs
		     .wbm_dat_i		(m4_dat_i),
		     .wbm_ack_i		(m4_ack),
		     .wbm_rty_i		(m4_rty),
		     .wbm_err_i		(m4_err),
                     
                     .confbus_i         (confbus4),
                     .confbus_we_i         (confbus_we4),
                     .confbus_addr_i         (confbus_addr4),

                     .confbus_o         (confbus5),
                     .confbus_we_o         (confbus_we5),
                     .confbus_addr_o         (confbus_addr5)

);
   defparam    if5.THISNODEADDRESS = 5'd5;
   // Port 5 is connected to node at (1,0) nodeport is 0
   defparam    if5.NUM_DESTS = 11;
   defparam    if5.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd3, 5'd4, 5'd6, 5'd7, 5'd8, 5'd9, 5'd10, 5'd11};
   defparam    if5.NEXTHOPDESTS = { 4'b0001, 4'b0001, 4'b0001, 4'b0010, 4'b0100, 4'b0001, 4'b0001, 4'b0001, 4'b1000, 4'b1000, 4'b1000 };
   defparam    if5.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if5.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if5.DESTS = { 4'b0001, 5'd0, 4'b0001, 5'd1, 4'b0001, 5'd2, 4'b0010, 5'd3, 4'b0100, 5'd4, 4'b0001, 5'd6, 4'b0001, 5'd7, 4'b0001, 5'd8, 4'b1000, 5'd9, 4'b1000, 5'd10, 4'b1000, 5'd11 };

   wire [31:0] 		 m5_dat_i;
   wire [31:0] 		 m5_dat_o;
   wire [31:0] 		 m5_adr;
   wire [3:0] 		 m5_sel;
   wire 		 m5_ack;
   wire 		 m5_rty;
   wire 		 m5_err;
   wire 		 m5_we;
   wire 		 m5_cyc;
   wire 		 m5_stb;
   wire 		 m5_cab;
   wire [4:0]		 m5_size;

   wire [31:0] 		 s5_dat_i;
   wire [31:0] 		 s5_dat_o;
   wire [31:0] 		 s5_adr;
   wire [3:0] 		 s5_sel;
   wire 		 s5_ack;
   wire 		 s5_rty;
   wire 		 s5_err;
   wire 		 s5_we;
   wire 		 s5_cyc;
   wire 		 s5_stb;
   wire 		 s5_cab;
   wire [4:0]		 s5_size;

   wb_if if5(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port5_i),
	     .soclink_i			(port5_o),

	     // Slave port
	     .wbs_dat_o			(m5_dat_i),
	     .wbs_ack_o			(m5_ack),
	     .wbs_rty_o			(m5_rty),
	     .wbs_err_o			(m5_err),
	     .wbs_adr_i			(m5_adr),
	     .wbs_dat_i			(m5_dat_o),
	     .wbs_stb_i			(m5_stb),
	     .wbs_cyc_i			(m5_cyc),
	     .wbs_cab_i			(m5_cab),
	     .wbs_we_i			(m5_we),
	     .wbs_sel_i			(m5_sel),
             .wbs_size_i                (m5_size),

	     // Master port
	     .wbm_dat_i			(s5_dat_i),
	     .wbm_ack_i			(s5_ack),
	     .wbm_rty_i			(s5_rty),
	     .wbm_err_i			(s5_err),
	     .wbm_stb_o			(s5_stb),
	     .wbm_cyc_o			(s5_cyc),
	     .wbm_cab_o			(s5_cab),
	     .wbm_we_o			(s5_we),
	     .wbm_adr_o			(s5_adr),
	     .wbm_dat_o			(s5_dat_o),
	     .wbm_sel_o			(s5_sel));



   wire [31:0] confbus6;
   wire [11:0] confbus_addr6;
   wire        confbus_we6;
   defparam busnode5.CONFADDR0 = 7'd15;
   defparam busnode5.CONFADDR1 = 7'd16;
   defparam busnode5.CONFADDR2 = 7'd17;
   defparam busnode5.BASEADDR = 5'd5;
   wb_bus_node busnode5(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s5_dat_i),
		   .wbs_ack_o		(s5_ack),
		   .wbs_rty_o		(s5_rty),
		   .wbs_err_o		(s5_err),
		   // Inputs
		   .wbs_stb_i		(s5_stb),
		   .wbs_cyc_i		(s5_cyc),
		   .wbs_cab_i		(s5_cab),
		   .wbs_we_i		(s5_we),
		   .wbs_adr_i		(s5_adr),
		   .wbs_dat_i		(s5_dat_o),
		   .wbs_sel_i		(s5_sel),

		     // Outputs
		     .wbm_stb_o		(m5_stb),
		     .wbm_cyc_o		(m5_cyc),
		     .wbm_cab_o		(m5_cab),
		     .wbm_we_o		(m5_we),
		     .wbm_adr_o		(m5_adr),
		     .wbm_dat_o		(m5_dat_o),
		     .wbm_sel_o		(m5_sel),
                     .wbm_size_o        (m5_size),
		     // Inputs
		     .wbm_dat_i		(m5_dat_i),
		     .wbm_ack_i		(m5_ack),
		     .wbm_rty_i		(m5_rty),
		     .wbm_err_i		(m5_err),
                     
                     .confbus_i         (confbus5),
                     .confbus_we_i         (confbus_we5),
                     .confbus_addr_i         (confbus_addr5),

                     .confbus_o         (confbus6),
                     .confbus_we_o         (confbus_we6),
                     .confbus_addr_o         (confbus_addr6)

);
   defparam    if6.THISNODEADDRESS = 5'd6;
   // Port 6 is connected to node at (0,1) nodeport is 4
   defparam    if6.NUM_DESTS = 11;
   defparam    if6.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd3, 5'd4, 5'd5, 5'd7, 5'd8, 5'd9, 5'd10, 5'd11};
   defparam    if6.NEXTHOPDESTS = { 4'b0001, 4'b0001, 4'b0001, 4'b0010, 4'b0010, 4'b0010, 4'b0100, 4'b1000, 4'b0010, 4'b0010, 4'b0010 };
   defparam    if6.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if6.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if6.DESTS = { 4'b0001, 5'd0, 4'b0001, 5'd1, 4'b0001, 5'd2, 4'b0010, 5'd3, 4'b0010, 5'd4, 4'b0010, 5'd5, 4'b0100, 5'd7, 4'b1000, 5'd8, 4'b0010, 5'd9, 4'b0010, 5'd10, 4'b0010, 5'd11 };

   wire [31:0] 		 m6_dat_i;
   wire [31:0] 		 m6_dat_o;
   wire [31:0] 		 m6_adr;
   wire [3:0] 		 m6_sel;
   wire 		 m6_ack;
   wire 		 m6_rty;
   wire 		 m6_err;
   wire 		 m6_we;
   wire 		 m6_cyc;
   wire 		 m6_stb;
   wire 		 m6_cab;
   wire [4:0]		 m6_size;

   wire [31:0] 		 s6_dat_i;
   wire [31:0] 		 s6_dat_o;
   wire [31:0] 		 s6_adr;
   wire [3:0] 		 s6_sel;
   wire 		 s6_ack;
   wire 		 s6_rty;
   wire 		 s6_err;
   wire 		 s6_we;
   wire 		 s6_cyc;
   wire 		 s6_stb;
   wire 		 s6_cab;
   wire [4:0]		 s6_size;

   wb_if if6(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port6_i),
	     .soclink_i			(port6_o),

	     // Slave port
	     .wbs_dat_o			(m6_dat_i),
	     .wbs_ack_o			(m6_ack),
	     .wbs_rty_o			(m6_rty),
	     .wbs_err_o			(m6_err),
	     .wbs_adr_i			(m6_adr),
	     .wbs_dat_i			(m6_dat_o),
	     .wbs_stb_i			(m6_stb),
	     .wbs_cyc_i			(m6_cyc),
	     .wbs_cab_i			(m6_cab),
	     .wbs_we_i			(m6_we),
	     .wbs_sel_i			(m6_sel),
             .wbs_size_i                (m6_size),

	     // Master port
	     .wbm_dat_i			(s6_dat_i),
	     .wbm_ack_i			(s6_ack),
	     .wbm_rty_i			(s6_rty),
	     .wbm_err_i			(s6_err),
	     .wbm_stb_o			(s6_stb),
	     .wbm_cyc_o			(s6_cyc),
	     .wbm_cab_o			(s6_cab),
	     .wbm_we_o			(s6_we),
	     .wbm_adr_o			(s6_adr),
	     .wbm_dat_o			(s6_dat_o),
	     .wbm_sel_o			(s6_sel));



   wire [31:0] confbus7;
   wire [11:0] confbus_addr7;
   wire        confbus_we7;
   defparam busnode6.CONFADDR0 = 7'd18;
   defparam busnode6.CONFADDR1 = 7'd19;
   defparam busnode6.CONFADDR2 = 7'd20;
   defparam busnode6.BASEADDR = 5'd6;
   wb_bus_node busnode6(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s6_dat_i),
		   .wbs_ack_o		(s6_ack),
		   .wbs_rty_o		(s6_rty),
		   .wbs_err_o		(s6_err),
		   // Inputs
		   .wbs_stb_i		(s6_stb),
		   .wbs_cyc_i		(s6_cyc),
		   .wbs_cab_i		(s6_cab),
		   .wbs_we_i		(s6_we),
		   .wbs_adr_i		(s6_adr),
		   .wbs_dat_i		(s6_dat_o),
		   .wbs_sel_i		(s6_sel),

		     // Outputs
		     .wbm_stb_o		(m6_stb),
		     .wbm_cyc_o		(m6_cyc),
		     .wbm_cab_o		(m6_cab),
		     .wbm_we_o		(m6_we),
		     .wbm_adr_o		(m6_adr),
		     .wbm_dat_o		(m6_dat_o),
		     .wbm_sel_o		(m6_sel),
                     .wbm_size_o        (m6_size),
		     // Inputs
		     .wbm_dat_i		(m6_dat_i),
		     .wbm_ack_i		(m6_ack),
		     .wbm_rty_i		(m6_rty),
		     .wbm_err_i		(m6_err),
                     
                     .confbus_i         (confbus6),
                     .confbus_we_i         (confbus_we6),
                     .confbus_addr_i         (confbus_addr6),

                     .confbus_o         (confbus7),
                     .confbus_we_o         (confbus_we7),
                     .confbus_addr_o         (confbus_addr7)

);
   defparam    if7.THISNODEADDRESS = 5'd7;
   // Port 7 is connected to node at (0,1) nodeport is 1
   defparam    if7.NUM_DESTS = 11;
   defparam    if7.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd3, 5'd4, 5'd5, 5'd6, 5'd8, 5'd9, 5'd10, 5'd11};
   defparam    if7.NEXTHOPDESTS = { 4'b0010, 4'b0010, 4'b0010, 4'b0100, 4'b0100, 4'b0100, 4'b0001, 4'b1000, 4'b0100, 4'b0100, 4'b0100 };
   defparam    if7.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if7.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h40000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if7.DESTS = { 4'b0010, 5'd0, 4'b0010, 5'd1, 4'b0010, 5'd2, 4'b0100, 5'd3, 4'b0100, 5'd4, 4'b0100, 5'd5, 4'b0001, 5'd6, 4'b1000, 5'd8, 4'b0100, 5'd9, 4'b0100, 5'd10, 4'b0100, 5'd11 };

   wire [31:0] 		 m7_dat_i;
   wire [31:0] 		 m7_dat_o;
   wire [31:0] 		 m7_adr;
   wire [3:0] 		 m7_sel;
   wire 		 m7_ack;
   wire 		 m7_rty;
   wire 		 m7_err;
   wire 		 m7_we;
   wire 		 m7_cyc;
   wire 		 m7_stb;
   wire 		 m7_cab;
   wire [4:0]		 m7_size;

   wire [31:0] 		 s7_dat_i;
   wire [31:0] 		 s7_dat_o;
   wire [31:0] 		 s7_adr;
   wire [3:0] 		 s7_sel;
   wire 		 s7_ack;
   wire 		 s7_rty;
   wire 		 s7_err;
   wire 		 s7_we;
   wire 		 s7_cyc;
   wire 		 s7_stb;
   wire 		 s7_cab;
   wire [4:0]		 s7_size;

   wb_if if7(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port7_i),
	     .soclink_i			(port7_o),

	     // Slave port
	     .wbs_dat_o			(m7_dat_i),
	     .wbs_ack_o			(m7_ack),
	     .wbs_rty_o			(m7_rty),
	     .wbs_err_o			(m7_err),
	     .wbs_adr_i			(m7_adr),
	     .wbs_dat_i			(m7_dat_o),
	     .wbs_stb_i			(m7_stb),
	     .wbs_cyc_i			(m7_cyc),
	     .wbs_cab_i			(m7_cab),
	     .wbs_we_i			(m7_we),
	     .wbs_sel_i			(m7_sel),
             .wbs_size_i                (m7_size),

	     // Master port
	     .wbm_dat_i			(s7_dat_i),
	     .wbm_ack_i			(s7_ack),
	     .wbm_rty_i			(s7_rty),
	     .wbm_err_i			(s7_err),
	     .wbm_stb_o			(s7_stb),
	     .wbm_cyc_o			(s7_cyc),
	     .wbm_cab_o			(s7_cab),
	     .wbm_we_o			(s7_we),
	     .wbm_adr_o			(s7_adr),
	     .wbm_dat_o			(s7_dat_o),
	     .wbm_sel_o			(s7_sel));



   wire [31:0] confbus8;
   wire [11:0] confbus_addr8;
   wire        confbus_we8;
   defparam busnode7.CONFADDR0 = 7'd21;
   defparam busnode7.CONFADDR1 = 7'd22;
   defparam busnode7.CONFADDR2 = 7'd23;
   defparam busnode7.BASEADDR = 5'd7;
   wb_bus_node busnode7(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s7_dat_i),
		   .wbs_ack_o		(s7_ack),
		   .wbs_rty_o		(s7_rty),
		   .wbs_err_o		(s7_err),
		   // Inputs
		   .wbs_stb_i		(s7_stb),
		   .wbs_cyc_i		(s7_cyc),
		   .wbs_cab_i		(s7_cab),
		   .wbs_we_i		(s7_we),
		   .wbs_adr_i		(s7_adr),
		   .wbs_dat_i		(s7_dat_o),
		   .wbs_sel_i		(s7_sel),

		     // Outputs
		     .wbm_stb_o		(m7_stb),
		     .wbm_cyc_o		(m7_cyc),
		     .wbm_cab_o		(m7_cab),
		     .wbm_we_o		(m7_we),
		     .wbm_adr_o		(m7_adr),
		     .wbm_dat_o		(m7_dat_o),
		     .wbm_sel_o		(m7_sel),
                     .wbm_size_o        (m7_size),
		     // Inputs
		     .wbm_dat_i		(m7_dat_i),
		     .wbm_ack_i		(m7_ack),
		     .wbm_rty_i		(m7_rty),
		     .wbm_err_i		(m7_err),
                     
                     .confbus_i         (confbus7),
                     .confbus_we_i         (confbus_we7),
                     .confbus_addr_i         (confbus_addr7),

                     .confbus_o         (confbus8),
                     .confbus_we_o         (confbus_we8),
                     .confbus_addr_o         (confbus_addr8)

);
   defparam    if8.THISNODEADDRESS = 5'd8;
   // Port 8 is connected to node at (0,1) nodeport is 0
   defparam    if8.NUM_DESTS = 11;
   defparam    if8.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd3, 5'd4, 5'd5, 5'd6, 5'd7, 5'd9, 5'd10, 5'd11};
   defparam    if8.NEXTHOPDESTS = { 4'b0010, 4'b0010, 4'b0010, 4'b0100, 4'b0100, 4'b0100, 4'b0001, 4'b1000, 4'b0100, 4'b0100, 4'b0100 };
   defparam    if8.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if8.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h48000000, 32'h50000000, 32'h58000000 };
   defparam    if8.DESTS = { 4'b0010, 5'd0, 4'b0010, 5'd1, 4'b0010, 5'd2, 4'b0100, 5'd3, 4'b0100, 5'd4, 4'b0100, 5'd5, 4'b0001, 5'd6, 4'b1000, 5'd7, 4'b0100, 5'd9, 4'b0100, 5'd10, 4'b0100, 5'd11 };

   wire [31:0] 		 m8_dat_i;
   wire [31:0] 		 m8_dat_o;
   wire [31:0] 		 m8_adr;
   wire [3:0] 		 m8_sel;
   wire 		 m8_ack;
   wire 		 m8_rty;
   wire 		 m8_err;
   wire 		 m8_we;
   wire 		 m8_cyc;
   wire 		 m8_stb;
   wire 		 m8_cab;
   wire [4:0]		 m8_size;

   wire [31:0] 		 s8_dat_i;
   wire [31:0] 		 s8_dat_o;
   wire [31:0] 		 s8_adr;
   wire [3:0] 		 s8_sel;
   wire 		 s8_ack;
   wire 		 s8_rty;
   wire 		 s8_err;
   wire 		 s8_we;
   wire 		 s8_cyc;
   wire 		 s8_stb;
   wire 		 s8_cab;
   wire [4:0]		 s8_size;

   wb_if if8(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port8_i),
	     .soclink_i			(port8_o),

	     // Slave port
	     .wbs_dat_o			(m8_dat_i),
	     .wbs_ack_o			(m8_ack),
	     .wbs_rty_o			(m8_rty),
	     .wbs_err_o			(m8_err),
	     .wbs_adr_i			(m8_adr),
	     .wbs_dat_i			(m8_dat_o),
	     .wbs_stb_i			(m8_stb),
	     .wbs_cyc_i			(m8_cyc),
	     .wbs_cab_i			(m8_cab),
	     .wbs_we_i			(m8_we),
	     .wbs_sel_i			(m8_sel),
             .wbs_size_i                (m8_size),

	     // Master port
	     .wbm_dat_i			(s8_dat_i),
	     .wbm_ack_i			(s8_ack),
	     .wbm_rty_i			(s8_rty),
	     .wbm_err_i			(s8_err),
	     .wbm_stb_o			(s8_stb),
	     .wbm_cyc_o			(s8_cyc),
	     .wbm_cab_o			(s8_cab),
	     .wbm_we_o			(s8_we),
	     .wbm_adr_o			(s8_adr),
	     .wbm_dat_o			(s8_dat_o),
	     .wbm_sel_o			(s8_sel));



   wire [31:0] confbus9;
   wire [11:0] confbus_addr9;
   wire        confbus_we9;
   defparam busnode8.CONFADDR0 = 7'd24;
   defparam busnode8.CONFADDR1 = 7'd25;
   defparam busnode8.CONFADDR2 = 7'd26;
   defparam busnode8.BASEADDR = 5'd8;
   wb_bus_node busnode8(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s8_dat_i),
		   .wbs_ack_o		(s8_ack),
		   .wbs_rty_o		(s8_rty),
		   .wbs_err_o		(s8_err),
		   // Inputs
		   .wbs_stb_i		(s8_stb),
		   .wbs_cyc_i		(s8_cyc),
		   .wbs_cab_i		(s8_cab),
		   .wbs_we_i		(s8_we),
		   .wbs_adr_i		(s8_adr),
		   .wbs_dat_i		(s8_dat_o),
		   .wbs_sel_i		(s8_sel),

		     // Outputs
		     .wbm_stb_o		(m8_stb),
		     .wbm_cyc_o		(m8_cyc),
		     .wbm_cab_o		(m8_cab),
		     .wbm_we_o		(m8_we),
		     .wbm_adr_o		(m8_adr),
		     .wbm_dat_o		(m8_dat_o),
		     .wbm_sel_o		(m8_sel),
                     .wbm_size_o        (m8_size),
		     // Inputs
		     .wbm_dat_i		(m8_dat_i),
		     .wbm_ack_i		(m8_ack),
		     .wbm_rty_i		(m8_rty),
		     .wbm_err_i		(m8_err),
                     
                     .confbus_i         (confbus8),
                     .confbus_we_i         (confbus_we8),
                     .confbus_addr_i         (confbus_addr8),

                     .confbus_o         (confbus9),
                     .confbus_we_o         (confbus_we9),
                     .confbus_addr_o         (confbus_addr9)

);
   defparam    if9.THISNODEADDRESS = 5'd9;
   // Port 9 is connected to node at (1,1) nodeport is 1
   defparam    if9.NUM_DESTS = 11;
   defparam    if9.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd3, 5'd4, 5'd5, 5'd6, 5'd7, 5'd8, 5'd10, 5'd11};
   defparam    if9.NEXTHOPDESTS = { 4'b0001, 4'b0001, 4'b0001, 4'b0010, 4'b0010, 4'b0010, 4'b0001, 4'b0001, 4'b0001, 4'b0100, 4'b1000 };
   defparam    if9.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if9.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h50000000, 32'h58000000 };
   defparam    if9.DESTS = { 4'b0001, 5'd0, 4'b0001, 5'd1, 4'b0001, 5'd2, 4'b0010, 5'd3, 4'b0010, 5'd4, 4'b0010, 5'd5, 4'b0001, 5'd6, 4'b0001, 5'd7, 4'b0001, 5'd8, 4'b0100, 5'd10, 4'b1000, 5'd11 };

   wire [31:0] 		 m9_dat_i;
   wire [31:0] 		 m9_dat_o;
   wire [31:0] 		 m9_adr;
   wire [3:0] 		 m9_sel;
   wire 		 m9_ack;
   wire 		 m9_rty;
   wire 		 m9_err;
   wire 		 m9_we;
   wire 		 m9_cyc;
   wire 		 m9_stb;
   wire 		 m9_cab;
   wire [4:0]		 m9_size;

   wire [31:0] 		 s9_dat_i;
   wire [31:0] 		 s9_dat_o;
   wire [31:0] 		 s9_adr;
   wire [3:0] 		 s9_sel;
   wire 		 s9_ack;
   wire 		 s9_rty;
   wire 		 s9_err;
   wire 		 s9_we;
   wire 		 s9_cyc;
   wire 		 s9_stb;
   wire 		 s9_cab;
   wire [4:0]		 s9_size;

   wb_if if9(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port9_i),
	     .soclink_i			(port9_o),

	     // Slave port
	     .wbs_dat_o			(m9_dat_i),
	     .wbs_ack_o			(m9_ack),
	     .wbs_rty_o			(m9_rty),
	     .wbs_err_o			(m9_err),
	     .wbs_adr_i			(m9_adr),
	     .wbs_dat_i			(m9_dat_o),
	     .wbs_stb_i			(m9_stb),
	     .wbs_cyc_i			(m9_cyc),
	     .wbs_cab_i			(m9_cab),
	     .wbs_we_i			(m9_we),
	     .wbs_sel_i			(m9_sel),
             .wbs_size_i                (m9_size),

	     // Master port
	     .wbm_dat_i			(s9_dat_i),
	     .wbm_ack_i			(s9_ack),
	     .wbm_rty_i			(s9_rty),
	     .wbm_err_i			(s9_err),
	     .wbm_stb_o			(s9_stb),
	     .wbm_cyc_o			(s9_cyc),
	     .wbm_cab_o			(s9_cab),
	     .wbm_we_o			(s9_we),
	     .wbm_adr_o			(s9_adr),
	     .wbm_dat_o			(s9_dat_o),
	     .wbm_sel_o			(s9_sel));



   wire [31:0] confbus10;
   wire [11:0] confbus_addr10;
   wire        confbus_we10;
   defparam busnode9.CONFADDR0 = 7'd27;
   defparam busnode9.CONFADDR1 = 7'd28;
   defparam busnode9.CONFADDR2 = 7'd29;
   defparam busnode9.BASEADDR = 5'd9;
   wb_bus_node busnode9(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s9_dat_i),
		   .wbs_ack_o		(s9_ack),
		   .wbs_rty_o		(s9_rty),
		   .wbs_err_o		(s9_err),
		   // Inputs
		   .wbs_stb_i		(s9_stb),
		   .wbs_cyc_i		(s9_cyc),
		   .wbs_cab_i		(s9_cab),
		   .wbs_we_i		(s9_we),
		   .wbs_adr_i		(s9_adr),
		   .wbs_dat_i		(s9_dat_o),
		   .wbs_sel_i		(s9_sel),

		     // Outputs
		     .wbm_stb_o		(m9_stb),
		     .wbm_cyc_o		(m9_cyc),
		     .wbm_cab_o		(m9_cab),
		     .wbm_we_o		(m9_we),
		     .wbm_adr_o		(m9_adr),
		     .wbm_dat_o		(m9_dat_o),
		     .wbm_sel_o		(m9_sel),
                     .wbm_size_o        (m9_size),
		     // Inputs
		     .wbm_dat_i		(m9_dat_i),
		     .wbm_ack_i		(m9_ack),
		     .wbm_rty_i		(m9_rty),
		     .wbm_err_i		(m9_err),
                     
                     .confbus_i         (confbus9),
                     .confbus_we_i         (confbus_we9),
                     .confbus_addr_i         (confbus_addr9),

                     .confbus_o         (confbus10),
                     .confbus_we_o         (confbus_we10),
                     .confbus_addr_o         (confbus_addr10)

);
   defparam    if10.THISNODEADDRESS = 5'd10;
   // Port 10 is connected to node at (1,1) nodeport is 2
   defparam    if10.NUM_DESTS = 11;
   defparam    if10.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd3, 5'd4, 5'd5, 5'd6, 5'd7, 5'd8, 5'd9, 5'd11};
   defparam    if10.NEXTHOPDESTS = { 4'b0001, 4'b0001, 4'b0001, 4'b0010, 4'b0010, 4'b0010, 4'b0001, 4'b0001, 4'b0001, 4'b0100, 4'b1000 };
   defparam    if10.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if10.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h58000000 };
   defparam    if10.DESTS = { 4'b0001, 5'd0, 4'b0001, 5'd1, 4'b0001, 5'd2, 4'b0010, 5'd3, 4'b0010, 5'd4, 4'b0010, 5'd5, 4'b0001, 5'd6, 4'b0001, 5'd7, 4'b0001, 5'd8, 4'b0100, 5'd9, 4'b1000, 5'd11 };

   wire [31:0] 		 m10_dat_i;
   wire [31:0] 		 m10_dat_o;
   wire [31:0] 		 m10_adr;
   wire [3:0] 		 m10_sel;
   wire 		 m10_ack;
   wire 		 m10_rty;
   wire 		 m10_err;
   wire 		 m10_we;
   wire 		 m10_cyc;
   wire 		 m10_stb;
   wire 		 m10_cab;
   wire [4:0]		 m10_size;

   wire [31:0] 		 s10_dat_i;
   wire [31:0] 		 s10_dat_o;
   wire [31:0] 		 s10_adr;
   wire [3:0] 		 s10_sel;
   wire 		 s10_ack;
   wire 		 s10_rty;
   wire 		 s10_err;
   wire 		 s10_we;
   wire 		 s10_cyc;
   wire 		 s10_stb;
   wire 		 s10_cab;
   wire [4:0]		 s10_size;

   wb_if if10(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port10_i),
	     .soclink_i			(port10_o),

	     // Slave port
	     .wbs_dat_o			(m10_dat_i),
	     .wbs_ack_o			(m10_ack),
	     .wbs_rty_o			(m10_rty),
	     .wbs_err_o			(m10_err),
	     .wbs_adr_i			(m10_adr),
	     .wbs_dat_i			(m10_dat_o),
	     .wbs_stb_i			(m10_stb),
	     .wbs_cyc_i			(m10_cyc),
	     .wbs_cab_i			(m10_cab),
	     .wbs_we_i			(m10_we),
	     .wbs_sel_i			(m10_sel),
             .wbs_size_i                (m10_size),

	     // Master port
	     .wbm_dat_i			(s10_dat_i),
	     .wbm_ack_i			(s10_ack),
	     .wbm_rty_i			(s10_rty),
	     .wbm_err_i			(s10_err),
	     .wbm_stb_o			(s10_stb),
	     .wbm_cyc_o			(s10_cyc),
	     .wbm_cab_o			(s10_cab),
	     .wbm_we_o			(s10_we),
	     .wbm_adr_o			(s10_adr),
	     .wbm_dat_o			(s10_dat_o),
	     .wbm_sel_o			(s10_sel));



   wire [31:0] confbus11;
   wire [11:0] confbus_addr11;
   wire        confbus_we11;
   defparam busnode10.CONFADDR0 = 7'd30;
   defparam busnode10.CONFADDR1 = 7'd31;
   defparam busnode10.CONFADDR2 = 7'd32;
   defparam busnode10.BASEADDR = 5'd10;
   wb_bus_node busnode10(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s10_dat_i),
		   .wbs_ack_o		(s10_ack),
		   .wbs_rty_o		(s10_rty),
		   .wbs_err_o		(s10_err),
		   // Inputs
		   .wbs_stb_i		(s10_stb),
		   .wbs_cyc_i		(s10_cyc),
		   .wbs_cab_i		(s10_cab),
		   .wbs_we_i		(s10_we),
		   .wbs_adr_i		(s10_adr),
		   .wbs_dat_i		(s10_dat_o),
		   .wbs_sel_i		(s10_sel),

		     // Outputs
		     .wbm_stb_o		(m10_stb),
		     .wbm_cyc_o		(m10_cyc),
		     .wbm_cab_o		(m10_cab),
		     .wbm_we_o		(m10_we),
		     .wbm_adr_o		(m10_adr),
		     .wbm_dat_o		(m10_dat_o),
		     .wbm_sel_o		(m10_sel),
                     .wbm_size_o        (m10_size),
		     // Inputs
		     .wbm_dat_i		(m10_dat_i),
		     .wbm_ack_i		(m10_ack),
		     .wbm_rty_i		(m10_rty),
		     .wbm_err_i		(m10_err),
                     
                     .confbus_i         (confbus10),
                     .confbus_we_i         (confbus_we10),
                     .confbus_addr_i         (confbus_addr10),

                     .confbus_o         (confbus11),
                     .confbus_we_o         (confbus_we11),
                     .confbus_addr_o         (confbus_addr11)

);
   defparam    if11.THISNODEADDRESS = 5'd11;
   // Port 11 is connected to node at (1,1) nodeport is 0
   defparam    if11.NUM_DESTS = 11;
   defparam    if11.NEXTHOPNODES = { 5'd0, 5'd1, 5'd2, 5'd3, 5'd4, 5'd5, 5'd6, 5'd7, 5'd8, 5'd9, 5'd10};
   defparam    if11.NEXTHOPDESTS = { 4'b0001, 4'b0001, 4'b0001, 4'b0010, 4'b0010, 4'b0010, 4'b0001, 4'b0001, 4'b0001, 4'b1000, 4'b0100 };
   defparam    if11.ADDRMASK = { 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000, 32'hf8000000 };
   defparam    if11.ADDRESS = { 32'h00000000, 32'h08000000, 32'h10000000, 32'h18000000, 32'h20000000, 32'h28000000, 32'h30000000, 32'h38000000, 32'h40000000, 32'h48000000, 32'h50000000 };
   defparam    if11.DESTS = { 4'b0001, 5'd0, 4'b0001, 5'd1, 4'b0001, 5'd2, 4'b0010, 5'd3, 4'b0010, 5'd4, 4'b0010, 5'd5, 4'b0001, 5'd6, 4'b0001, 5'd7, 4'b0001, 5'd8, 4'b1000, 5'd9, 4'b0100, 5'd10 };

   wire [31:0] 		 m11_dat_i;
   wire [31:0] 		 m11_dat_o;
   wire [31:0] 		 m11_adr;
   wire [3:0] 		 m11_sel;
   wire 		 m11_ack;
   wire 		 m11_rty;
   wire 		 m11_err;
   wire 		 m11_we;
   wire 		 m11_cyc;
   wire 		 m11_stb;
   wire 		 m11_cab;
   wire [4:0]		 m11_size;

   wire [31:0] 		 s11_dat_i;
   wire [31:0] 		 s11_dat_o;
   wire [31:0] 		 s11_adr;
   wire [3:0] 		 s11_sel;
   wire 		 s11_ack;
   wire 		 s11_rty;
   wire 		 s11_err;
   wire 		 s11_we;
   wire 		 s11_cyc;
   wire 		 s11_stb;
   wire 		 s11_cab;
   wire [4:0]		 s11_size;

   wb_if if11(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port11_i),
	     .soclink_i			(port11_o),

	     // Slave port
	     .wbs_dat_o			(m11_dat_i),
	     .wbs_ack_o			(m11_ack),
	     .wbs_rty_o			(m11_rty),
	     .wbs_err_o			(m11_err),
	     .wbs_adr_i			(m11_adr),
	     .wbs_dat_i			(m11_dat_o),
	     .wbs_stb_i			(m11_stb),
	     .wbs_cyc_i			(m11_cyc),
	     .wbs_cab_i			(m11_cab),
	     .wbs_we_i			(m11_we),
	     .wbs_sel_i			(m11_sel),
             .wbs_size_i                (m11_size),

	     // Master port
	     .wbm_dat_i			(s11_dat_i),
	     .wbm_ack_i			(s11_ack),
	     .wbm_rty_i			(s11_rty),
	     .wbm_err_i			(s11_err),
	     .wbm_stb_o			(s11_stb),
	     .wbm_cyc_o			(s11_cyc),
	     .wbm_cab_o			(s11_cab),
	     .wbm_we_o			(s11_we),
	     .wbm_adr_o			(s11_adr),
	     .wbm_dat_o			(s11_dat_o),
	     .wbm_sel_o			(s11_sel));



   wire [31:0] confbus12;
   wire [11:0] confbus_addr12;
   wire        confbus_we12;
   defparam busnode11.CONFADDR0 = 7'd33;
   defparam busnode11.CONFADDR1 = 7'd34;
   defparam busnode11.CONFADDR2 = 7'd35;
   defparam busnode11.BASEADDR = 5'd11;
   wb_bus_node busnode11(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s11_dat_i),
		   .wbs_ack_o		(s11_ack),
		   .wbs_rty_o		(s11_rty),
		   .wbs_err_o		(s11_err),
		   // Inputs
		   .wbs_stb_i		(s11_stb),
		   .wbs_cyc_i		(s11_cyc),
		   .wbs_cab_i		(s11_cab),
		   .wbs_we_i		(s11_we),
		   .wbs_adr_i		(s11_adr),
		   .wbs_dat_i		(s11_dat_o),
		   .wbs_sel_i		(s11_sel),

		     // Outputs
		     .wbm_stb_o		(m11_stb),
		     .wbm_cyc_o		(m11_cyc),
		     .wbm_cab_o		(m11_cab),
		     .wbm_we_o		(m11_we),
		     .wbm_adr_o		(m11_adr),
		     .wbm_dat_o		(m11_dat_o),
		     .wbm_sel_o		(m11_sel),
                     .wbm_size_o        (m11_size),
		     // Inputs
		     .wbm_dat_i		(m11_dat_i),
		     .wbm_ack_i		(m11_ack),
		     .wbm_rty_i		(m11_rty),
		     .wbm_err_i		(m11_err),
                     
                     .confbus_i         (confbus11),
                     .confbus_we_i         (confbus_we11),
                     .confbus_addr_i         (confbus_addr11),

                     .confbus_o         (confbus12),
                     .confbus_we_o         (confbus_we12),
                     .confbus_addr_o         (confbus_addr12)

);

    assign   confbus_o = confbus12;

endmodule
