`default_nettype none

module system(input wire clk_i,
	      input wire rx_i,
	      input wire rst_i,
	      output wire tx_o,
	      output wire [7:0] led_o);

   reg 	     rst_int;
   reg 	     rst_int2;
   wire        clk;
   

   // Normally one would expect this to be automatically inserted
   // by the Xilinx tools but this is not always the case I have
   // noticed.
   BUFG clkbuffer(.I(clk_i),.O(clk));

   
   // Synchronize reset signal
   always @(posedge clk) begin
      rst_int2 <= rst_i;
      rst_int <= rst_int2;
   end


   
   wire           confwbstb;
   wire 	  confwbcyc;
   wire 	  confwbcab;
   wire 	  confwbwe;
   wire [31:0] 	  confwbadr;
   wire [31:0] 	  confwbdati;
   wire [3:0] 	  confwbsel;
   wire [31:0] 	  confwbdato;
   wire 	  confwback;
   wire 	  confwbrty;
   wire 	  confwberr;
   
   
   
   

   serial_wb_top mcu(
		     // Outputs
		     .tx_o		(tx_o),
		     .parport_o		(led_o),
		     .parport_readstrobe_o(),
		     .parport_writestrobe_o(),
		     .wbm_dat_o		(confwbdato),
		     .wbm_adr_o		(confwbadr),
		     .wbm_cyc_o		(confwbcyc),
		     .wbm_lock_o	(),
		     .wbm_sel_o		(confwbsel),
		     .wbm_stb_o		(confwbstb),
		     .wbm_we_o		(confwbwe),
		     // Inputs
		     .clk_i		(clk),
		     .rst_i		(rst_int),
		     .rx_i		(rx_i),
		     .parport_i		({32'habcd1234}),
		     .wbm_dat_i		(confwbdati),
		     .wbm_ack_i		(confwback),
		     .wbm_err_i		(confwberr),
		     .wbm_rty_i		(confwbrty));

   wire [31:0] 		confbus_o;
   wire [11:0] 		confbus_addr;
   wire 		confbus_we;
   wire [31:0] 		confbus_i;

   assign 		confwberr = 0;
   assign 		confwbrty = 0;
   
   confport confport(
		    // Outputs
		    .wb_ack_o		(confwback),
		    .wb_dat_o		(confwbdati[31:0]),
		    .confbus_o		(confbus_o[31:0]),
		    .confbus_addr_o	(confbus_addr[11:0]),
		    .confbus_we_o	(confbus_we),
		    // Inputs
		    .clk_i		(clk),
		    .rst_i		(rst_i),
		    .wb_we_i		(confwbwe),
		    .wb_stb_i		(confwbstb),
		    .wb_cyc_i		(confwbcyc),
		    .wb_dat_i		(confwbdato[31:0]),
		    .wb_adr_i		(confwbadr[31:0]),
		    .confbus_i		(confbus_i[31:0]));


   noc noc(
 	   // Outputs
 	   .confbus_o			(confbus_i),
 	   // Inputs
 	   .clk			(clk),
 	   .rst				(rst_i),
 	   .confbus_i			(confbus_o),
 	   .confbus_addr_i		(confbus_addr),
 	   .confbus_we_i		(confbus_we));
   

endmodule // system
