`timescale 1ns / 10ps
module wb_bus_node(
    input wire clk_i,
    input wire rst_i,

		   // Wishbone slave interface
    input wire [31:0] wbs_adr_i,
    input wire [31:0] wbs_dat_i,
    input wire 	      wbs_stb_i,
    input wire 	      wbs_cyc_i,
    input wire 	      wbs_cab_i,
    input wire 	      wbs_we_i,
    input wire [3:0]  wbs_sel_i,

    output wire [31:0] wbs_dat_o,
    output wire        wbs_ack_o,
    output wire        wbs_rty_o,
    output wire        wbs_err_o,


		   // Wishbone master  port
    output wire        wbm_stb_o,
    output wire        wbm_cyc_o,
    output wire        wbm_cab_o,
    output wire        wbm_we_o,
    output wire [31:0] wbm_adr_o,
    output wire [31:0] wbm_dat_o,
    output wire [3:0]  wbm_sel_o,
    output wire [4:0]  wbm_size_o,
    input wire [31:0]  wbm_dat_i,
    input wire 	       wbm_ack_i,
    input wire 	       wbm_rty_i,
    input wire 	       wbm_err_i,

    input wire [31:0]  confbus_i,
    output wire [31:0] confbus_o,
    input wire 	       confbus_we_i,
    output wire        confbus_we_o,
    input wire [11:0]  confbus_addr_i,
    output wire [11:0] confbus_addr_o
		   );



   
   

   // Wishbone slave port 0
   wire 	       wbs0_stb;
   wire 	       wbs0_cyc;
   wire 	       wbs0_cab;
   wire 	       wbs0_we;
   wire [31:0] 	       wbs0_adr;
   wire [31:0] 	       wbs0_dat_i;
   wire [3:0] 	       wbs0_sel;
   wire [31:0] 	       wbs0_dat_o;
   wire [4:0] 	       wbs0_size;
   wire 	       wbs0_ack;
   wire 	       wbs0_rty;
   wire 	       wbs0_err;

   // Wishbone slave port 1
   wire 	       wbs1_stb;
   wire 	       wbs1_cyc;
   wire 	       wbs1_cab;
   wire 	       wbs1_we;
   wire [31:0] 	       wbs1_adr;
   wire [31:0] 	       wbs1_dat_i;
   wire [3:0] 	       wbs1_sel;
   wire [31:0] 	       wbs1_dat_o;
   wire [4:0] 	       wbs1_size;
   wire 	       wbs1_ack;
   wire 	       wbs1_rty;
   wire 	       wbs1_err;

   // Wishbone slave port 2
   wire 	       wbs2_stb;
   wire 	       wbs2_cyc;
   wire 	       wbs2_cab;
   wire 	       wbs2_we;
   wire [31:0] 	       wbs2_adr;
   wire [31:0] 	       wbs2_dat_i;
   wire [3:0] 	       wbs2_sel;
   wire [31:0] 	       wbs2_dat_o;
   wire [4:0] 	       wbs2_size;
   wire 	       wbs2_ack;
   wire 	       wbs2_rty;
   wire 	       wbs2_err;

   // Wishbone slave port 3
   wire 	       wbs3_stb;
   wire 	       wbs3_cyc;
   wire 	       wbs3_cab;
   wire 	       wbs3_we;
   wire [31:0] 	       wbs3_adr;
   wire [31:0] 	       wbs3_dat_i;
   wire [3:0] 	       wbs3_sel;
   wire [31:0] 	       wbs3_dat_o;
   wire [4:0] 	       wbs3_size;
   wire 	       wbs3_ack;
   wire 	       wbs3_rty;
   wire 	       wbs3_err;


   // Wishbone master  port 0
   wire 	       wbm0_stb;
   wire 	       wbm0_cyc;
   wire 	       wbm0_cab;
   wire 	       wbm0_we;
   wire [31:0] 	       wbm0_adr;
   wire [31:0] 	       wbm0_dat_i;
   wire [3:0] 	       wbm0_sel;
   wire [31:0] 	       wbm0_dat_o;
   wire [4:0] 	       wbm0_size;
   wire 	       wbm0_ack;
   wire 	       wbm0_rty;
   wire 	       wbm0_err;

   // Wishbone master  port 1
   wire 	       wbm1_stb;
   wire 	       wbm1_cyc;
   wire 	       wbm1_cab;
   wire 	       wbm1_we;
   wire [31:0] 	       wbm1_adr;
   wire [31:0] 	       wbm1_dat_i;
   wire [3:0] 	       wbm1_sel;
   wire [31:0] 	       wbm1_dat_o;
   wire [4:0] 	       wbm1_size;
   wire 	       wbm1_ack;
   wire 	       wbm1_rty;
   wire 	       wbm1_err;
   
   // Wishbone master  port 2
   wire 	       wbm2_stb;
   wire 	       wbm2_cyc;
   wire 	       wbm2_cab;
   wire 	       wbm2_we;
   wire [31:0] 	       wbm2_adr;
   wire [31:0] 	       wbm2_dat_i;
   wire [3:0] 	       wbm2_sel;
   wire [31:0] 	       wbm2_dat_o;
   wire [4:0] 	       wbm2_size;
   wire 	       wbm2_ack;
   wire 	       wbm2_rty;
   wire 	       wbm2_err;

   // Wishbone master  port 3
   wire 	       wbm3_stb;
   wire 	       wbm3_cyc;
   wire 	       wbm3_cab;
   wire 	       wbm3_we;
   wire [31:0] 	       wbm3_adr;
   wire [31:0] 	       wbm3_dat_i;
   wire [3:0] 	       wbm3_sel;
   wire [31:0] 	       wbm3_dat_o;
   wire [4:0] 	       wbm3_size;
   wire 	       wbm3_ack;
   wire 	       wbm3_rty;
   wire 	       wbm3_err;


   parameter [4:0]     BASEADDR = 5'h0;
   defparam 	       bus0.BASEADDR = BASEADDR;

   parameter [6:0]     CONFADDR0 = 7'h0;
   parameter [6:0]     CONFADDR1 = 7'h1;
   parameter [6:0]     CONFADDR2 = 7'h2;

   defparam 	       master1.CONFADDR = CONFADDR0;
//   defparam 	       master2.CONFADDR = CONFADDR1;
//   defparam 	       master3.CONFADDR = CONFADDR2;
   
   
   


   assign 	       wbs0_adr   = wbs_adr_i;
   assign 	       wbs0_dat_i = wbs_dat_i;
   assign 	       wbs0_stb   = wbs_stb_i;
   assign 	       wbs0_cyc   = wbs_cyc_i;
   assign 	       wbs0_cab   = wbs_cab_i;
   assign 	       wbs0_we    = wbs_we_i;
   assign 	       wbs0_sel   = wbs_sel_i;
   assign 	       wbs0_size = 5'b01000; // Only used by wb_if anyway.
   
   assign 	       wbs_dat_o  = wbs0_dat_o;
   assign 	       wbs_ack_o  = wbs0_ack;
   assign 	       wbs_rty_o  = wbs0_rty;
   assign 	       wbs_err_o  = wbs0_err;

   assign 	       wbm_adr_o  = wbm0_adr;
   assign 	       wbm_dat_o  = wbm0_dat_o;
   assign 	       wbm_stb_o  = wbm0_stb;
   assign 	       wbm_cyc_o  = wbm0_cyc;
   assign 	       wbm_cab_o  = wbm0_cab;
   assign 	       wbm_we_o   = wbm0_we;
   assign 	       wbm_sel_o  = wbm0_sel;
   assign 	       wbm_size_o = wbm0_size;

   assign 	       wbm0_dat_i = wbm_dat_i;
   assign 	       wbm0_ack   = wbm_ack_i;
   assign 	       wbm0_rty   = wbm_rty_i;
   assign 	       wbm0_err   = wbm_err_i;
   


   
   // Port 0 is "uplink port" (at least for the master side)

   wb_dualbus bus0(.clk_i			(clk_i),
		   .rst_i			(rst_i),
		   .wbm0_ack_i              (wbm0_ack),
		   .wbm0_adr_o              (wbm0_adr),
		   .wbm0_cab_o              (wbm0_cab),
		   .wbm0_cyc_o              (wbm0_cyc),
		   .wbm0_dat_i              (wbm0_dat_i[31:0]),
		   .wbm0_dat_o              (wbm0_dat_o[31:0]),
		   .wbm0_err_i              (wbm0_err),
		   .wbm0_rty_i              (wbm0_rty),
		   .wbm0_sel_o              (wbm0_sel),
		   .wbm0_stb_o              (wbm0_stb),
		   .wbm0_we_o               (wbm0_we),
		   .wbm0_size_o             (wbm0_size),
		   .wbm1_ack_i              (wbm1_ack),
		   .wbm1_adr_o              (wbm1_adr),
		   .wbm1_cab_o              (wbm1_cab),
		   .wbm1_cyc_o              (wbm1_cyc),
		   .wbm1_dat_i              (wbm1_dat_i[31:0]),
		   .wbm1_dat_o              (wbm1_dat_o[31:0]),
		   .wbm1_err_i              (wbm1_err),
		   .wbm1_rty_i              (wbm1_rty),
		   .wbm1_sel_o              (wbm1_sel),
		   .wbm1_stb_o              (wbm1_stb),
		   .wbm1_we_o               (wbm1_we),
		   .wbm1_size_o             (wbm1_size),
		   .wbm2_ack_i              (wbm2_ack),
		   .wbm2_adr_o              (wbm2_adr),
		   .wbm2_cab_o              (wbm2_cab),
		   .wbm2_cyc_o              (wbm2_cyc),
		   .wbm2_dat_i              (wbm2_dat_i[31:0]),
		   .wbm2_dat_o              (wbm2_dat_o[31:0]),
		   .wbm2_err_i              (wbm2_err),
		   .wbm2_rty_i              (wbm2_rty),
		   .wbm2_sel_o              (wbm2_sel),
		   .wbm2_stb_o              (wbm2_stb),
		   .wbm2_we_o               (wbm2_we),
		   .wbm2_size_o             (wbm2_size),
		   .wbm3_ack_i              (wbm3_ack),
		   .wbm3_adr_o              (wbm3_adr),
		   .wbm3_cab_o              (wbm3_cab),
		   .wbm3_cyc_o              (wbm3_cyc),
		   .wbm3_dat_i              (wbm3_dat_i[31:0]),
		   .wbm3_dat_o              (wbm3_dat_o[31:0]),
		   .wbm3_err_i              (wbm3_err),
		   .wbm3_rty_i              (wbm3_rty),
		   .wbm3_sel_o              (wbm3_sel),
		   .wbm3_stb_o              (wbm3_stb),
		   .wbm3_we_o               (wbm3_we),
		   .wbm3_size_o             (wbm3_size),

		   .wbm4_ack_i              (1'b0),
		   .wbm4_adr_o              (),
		   .wbm4_cab_o              (),
		   .wbm4_cyc_o              (),
		   .wbm4_dat_i              (32'b0),
		   .wbm4_dat_o              (),
		   .wbm4_err_i              (1'b0),
		   .wbm4_rty_i              (1'b0),
		   .wbm4_sel_o              (),
		   .wbm4_stb_o              (),
		   .wbm4_we_o               (),
		   .wbm4_size_o             (),

		   .wbs0_ack_o              (wbs0_ack),
		   .wbs0_adr_i              (wbs0_adr),
		   .wbs0_cab_i              (wbs0_cab),
		   .wbs0_cyc_i              (wbs0_cyc),
		   .wbs0_dat_i              (wbs0_dat_i[31:0]),
		   .wbs0_dat_o              (wbs0_dat_o[31:0]),
		   .wbs0_err_o              (wbs0_err),
		   .wbs0_rty_o              (wbs0_rty),
		   .wbs0_sel_i              (wbs0_sel),
		   .wbs0_stb_i              (wbs0_stb),
		   .wbs0_we_i               (wbs0_we),
		   .wbs0_size_i             (wbs0_size),
		   .wbs1_ack_o              (wbs1_ack),
		   .wbs1_adr_i              (wbs1_adr),
		   .wbs1_cab_i              (wbs1_cab),
		   .wbs1_cyc_i              (wbs1_cyc),
		   .wbs1_dat_i              (wbs1_dat_i[31:0]),
		   .wbs1_dat_o              (wbs1_dat_o[31:0]),
		   .wbs1_err_o              (wbs1_err),
		   .wbs1_rty_o              (wbs1_rty),
		   .wbs1_sel_i              (wbs1_sel),
		   .wbs1_stb_i              (wbs1_stb),
		   .wbs1_we_i               (wbs1_we),
		   .wbs1_size_i             (wbs1_size),
		   .wbs2_ack_o              (wbs2_ack),
		   .wbs2_adr_i              (wbs2_adr),
		   .wbs2_cab_i              (wbs2_cab),
		   .wbs2_cyc_i              (wbs2_cyc),
		   .wbs2_dat_i              (wbs2_dat_i[31:0]),
		   .wbs2_dat_o              (wbs2_dat_o[31:0]),
		   .wbs2_err_o              (wbs2_err),
		   .wbs2_rty_o              (wbs2_rty),
		   .wbs2_sel_i              (wbs2_sel),
		   .wbs2_stb_i              (wbs2_stb),
		   .wbs2_we_i               (wbs2_we),
		   .wbs2_size_i             (wbs2_size),
		   .wbs3_ack_o              (wbs3_ack),
		   .wbs3_adr_i              (wbs3_adr),
		   .wbs3_cab_i              (wbs3_cab),
		   .wbs3_cyc_i              (wbs3_cyc),
		   .wbs3_dat_i              (wbs3_dat_i[31:0]),
		   .wbs3_dat_o              (wbs3_dat_o[31:0]),
		   .wbs3_err_o              (wbs3_err),
		   .wbs3_rty_o              (wbs3_rty),
		   .wbs3_sel_i              (wbs3_sel),
		   .wbs3_stb_i              (wbs3_stb),
		   .wbs3_we_i               (wbs3_we),
		   .wbs3_size_i             (wbs3_size),

		   .wbs4_ack_o              (),
		   .wbs4_adr_i              (32'b0),
		   .wbs4_cab_i              (1'b0),
		   .wbs4_cyc_i              (1'b0),
		   .wbs4_dat_i              (32'b0),
		   .wbs4_dat_o              (),
		   .wbs4_err_o              (),
		   .wbs4_rty_o              (),
		   .wbs4_sel_i              (4'b0),
		   .wbs4_stb_i              (1'b0),
		   .wbs4_we_i               (1'b0),
		   .wbs4_size_i             (5'b1));


   wb_slave slave1(
		   // Outputs
		   .wbs_dat_o		(wbm1_dat_i[31:0]),
		   .wbs_ack_o		(wbm1_ack),
		   .wbs_rty_o		(wbm1_rty),
		   .wbs_err_o		(wbm1_err),
		   // Inputs
		   .clk_i		(clk_i),
		   .rst_i		(rst_i),
		   .wbs_stb_i		(wbm1_stb),
		   .wbs_cyc_i		(wbm1_cyc),
		   .wbs_cab_i		(wbm1_cab),
		   .wbs_we_i		(wbm1_we),
		   .wbs_adr_i		(wbm1_adr[31:0]),
		   .wbs_dat_i		(wbm1_dat_o[31:0]),
		   .wbs_sel_i		(wbm1_sel[3:0]));


   assign 	       wbm2_ack = 0;
   assign 	       wbm2_rty = 0;
   assign 	       wbm2_err = 0;
   assign 	       wbm2_dat_i = 0;

   assign 	       wbm3_ack = 0;
   assign 	       wbm3_rty = 0;
   assign 	       wbm3_err = 0;
   assign 	       wbm3_dat_i = 0;
   
   


   wire [31:0] 	       confbus1;
   wire 	       confbus_we1;
   wire [11:0] 	       confbus_addr1;

   wire [31:0] 	       confbus2;
   wire 	       confbus_we2;
   wire [11:0] 	       confbus_addr2;

   
   wb_memtester master1(
		     // Outputs
		     .wbm_stb_o		(wbs1_stb),
		     .wbm_cyc_o		(wbs1_cyc),
		     .wbm_cab_o		(wbs1_cab),
		     .wbm_we_o		(wbs1_we),
		     .wbm_adr_o		(wbs1_adr),
		     .wbm_dat_o		(wbs1_dat_i),
		     .wbm_sel_o		(wbs1_sel),
		     .wbm_size_o        (wbs1_size),
		     // Inputs
		     .clk_i		(clk_i),
		     .rst_i		(rst_i),
		     .wbm_dat_i		(wbs1_dat_o),
		     .wbm_ack_i		(wbs1_ack),
		     .wbm_rty_i		(wbs1_rty),
		     .wbm_err_i		(wbs1_err),
			.confbus_i(confbus_i),
			.confbus_we_i(confbus_we_i),
			.confbus_addr_i(confbus_addr_i),
			.confbus_o(confbus_o),
			.confbus_we_o(confbus_we_o),
			.confbus_addr_o(confbus_addr_o));
/*
   wb_memtester master2(
 			// Outputs
 			.wbm_stb_o		(wbs2_stb),
 			.wbm_cyc_o		(wbs2_cyc),
 			.wbm_cab_o		(wbs2_cab),
 			.wbm_we_o		(wbs2_we),
 			.wbm_adr_o		(wbs2_adr),
 			.wbm_dat_o		(wbs2_dat_i),
 			.wbm_sel_o		(wbs2_sel),
			.wbm_size_o             (wbs2_size),
 			// Inputs
 			.clk_i		(clk_i),
 			.rst_i		(rst_i),
 			.wbm_dat_i		(wbs2_dat_o),
 			.wbm_ack_i		(wbs2_ack),
 			.wbm_rty_i		(wbs2_rty),
 			.wbm_err_i		(wbs2_err),
			.confbus_i (confbus1),
			.confbus_we_i (confbus_we1),
			.confbus_addr_i (confbus_addr1),
			.confbus_o (confbus2),
			.confbus_we_o (confbus_we2),
			.confbus_addr_o (confbus_addr2));
*/

   assign 	       wbs2_stb = 	     0;
   assign 	       wbs2_cyc = 0;
   assign 	       wbs2_dat_i = 0;
   assign 	       wbs2_we = 0;
   assign 	       wbs2_cab = 0;
   assign 	       wbs2_adr = 0;
   assign 	       wbs2_size = 0;
   assign 	       wbs2_sel = 0;

   


   assign 	       wbs3_stb = 	     0;
   assign 	       wbs3_cyc = 0;
   assign 	       wbs3_dat_i = 0;
   assign 	       wbs3_we = 0;
   assign 	       wbs3_cab = 0;
   assign 	       wbs3_adr = 0;
   assign 	       wbs3_size = 0;
   assign 	       wbs3_sel = 0;

endmodule // wb_bus_node
