`timescale 1ns / 10ps
`include "noc_pkt.vh"

module tb_pkt_testsource(input wire clk,
		  input wire rst,
		  input wire oktosend,
    output reg [`CTRLWIDTH-2:0] ctrl,
    output reg [`DATAWIDTH:0] 	data);

   integer 		 finished = 0;
   


   integer 	   debugfd;

   wire [3:0] 	   route;
   wire 	   noroute;
   reg [4:0] 			destidx;

   parameter 	 NUM_DESTS  = 1;

   parameter [(NUM_DESTS * 5) - 1: 0] NODES  = {5'bxxxxx};
   parameter [(NUM_DESTS * 4) - 1: 0]  DESTS = { 4'bxxxx};
   defparam 			       routelookup.NUM_DESTS = NUM_DESTS;
   defparam 			       routelookup.NODES = NODES;
   defparam 			       routelookup.DESTS = DESTS;


   tb_route routelookup(
			     // Outputs
			     .route		(route[3:0]),
			     .noroute		(noroute),
			     // Inputs
			     .nodeaddr		(destidx));

   function integer rand;
      input integer range;
      integer divide;
      integer thenum;
      begin
	 divide = 32'h7fffffff / range;
	 thenum = $random();
	 thenum = thenum & 32'h7fffffff;
	 rand = thenum / divide;
	 if(rand == range) begin
	    $display("rand: Very unlikely, rounding down");
	    rand = range - 1;
	 end
      end
   endfunction // rand
   
   reg [7:0] destaddr[15:0];

   integer   sentwords;
   

   integer 			length;
   reg [8:0] 			dest;
   integer 			delay;
   integer 			thedata;
   integer 			i;
   integer 			j;
   integer 			backoff;

   reg [3:0] 			seqno;


   reg [`DATAWIDTH:0] 			packetbuf[511:0];
   reg [`CTRLWIDTH:0] 			packetbufctrl[511:0];

   reg [`DATAWIDTH-1:0] 			tempdata;
   reg [`CTRLWIDTH-1:0] 			tempctrl;
   always begin
      sentwords = 0;
      seqno <= 0;
      ctrl <= 0;
      data <= 0;
      

      @(posedge clk);
      @(posedge clk);
      while(rst) begin
	 @(posedge clk);
      end

      @(posedge clk);
      @(posedge clk);
      @(posedge clk);

      $display("Started test source %m");

      for(i = 0; i < 1000; i = i + 1) begin
//	 length = rand(80) + 1;
	 length = 64;

	 
	 destidx = rand(32);
	 #1;
	 while(noroute) begin
	    destidx = rand(32);
	    #1;
	 end
	 dest = {route,destidx};
	 
	 $display("%t: Sending %d bytes to %d (seqno %d)",$time,length,dest,i);

	 if(!dest[8:5]) begin
	    $display("No nexthop found, fix your defparams!");
	    $stop;
	 end
	 
	 // Create random packet for sending..
	 for(j = 0; j < length; j = j + 1) begin
	    
	    
	    thedata =  {$random,$random};
	    seqno = j % 16;
	    tempdata = {`DESTWIDTH{1'b0}};
	    tempctrl = 0;

	    tempctrl[`NEXTHOPWIDTH - 1 + `NEXTHOPBIT :`NEXTHOPBIT] = dest[8:5];
	    if(!dest[8:5]) begin
	       $display("%m: No next hop destination, this cannot be right for a NoC uplink, aborting");
	       $stop;
	    end
	    
	    tempctrl[`LASTBIT] = 1'b0;
	    tempctrl[`DESTWIDTH-1:0] = dest[4:0];
	    tempdata = thedata;
	    tempdata[3:0] = seqno[3:0];
	    if(j == length - 1) begin
	       tempctrl[`LASTBIT] = 1'b1;
	    end
	    packetbuf[j] = tempdata;
	    packetbufctrl[j] = tempctrl;
	    $fwrite(debugfd,"%t, %m: wrote data %x\n",$time,{tempctrl[`LASTBIT:0],tempdata});
	    
	 end // for (j = 0; j < length; j = j + 1)
	 
	 
	 for(j = 0; (j < length); j = j + 1) begin

	    data <= packetbuf[j];
	    ctrl <= packetbufctrl[j];
	    while(!oktosend) begin
	       ctrl[`SENDBIT] <= 0;
//	       $display("%m: IDLE");
	       @(posedge clk);
	    end
	    ctrl[`SENDBIT] <= 1;
	    
	    $fwrite(debugfd,"debug: %t, %m: wrote %x to destination %x, destination index %d\n",$time,packetbuf[j],dest,destidx);
	    
//	    $display("%m: UNIDLE");
	    @(posedge clk);
	 end // while (length > 0)
	 


	 sentwords = sentwords + length;

	 delay = 0;
	 $fwrite(debugfd,"debug: %t, %m: Delay until next packet is %d\n",$time,delay);

	 data <= 0;
	 ctrl <= 0;
	 
      
	 while(delay > 0) begin
	    delay = delay - 1;
	    @(posedge clk);
	 end
	       
      end // for (i = 0; i < 3000; i = i + 1)

      // Wait forever...

      finished = 1;
      while(1) begin
	 @(posedge clk);
	 #100000000;
      end
   end
   

   task setdebugfd;
      input integer dbgfd;
      begin
	 debugfd = dbgfd;

      end
   endtask // setdebugfd

endmodule // testsource
