#!/usr/bin/perl -w

# Create a wrapper around a module so that it is easier
# to synthesize it in order to find the maximum frequency.
# (The wrapper adds a scanchain with flip-flops around all inputs
# and outputs of the module. This should avoid any delays caused
# by IOBs.)
#
# Requires Verilog-Perl to be installed.

use strict;

my $filename = $ARGV[0];

#$Verilog::Netlist::Debug = 1;
use Verilog::Netlist;


use Verilog::Getopt;
my $opt = new Verilog::Getopt;
$opt->parameter( "+incdir+include"
);

# Prepare netlist
my $nl = new Verilog::Netlist (options => $opt,
    link_read_nonfatal=>1,
			       );


foreach my $i (@ARGV)  {
    $nl->read_file (filename=>$i);
}

# Read in any sub-modules
$nl->link();
$nl->lint();
$nl->exit_if_error();

foreach my $mod ($nl->top_modules_sorted) {
    create_wrapper ($mod);
    }


my $totalarrayinsize = 0;
my $totalarrayoutsize = 0;

# FIXME - skippa clk och rst...

my @portarray;
sub create_wrapper {
    my $mod = shift;

    my $first = 1;
    
#    print "module wrap_".$mod->name. "(\n";
    foreach my $sig ($mod->ports_sorted) {
	my $isarray = " ";
	my $msb = 0;
	my $lsb = 0;

	if(($sig->name eq "clk") || ($sig->name eq "rst")) {
	    next;
	}

	foreach my $net ( $sig->net ) {
	    if($net->msb) {
		$isarray = sprintf("[%s:%s] ",$net->msb,$net->lsb);
		$msb = eval($net->msb);
		$lsb = eval($net->lsb);
	    }
	}


	my $isoutput = 0;
	if($sig->direction eq "out") {
	    $totalarrayoutsize += $msb - $lsb + 1;
	    $isoutput = 1;
	}else {
	    $totalarrayinsize += $msb - $lsb + 1;
	}

	push(@portarray,[$sig->name,$msb,$lsb,$isoutput]);

    }

    print "module wrap(\n";
    print "    input wire clk,\n";
    print "    input wire rst,\n";
    print "    input wire scanin_i,\n";
    print "    input wire scanenable_i,\n";
    print "    output reg scanout_o);\n\n\n";

    printf("    reg [%d:0] scaninchain;\n",$totalarrayinsize - 1);
    printf("    reg [%d:0] scanoutchain;\n",$totalarrayoutsize - 1 + 1); # Account for 0 at end of scanoutchain...
    printf("    reg scanin1,scanin2;\n\n");
    printf("    always @(posedge clk) {scanin2,scanin1} <= {scanin1,scanin_i};\n");
    printf("    always @(posedge clk) scaninchain[%d:0] <= {scaninchain[%d:0], scanin2};\n\n",$totalarrayinsize - 1,$totalarrayinsize - 2);
    printf("    reg scanenable1,scanenable2;\n\n");
    printf("    always @(posedge clk) {scanenable2,scanenable1} <= {scanenable1, scanenable_i};\n");
    print "    always @(posedge clk) scanoutchain[0] <= 1'b0;\n\n";

    printf("    reg rst1,rst2;\n");
    printf("    always @(posedge clk) {rst2,rst1} <= {rst1,rst};\n\n\n\n");


    my $foo;
    my $firstavailable = 0;
    my $firstoutavailable = 1;
    foreach $foo (@portarray) {
	my @ap;
	@ap = @$foo;
	my $name = $ap[0];
	my $msb = $ap[1];
	my $lsb = $ap[2];
	my $isoutput = $ap[3];

	print STDERR "Synt wrapper I/O $name: [$msb:$lsb]\n";

	if(!$isoutput) {
	    if($msb || $lsb) {
		printf("    wire [$msb:$lsb] $name = scaninchain[%d:%d];\n",$firstavailable + $msb,$firstavailable);
	    }else{
		printf("    wire $name = scaninchain[%d];\n",$firstavailable);
	    }
	    $firstavailable += $msb + 1;
	}else {
	    if($msb || $lsb) {
		printf("    wire [$msb:$lsb] $name;\n");
	    }else{
		printf("    wire $name;\n");
	    }
	    printf("    always @(posedge clk) if(scanenable2) scanoutchain[%d:%d] <= $name; else scanoutchain[%d:%d] <= scanoutchain[%d:%d];\n",
		   $firstoutavailable + $msb,$firstoutavailable,$firstoutavailable + $msb, $firstoutavailable, $firstoutavailable + $msb - 1,$firstoutavailable - 1);
	    $firstoutavailable += $msb + 1;
	}
    }

    print "    reg scanout2,scanout1;\n";
    printf("    always @(posedge clk) {scanout2,scanout1} <= {scanout1, scanoutchain[%d]};\n",$firstoutavailable-1);
    printf("    always @(posedge clk) scanout_o <= scanout2;\n");
    
    printf("\n\n\n    %s testdevice(\n",$mod->name);
    printf("        .clk(clk),\n");
    printf("        .rst(rst2),\n");

    my $firstprint = 1;
    foreach $foo (@portarray) {
	my @ap;
	@ap = @$foo;
	my $name = $ap[0];
	my $array = $ap[1];
	my $isoutput = $ap[2];

	if($firstprint) {
	    $firstprint = 0;
	}else{
	    print ",\n";
	}
	printf("        .%s(%s)",$name,$name);
    }

    print ");\n\n\n";

    print "endmodule\n";


    

}

