#!/usr/bin/perl -w

use strict;

my $xsize;
my $ysize;
my $x;
my $y;
my $portnum = 0;

my $linkmsb;

my @portx;
my @porty;

if($#ARGV + 1 < 4) {
    print STDERR "Usage: creatmesh <xsize> <ysize> <modulename> <type>\n";
    print STDERR "       type can be tb_pkt: Testbench for packet based NoC\n";
    exit 1;
}

$xsize="$ARGV[0]";
$ysize="$ARGV[1]";



sub print_testbench_header {
    print "// Testbench for a mesh with $xsize x $ysize\n";
    print "`timescale 1ns / 10ps\n";
    print "`default_nettype none\n";
    
    print "module $_[0];\n";
    print "   reg clk, rst;\n";
}

sub print_synthesizable_header {
print "// Testbench for a mesh with $xsize x $ysize\n
`timescale 1ns / 10ps
`default_nettype none
module $_[0](
    input wire clk, rst,
    input wire [31:0] confbus_i,
    input wire [11:0] confbus_addr_i,
    input wire 	      confbus_we_i,
    output wire [31:0] confbus_o
	   );
";

}


sub create_and_print_mesh {
    my $xsize = $_[0];
    my $ysize = $_[1];
    my $numports = $_[2];
    my $nodetype = $_[3];
    my $x;
    my $y;

    if(($numports != 4) && ($numports != 5)) {
	print STDERR "Does not know how to handle nodes with $numports ports!\n";
	exit 1;
    }

    for($y=0;$y < $ysize;$y = $y + 1) {
	for($x=0;$x < $xsize;$x = $x + 1) {
	    my $node = $x+$y*$xsize;
	    my $eastnode = $node + 1;
	    my $southnode = $node + $xsize;
	    my $westnode = $node - 1;
	    my $northnode = $node - $xsize;
	    my $tempportnum = $portnum;
	    
#	print "Writing node $x, $y\n";
	    
	    if($x < $xsize - 1 ) {
		print "   wire [$linkmsb:0] link_${node}_to_$eastnode;\n";
		print "   wire [$linkmsb:0] link_${eastnode}_to_$node;\n";
	    }

	    if($y < $ysize - 1 ) {
		print "   wire [$linkmsb:0] link_${node}_to_$southnode;\n";
		print "   wire [$linkmsb:0] link_${southnode}_to_$node;\n";
	    }

	    if(($x == 0) || ($x == $xsize - 1)){
		print "   wire [$linkmsb:0] port${tempportnum}_o;\n";
		print "   wire [$linkmsb:0] port${tempportnum}_i;\n";
		$tempportnum = $tempportnum + 1;
	    }

	    if(($y == 0) || ($y == $ysize - 1)){
		print "   wire [$linkmsb:0] port${tempportnum}_o;\n";
		print "   wire [$linkmsb:0] port${tempportnum}_i;\n";
		$tempportnum = $tempportnum + 1;
	    }

	    if ($numports == 5) {
		print "   wire [$linkmsb:0] port${tempportnum}_o;\n";
		print "   wire [$linkmsb:0] port${tempportnum}_i;\n";
	    }

	    
	    print "   $nodetype$numports node$node(\n";
	    print "           .clk(clk), .rst(rst),\n\n";


	    if($x == 0) {
		print "           .west_o        (port${portnum}_o),\n";
		print "           .west_i        (port${portnum}_i),\n";
		$portx[$portnum] = $x - 1;
		$porty[$portnum] = $y;
		$portnum = $portnum + 1;
	    }else{
		print "           .west_o        (link_${node}_to_${westnode}),\n";
		print "           .west_i        (link_${westnode}_to_${node}),\n";
	    }

	    if($y == 0) {
		print "           .north_o        (port${portnum}_o),\n";
		print "           .north_i        (port${portnum}_i),\n";
		$portx[$portnum] = $x;
		$porty[$portnum] = $y - 1;
		$portnum = $portnum + 1;
	    }else{
		print "           .north_o        (link_${node}_to_${northnode}),\n";
		print "           .north_i        (link_${northnode}_to_${node}),\n";
	    }

	    if($y == $ysize - 1) {
		print "           .south_o        (port${portnum}_o),\n";
		print "           .south_i        (port${portnum}_i),\n";
		$portx[$portnum] = $x;
		$porty[$portnum] = $y + 1;
		$portnum = $portnum + 1;
	    }else{
		print "           .south_o        (link_${node}_to_${southnode}),\n";
		print "           .south_i        (link_${southnode}_to_${node}),\n";
	    }

	    if($x == $xsize - 1) {
		print "           .east_o        (port${portnum}_o),\n";
		print "           .east_i        (port${portnum}_i)";
		$portx[$portnum] = $x + 1;
		$porty[$portnum] = $y;
		$portnum = $portnum + 1;
	    }else{
		print "           .east_o        (link_${node}_to_${eastnode}),\n";
		print "           .east_i        (link_${eastnode}_to_${node})";
	    }

	    if($numports == 5) {
		# Always a local port
		print ",\n           .local_o        (port${portnum}_o),\n";
		print "              .local_i        (port${portnum}_i));\n";
		$portx[$portnum] = $x;
		$porty[$portnum] = $y;
		$portnum = $portnum + 1;
	    }else{
		print ");\n";
	    }

	}
    }

}

sub print_constants{
    my $i;
    $portnum = $_[0];
    
    print "   localparam [31:0] ROUTE_NONE = 32'h00000000;\n";
    for($i = 0; $i < $portnum; $i = $i + 1) {
	printf("   localparam [31:0] ROUTE_%d = 32'h%08x;\n",$i, 1 << $i);
    }
}



sub findroute_west {
    my $x = $_[0];
    my $y = $_[1];
    my $xdest = $_[2];
    my $ydest = $_[3];
    
    if($xdest < $x) {
	if($x == 0) {
	    if($y != $ydest) {
		return 0;
	    }
	}

	return 1;
    }

    return 0;
}

sub findroute_east {
    my $x = $_[0];
    my $y = $_[1];
    my $xdest = $_[2];
    my $ydest = $_[3];

    if($xdest > $x) {

	if($x == $xsize - 1) {
	    if($y != $ydest) {
		return 0;
	    }
	}

	return 1;
    }

    return 0;
}

sub findroute_north {
    my $x = $_[0];
    my $y = $_[1];
    my $xdest = $_[2];
    my $ydest = $_[3];
    

    if(($x == $xdest) || (($x == 0) && ($xdest == -1)) || (($x == $xsize - 1) && ($xdest ==  $xsize))) {
	if($ydest < $y) {
	    return 1;
	}
    }

    return 0;
}

sub findroute_south {
    my $x = $_[0];
    my $y = $_[1];
    my $xdest = $_[2];
    my $ydest = $_[3];

    if(($x == $xdest) || (($x == 0) && ($xdest == -1)) || (($x == $xsize - 1) && ($xdest ==  $xsize))) {
	if($ydest > $y) {
	    return 1;
	}
    }

    return 0;
}

sub findroute_local {
    my $x = $_[0];
    my $y = $_[1];
    my $xdest = $_[2];
    my $ydest = $_[3];

    if( ($x == $xdest) && ($y == $ydest) ) {
	return 1;
    }

    return 0;
}

sub outofbounds {
    my $x = $_[0];
    my $y = $_[1];

    if($x < 0) {
	return 1;
    }
    if($y < 0) {
	return 1;
    }
    if($x >= $xsize) {
	return 1;
    }
    if($y >= $ysize) {
	return 1;
    }
}

sub getexit {
    my $destnode = $_[0];  # This is where we want to go
    my $x = $_[1];         # We are here
    my $y = $_[2];         # We are here

    my $xdest = $portx[$destnode];
    my $ydest = $porty[$destnode];

    if(($xdest == $x) && ($ydest == $y)) {
	return 4; #Local
    }

    if(($xdest < $x) && !outofbounds($x-1,$y)){
	return 0; # Go west
    }

    if(($xdest < $x) && ($ydest == $y)){
	return 0; # Go west
    }

    if(($xdest > $x) && !outofbounds($x+1,$y)){
	return 2; # Go east
    }

    if(($xdest > $x) && ($ydest == $y)){
	return 2; # Go west
    }

    if($ydest < $y) {
	return 1; # Go north
    }

    if($ydest > $y) {
	return 3; # Go south
    }

    print STDERR "Cannot happen!\n";
    exit 1;
}



sub findroute {
    my $destnode = $_[0];
    my $x = $_[1];         # We are here
    my $y = $_[2];         # We are here

    # First find where we will go for the next hop...

    my $nexthop = getexit($destnode,$x,$y);


    my $nexthop_destination;
    if($nexthop == 0){
	# We enter next node from the east
	if(outofbounds($x-1,$y)){
	    return -1;
	}
	$nexthop_destination = getexit($destnode,$x-1,$y);
	if($nexthop_destination > 2){
	    $nexthop_destination--;
	}
    }elsif($nexthop == 1) {
	# We enter next node from the south
	if(outofbounds($x,$y-1)){
	    return -1;
	}
	$nexthop_destination = getexit($destnode,$x,$y-1);
	if($nexthop_destination > 3){
	    $nexthop_destination--;
	}
    }elsif($nexthop == 2) {
	# We enter next node from the west
	if(outofbounds($x+1,$y)){
	    return -1;
	}
	$nexthop_destination = getexit($destnode,$x+1,$y);
	if($nexthop_destination > 0){
	    $nexthop_destination--;
	}
    }elsif($nexthop == 3) {
	# We enter next node from the north
	if(outofbounds($x,$y+1)){
	    return -1;
	}
	$nexthop_destination = getexit($destnode,$x,$y+1);
	if($nexthop_destination > 1){
	    $nexthop_destination--;
	}
    }elsif($nexthop == 4) {
	return -1; # Already done, no nexthop
    }

    return $nexthop_destination;

    print STDERR "Internal error in findroute";
    exit 1;
}

sub printroutes_for_nexthop {
    my $x = $_[0];
    my $y = $_[1];
    my $node = $_[2];
    my $destport;
    my $route0 = "ROUTE_NONE";
    my $route1 = "ROUTE_NONE";
    my $route2 = "ROUTE_NONE";
    my $route3 = "ROUTE_NONE";

    for($destport = 0; $destport < $portnum; $destport = $destport + 1) {
	my $dir;
	$dir = findroute($destport,$x,$y);


	if($dir == 0) {
	    $route0 = "$route0 | ROUTE_$destport";
	}elsif($dir == 1) {
	    $route1 = "$route1 | ROUTE_$destport";
	}elsif($dir == 2) {
	    $route2 = "$route2 | ROUTE_$destport";
	}elsif($dir == 3) {
	    $route3 = "$route3 | ROUTE_$destport";
	}
    }

    printf("\n");
    printf("    defparam node%d.UNIFIED_ROUTELOOKUP = 1;\n",$node);
    printf("    defparam node%d.AROUTE = %s;\n",$node,$route0);
    printf("    defparam node%d.BROUTE = %s;\n",$node,$route1);
    printf("    defparam node%d.CROUTE = %s;\n",$node,$route2);
    printf("    defparam node%d.DROUTE = %s;\n",$node,$route3);
    printf("\n");
}


sub getroute_west {
    my $x = $_[0];
    my $y = $_[1];
    my $port;
    my $route = "";

    if(outofbounds($x,$y)) {
	return "ROUTE_NONE";
    }

    for($port  = 0; $port < $portnum; $port = $port + 1) {
	if( findroute_west($x,$y,$portx[$port],$porty[$port]) ) {
	    if($route eq "") {
		$route = "ROUTE_$port";
	    }else {
		$route = "$route | ROUTE_$port";
	    }
	}
    }
    if($route eq ""){
	return "ROUTE_NONE";
    }else{
	return $route;
    }
}

sub getroute_east {
    my $x = $_[0];
    my $y = $_[1];
    my $port;
    my $route = "";

    if(outofbounds($x,$y)) {
	return "ROUTE_NONE";
    }


    for($port  = 0; $port < $portnum; $port = $port + 1) {
	if( findroute_east($x,$y,$portx[$port],$porty[$port]) ) {
	    if($route eq "") {
		$route = "ROUTE_$port";
	    }else {
		$route = "$route | ROUTE_$port";
	    }
	}
    }
    if($route eq ""){
	return "ROUTE_NONE";
    }else{
	return $route;
    }
}

sub getroute_north {
    my $x = $_[0];
    my $y = $_[1];
    my $port;
    my $route = "";
    
    if(outofbounds($x,$y)) {
	return "ROUTE_NONE";
    }

    for($port  = 0; $port < $portnum; $port = $port + 1) {
	if( findroute_north($x,$y,$portx[$port],$porty[$port]) ) {
	    if($route eq "") {
		$route = "ROUTE_$port";
	    }else {
		$route = "$route | ROUTE_$port";
	    }
	}
    }
    if($route eq ""){
	return "ROUTE_NONE";
    }else{
	return $route;
    }
}

sub getroute_south {
    my $x = $_[0];
    my $y = $_[1];
    my $port;
    my $route = "";
    
    if(outofbounds($x,$y)) {
	return "ROUTE_NONE";
    }

    for($port  = 0; $port < $portnum; $port = $port + 1) {
	if( findroute_south($x,$y,$portx[$port],$porty[$port]) ) {
	    if($route eq "") {
		$route = "ROUTE_$port";
	    }else {
		$route = "$route | ROUTE_$port";
	    }
	}
    }
    if($route eq ""){
	return "ROUTE_NONE";
    }else{
	return $route;
    }
}

sub getroute_local {
    my $x = $_[0];
    my $y = $_[1];
    my $port;
    my $route = "";
    
    if(outofbounds($x,$y)) {
	return "ROUTE_NONE";
    }

    for($port  = 0; $port < $portnum; $port = $port + 1) {
	if( findroute_local($x,$y,$portx[$port],$porty[$port]) ) {
	    if($route eq "") {
		$route = "ROUTE_$port";
	    }else {
		$route = "$route | ROUTE_$port";
	    }
	}
    }
    if($route eq ""){
	return "ROUTE_NONE";
    }else{
	return $route;
    }
}

sub printroutes {
    my $x = $_[0];
    my $y = $_[1];
    my $nodenum = $_[2];
    my $numports = $_[3];

    my $route = 0;
    my $port;

    printf("   defparam node%d.UNIFIED_ROUTELOOKUP = 0;\n",$nodenum);
    printf("   defparam node%d.WEST_WESTROUTE = %s;\n",  $nodenum,getroute_west($x-1, $y));
    printf("   defparam node%d.NORTH_WESTROUTE = %s;\n", $nodenum,getroute_west($x, $y-1));
    printf("   defparam node%d.SOUTH_WESTROUTE = %s;\n", $nodenum,getroute_west($x, $y+1));

    printf("   defparam node%d.EAST_EASTROUTE = %s;\n",  $nodenum,getroute_east($x+1, $y));
    printf("   defparam node%d.NORTH_EASTROUTE = %s;\n", $nodenum,getroute_east($x, $y-1));
    printf("   defparam node%d.SOUTH_EASTROUTE = %s;\n", $nodenum,getroute_east($x, $y+1));

    printf("   defparam node%d.WEST_SOUTHROUTE = %s;\n",  $nodenum,getroute_south($x-1, $y));
    printf("   defparam node%d.EAST_SOUTHROUTE = %s;\n",  $nodenum,getroute_south($x+1, $y));
    printf("   defparam node%d.SOUTH_SOUTHROUTE = %s;\n", $nodenum,getroute_south($x, $y+1));

    printf("   defparam node%d.WEST_NORTHROUTE = %s;\n",  $nodenum,getroute_north($x-1, $y));
    printf("   defparam node%d.EAST_NORTHROUTE = %s;\n",  $nodenum,getroute_north($x+1, $y));
    printf("   defparam node%d.NORTH_NORTHROUTE = %s;\n", $nodenum,getroute_north($x, $y-1));

    if($numports == 5) {
	printf("   defparam node%d.WEST_LOCALROUTE = %s;\n",  $nodenum,getroute_local($x-1, $y));
	printf("   defparam node%d.EAST_LOCALROUTE = %s;\n",  $nodenum,getroute_local($x+1, $y));
	printf("   defparam node%d.NORTH_LOCALROUTE = %s;\n", $nodenum,getroute_local($x, $y-1));
	printf("   defparam node%d.SOUTH_LOCALROUTE = %s;\n", $nodenum,getroute_local($x, $y+1));
    }
}






#getroute_east(3,3);
#printf("WEST_WEST is %s\n", getroute_west(3-1,3));
#printf("WEST_NORTH is %s\n", getroute_north(3-1,3));

#printroutes(3,0,3);
#exit 0;

sub print_mesh_routing_table {
    my $port;
    
    for($y=0;$y < $ysize;$y = $y + 1) {
	for($x=0;$x < $xsize;$x = $x + 1) {
	    my $node = $x+$y*$xsize;
            printroutes_for_nexthop($x,$y,$node);
	}
    }
}

sub print_mesh_routing_table_nonunified {
    my $numports = $_[0];
    my $port;
    
    for($y=0;$y < $ysize;$y = $y + 1) {
	for($x=0;$x < $xsize;$x = $x + 1) {
	    my $node = $x+$y*$xsize;
            printroutes($x,$y,$node,$numports);
	}
    }
}

sub getnodeinfo {
    my $port = $_[0];


    my $nodeport;
    my $nodex;
    my $nodey;

# Figure out which port we are connected to on the correct node
    if($portx[$port] < 0) {
	$nodeport = 4; # west
	$nodex = $portx[$port] + 1;
	$nodey = $porty[$port];
    }elsif($portx[$port] > $xsize - 1) {
	$nodex = $portx[$port] - 1;
	$nodey = $porty[$port];
	$nodeport = 2; # east
    }elsif($porty[$port] < 0) {
	$nodex = $portx[$port];
	$nodey = $porty[$port] + 1;
	$nodeport = 3; # north
    }elsif($porty[$port] > $ysize - 1) {
	$nodex = $portx[$port];
	$nodey = $porty[$port] - 1;
	$nodeport = 1; # south
    }else{
	$nodeport = 0;
	$nodex = $portx[$port];
	$nodey = $porty[$port];
    }

    return ($nodeport,$nodex,$nodey);
}

sub getnexthop {
    my $destnode = $_[0];
    my $nodeport = $_[1];
    my $nodex = $_[2];
    my $nodey = $_[3];
    my $nextroute = "";

    if(findroute_local($nodex,$nodey,$portx[$destnode],$porty[$destnode])) {
	$nextroute = "${nextroute}1";
    }else{
	$nextroute = "${nextroute}0";
    }
    
    if(findroute_south($nodex,$nodey,$portx[$destnode],$porty[$destnode])) {
	$nextroute = "${nextroute}1";
    }else{
	$nextroute = "${nextroute}0";
    }
    
    
    if(findroute_east($nodex,$nodey,$portx[$destnode],$porty[$destnode])) {
	$nextroute = "${nextroute}1";
    }else{
	$nextroute = "${nextroute}0";
    }
    
    
    if(findroute_north($nodex,$nodey,$portx[$destnode],$porty[$destnode])) {
	$nextroute = "${nextroute}1";
    }else{
	$nextroute = "${nextroute}0";
    }
    
    
    if(findroute_west($nodex,$nodey,$portx[$destnode],$porty[$destnode])) {
	$nextroute = "${nextroute}1";
    }else{
	$nextroute = "${nextroute}0";
    }
    

    # Remove the connection which would go back to the current sender
    # since it does not exist
    my $tmp       = substr($nextroute,0,$nodeport);
    $nextroute = substr($nextroute,$nodeport+1,5-$nodeport);
    $nextroute = "4'b$tmp$nextroute";
    return $nextroute;

    
}

# Print a comma separated list of the passed arguments as in:
# printlist(formatstring, @list)
# where formatstring is a (perl) printf style format string
sub printlist {
    my $first = 1;
    foreach $x (@_) {
	if(!$first) {
	    printf(", ");
	}
	$first = 0;
        print $x;
    }
	
}

sub print_interface_nexthop_routes {
    my $port = $_[0];
    my $nodeport;
    my $destport;
    my $nodex;
    my $nodey;

    my @nodes;
    my @dests;

    
# Figure out which port we are connected to on the correct node
    ($nodeport,$nodex,$nodey) = getnodeinfo($port);

    print "   // Port $port is connected to node at ($nodex,$nodey) nodeport is $nodeport\n";

    printf("   defparam    if$port.NUM_DESTS = %d;\n",$portnum - 1);

    my $first = 1;
    my $nextroute;
    for($destport = 0; $destport < $portnum; $destport = $destport + 1){
	if($destport != $port){
	    push(@nodes,sprintf("5'd%d",$destport));

	    $nextroute = getnexthop($destport,$nodeport,$nodex,$nodey);
	    push(@dests,$nextroute);
	    
	}
    }
    print "   defparam    if$port.NEXTHOPNODES = { ";
    printlist(@nodes);
    print "};\n";

    print "   defparam    if$port.NEXTHOPDESTS = { ";
    printlist(@dests);
    print " };\n";

}


sub print_interface_wishbone_routes {
    my $port = $_[0];
    my $nodeport;
    my $destport;
    my $nodex;
    my $nodey;

    my @address;
    my @dests;
    my @addrmask;

    
# Figure out which port we are connected to on the correct node
    ($nodeport,$nodex,$nodey) = getnodeinfo($port);

    my $first = 1;
    my $nextroute;
    for($destport = 0; $destport < $portnum; $destport = $destport + 1){
	if($destport != $port){
# FIXME - change master later to accomodate for non power of 2 sizes!
	    push(@addrmask,"32'hf8000000");
	    push(@address,sprintf("32'h%08x",$destport << 27));
	    $nextroute = getnexthop($destport,$nodeport,$nodex,$nodey);
	    push(@dests,"$nextroute, 5'd$destport");


	    
	}
    }
    print "   defparam    if$port.ADDRMASK = { ";
    printlist(@addrmask);
    print " };\n";

    print "   defparam    if$port.ADDRESS = { ";
    printlist(@address);
    print " };\n";
    print "   defparam    if$port.DESTS = { ";
    printlist(@dests);
    print " };\n";

}

sub print_testbench_interfaces {
    print "\n   // Testbench part follows\n\n";

    my $port;
    for ($port = 0; $port < $portnum; $port = $port + 1) {
	print "   defparam    if$port.THISNODEADDRESS = 5'd$port;\n";
	
	print_interface_nexthop_routes($port);


	print "
   tb_pkt_node if${port}(
	     .clk			(clk),
	     .rst			(rst),

	     // NoC uplink/downlink
	     .link_o			(port${port}_i),
	     .link_i			(port${port}_o));

";

    }
}


sub print_testbench_driver {
    my $port;

    print "
   reg finished;
   initial begin
      finished = 0;
      while(!finished) begin
         clk = 0;
         #50;
         clk = 1;
         #50;
      end
   end
 
   integer debugfd;
   integer cyclecount;
   integer totalsent;
   initial begin
      rst = 1;
      #5001;
      @(posedge clk);
      rst <= 0;

      cyclecount <= 0;
      while(1) begin
	 @(posedge clk);
	 cyclecount <= cyclecount + 1;
      end
   end
   initial begin
      debugfd = \$fopen(\"log.txt\",\"w\");
";

    for($port = 0; $port < $portnum; $port = $port + 1) {
	print "      if$port.source.setdebugfd(debugfd);\n";
	print "      if$port.sink.setdebugfd(debugfd);\n";
    }

    print "
      
      while(1) begin
	 if(!(cyclecount & 32'hffff)) begin
	    \$display(\"Clock cycle is %d\",cyclecount);
	 end

         

         @(posedge clk);
         if(
";

    for($port = 0; $port < $portnum; $port = $port + 1) {
	print "            if$port.source.finished &&\n";
    }
    print "            1) begin
           if(
";


    for($port = 0; $port < $portnum; $port = $port + 1) {
	print "            if$port.source.sentwords +\n";
    }
    print "            0 ==\n";
    for($port = 0; $port < $portnum; $port = $port + 1) {
	print "            if$port.sink.recvwords +\n";
    }
    print "            0) begin\n";

    print "               \$display(\"Finished everything\\n\");\n";
    print "               totalsent = 0;\n";
    for($port = 0; $port < $portnum; $port = $port + 1) {
	print "            totalsent = totalsent + if$port.sink.recvwords;\n";
    }
    print "               \$display(\"Sent a total of %d bytes in %d clock cycles\",totalsent,cyclecount);\n";
    print "                \$stop;\n";
    print "         end
         end
      end
   end
";

}



sub print_wishbone {
    my $prefix= $_[0];

    print "
   wire [31:0] 		 ${prefix}_dat_i;
   wire [31:0] 		 ${prefix}_dat_o;
   wire [31:0] 		 ${prefix}_adr;
   wire [3:0] 		 ${prefix}_sel;
   wire 		 ${prefix}_ack;
   wire 		 ${prefix}_rty;
   wire 		 ${prefix}_err;
   wire 		 ${prefix}_we;
   wire 		 ${prefix}_cyc;
   wire 		 ${prefix}_stb;
   wire 		 ${prefix}_cab;
   wire [4:0]		 ${prefix}_size;
"
}

sub print_wishbone_noc_interface {
    my $port = $_[0];

    print "
   wb_if if${port}(
	     .clk_i			(clk),
	     .rst_i			(rst),

	     // SoC uplink/downlink
	     .soclink_o			(port${port}_i),
	     .soclink_i			(port${port}_o),

	     // Slave port
	     .wbs_dat_o			(m${port}_dat_i),
	     .wbs_ack_o			(m${port}_ack),
	     .wbs_rty_o			(m${port}_rty),
	     .wbs_err_o			(m${port}_err),
	     .wbs_adr_i			(m${port}_adr),
	     .wbs_dat_i			(m${port}_dat_o),
	     .wbs_stb_i			(m${port}_stb),
	     .wbs_cyc_i			(m${port}_cyc),
	     .wbs_cab_i			(m${port}_cab),
	     .wbs_we_i			(m${port}_we),
	     .wbs_sel_i			(m${port}_sel),
             .wbs_size_i                (m${port}_size),

	     // Master port
	     .wbm_dat_i			(s${port}_dat_i),
	     .wbm_ack_i			(s${port}_ack),
	     .wbm_rty_i			(s${port}_rty),
	     .wbm_err_i			(s${port}_err),
	     .wbm_stb_o			(s${port}_stb),
	     .wbm_cyc_o			(s${port}_cyc),
	     .wbm_cab_o			(s${port}_cab),
	     .wbm_we_o			(s${port}_we),
	     .wbm_adr_o			(s${port}_adr),
	     .wbm_dat_o			(s${port}_dat_o),
	     .wbm_sel_o			(s${port}_sel));
";

}

sub print_wishbone_busnode {
    my $nodenum = $_[0];
    my $nodeplusone = $nodenum + 1;
    my $confaddr0 = $nodenum * 3 + 0;
    my $confaddr1 = $nodenum * 3 + 1;
    my $confaddr2 = $nodenum * 3 + 2;

    print "


   wire [31:0] confbus${nodeplusone};
   wire [11:0] confbus_addr${nodeplusone};
   wire        confbus_we${nodeplusone};
   defparam busnode${nodenum}.CONFADDR0 = 7'd$confaddr0;
   defparam busnode${nodenum}.CONFADDR1 = 7'd$confaddr1;
   defparam busnode${nodenum}.CONFADDR2 = 7'd$confaddr2;
   defparam busnode${nodenum}.BASEADDR = 5'd${nodenum};
   wb_bus_node busnode${nodenum}(
		   .clk_i		(clk),
		   .rst_i		(rst),

		   // Outputs
		   .wbs_dat_o		(s${nodenum}_dat_i),
		   .wbs_ack_o		(s${nodenum}_ack),
		   .wbs_rty_o		(s${nodenum}_rty),
		   .wbs_err_o		(s${nodenum}_err),
		   // Inputs
		   .wbs_stb_i		(s${nodenum}_stb),
		   .wbs_cyc_i		(s${nodenum}_cyc),
		   .wbs_cab_i		(s${nodenum}_cab),
		   .wbs_we_i		(s${nodenum}_we),
		   .wbs_adr_i		(s${nodenum}_adr),
		   .wbs_dat_i		(s${nodenum}_dat_o),
		   .wbs_sel_i		(s${nodenum}_sel),

		     // Outputs
		     .wbm_stb_o		(m${nodenum}_stb),
		     .wbm_cyc_o		(m${nodenum}_cyc),
		     .wbm_cab_o		(m${nodenum}_cab),
		     .wbm_we_o		(m${nodenum}_we),
		     .wbm_adr_o		(m${nodenum}_adr),
		     .wbm_dat_o		(m${nodenum}_dat_o),
		     .wbm_sel_o		(m${nodenum}_sel),
                     .wbm_size_o        (m${nodenum}_size),
		     // Inputs
		     .wbm_dat_i		(m${nodenum}_dat_i),
		     .wbm_ack_i		(m${nodenum}_ack),
		     .wbm_rty_i		(m${nodenum}_rty),
		     .wbm_err_i		(m${nodenum}_err),
                     
                     .confbus_i         (confbus${nodenum}),
                     .confbus_we_i         (confbus_we${nodenum}),
                     .confbus_addr_i         (confbus_addr${nodenum}),

                     .confbus_o         (confbus${nodeplusone}),
                     .confbus_we_o         (confbus_we${nodeplusone}),
                     .confbus_addr_o         (confbus_addr${nodeplusone})

);
"
    
}

sub print_synthesizable_nodes {
    print "    wire [31:0] confbus0 = confbus_i;\n";
    print "    wire [11:0] confbus_addr0 = confbus_addr_i;\n";
    print "    wire        confbus_we0 = confbus_we_i;\n\n";
    my $port;
    for ($port = 0; $port < $portnum; $port = $port + 1) {
	print "   defparam    if$port.THISNODEADDRESS = 5'd$port;\n";
	print_interface_nexthop_routes($port);
	print_interface_wishbone_routes($port);

	print_wishbone("m$port");
	print_wishbone("s$port");
	print_wishbone_noc_interface("$port");
	print_wishbone_busnode("$port");
    }
}


if($ARGV[0] < 2) {
    print STDERR "Not supporting a mesh with x < 2\n";
    exit 1;
}

if($ARGV[1] < 2) {
    print STDERR "Not supporting a mesh with y < 2\n";
    exit 1;
}

if($ARGV[3] eq "tb_pkt") {
    $linkmsb = "`DATAWIDTH+`CTRLWIDTH-1";
    print STDERR "Creating a testbench for the packet based NoC\n";
    print_testbench_header($ARGV[2]);
    print "`include \"noc_pkt.vh\"\n";
    create_and_print_mesh($xsize,$ysize,5,"node");
    print_constants($portnum);
    print_mesh_routing_table();
    print_testbench_interfaces();
    print_testbench_driver();
}elsif($ARGV[3] eq "syn_pkt") {
    $linkmsb = 48;
    print STDERR "Creating a synthesizable NoC\n";
    print_synthesizable_header($ARGV[2]);
    create_and_print_mesh($xsize,$ysize,5,"node");
    print_constants($portnum);
    print_mesh_routing_table();
    print_synthesizable_nodes();
    print "\n    assign   confbus_o = confbus$portnum;\n\n";
}else{
    print STDERR "ERROR specifing parameter!\n";
    exit 1;
}

printf("endmodule\n");

exit 0;
