/* Utility to talk to the serial_wb module over RS232 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdint.h>


#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

void setuartspeedto115200(int fd)
{
  struct termios serport;

  if(tcgetattr(fd,&serport) != 0){
    perror("tcgetattr()");
    exit(1);
  }
  

  cfsetspeed(&serport,B115200);
  serport.c_cflag = B115200 | CS8 | CREAD | CLOCAL;
  serport.c_iflag = IGNPAR;
  serport.c_oflag = 0;
  serport.c_lflag = 0;
  
  if(tcsetattr(fd,TCSANOW,&serport) != 0){
    perror("tcsetattr");
    exit(1);
  }

}

int init_device(char *name)
{
	int fd;
	char c;
	fd = open("/dev/ttyS1",O_RDWR);
	if(fd < 0) {
	  perror("open");
	  exit(1);
	}
	setuartspeedto115200(fd);
	c = 3; /* ctrl-c */
	write(fd,&c,1);
	printf("Waiting for reply.");
	fflush(stdout);
	while(1){
	  read(fd,&c,1);
	  printf(".");
	  fflush(stdout);
	  if(c == '>'){
	    printf("\n");
	    return fd;
	  }
	}
}

void writestr(int fd,char *str)
{
	write(fd,str,strlen(str));
}

void writeexpected(int fd,char *buf,int len)
{
  int i,ret;
  char c;
  for(i=0; i < len; i++){
    if(buf[i] != ' ') {
    ret = write(fd,&buf[i],1);
    if(ret < 0) { perror("write"); exit(1); }
    }
    ret = read(fd,&c,1);
    if(ret < 0) { perror("write"); exit(1); }
    if(c != buf[i]){
      printf("Did not read expected data\n");
      exit(1);
    }
  }
}
int getline(int fd, char *buf, int len)
{
	int i;
	for(i=0;i < len; i++){
		read(fd,&buf[i],1);
		if(buf[i] == '\r'){
			buf[i] = 0;
			return i;
		}
	}
	buf[i-1] = 0;
	return i-1;
}

char read1(int fd){
	char c;
	read(fd,&c,1);
	return c;
}

void read_expected(int fd, char e)
{
	char c;
	c = read1(fd);
	if(c != e){
		printf("Error in reading, expected '%c', got '%c'\n",e,c);
		exit(1);
	}
}

uint32_t read_wb(uint32_t addr, int fd)
{
	char buf[500];
	int i;
	int ret;

	sprintf(buf,"d %08x",addr);

	writeexpected(fd,buf,10);

	read_expected(fd,'\r');
	read_expected(fd,'\n');
	for(i=0; i < 8; i++){
		buf[i] = read1(fd);
	}
	buf[8] = 0;
	read_expected(fd,'\r');
	read_expected(fd,'\n');
	read_expected(fd,'>');
	return strtoul(buf,NULL,16);
}



void write_wb(uint32_t addr, uint32_t val, int fd)
{
	char buf[500];
	int i;

	sprintf(buf,"m %08x %08x\r\n",addr,val);
	write(fd,buf,19);

	for(i=0; i < 21; i++){
		read_expected(fd,buf[i]);
	}
	read_expected(fd,'>');
}

void write_par(uint32_t val,int fd)
{
  int i;
  char buf[500];
  sprintf(buf,"o %08x\r\n",val);
  write(fd,buf,10);
  for(i=0; i < 12; i++){
    read_expected(fd,buf[i]);
  }
  read_expected(fd,'>');
}


void confbus_write(uint32_t addr, uint32_t val,int fd)
{
  write_wb(0x4,addr,fd);
  write_wb(0x8,val,fd);
}

uint32_t confbus_read(uint32_t addr,int fd)
{
  return read_wb(addr,fd);
}


int main(int argc,char **argv)
{
	int fd;
	uint32_t foo;

	fd = init_device("/dev/ttyS1");

	foo = read_wb(0xf0,fd);
	printf("Got 0x%x\n",foo);

	//	write_wb(0x00,0xdeadbeef,fd);

	foo = read_wb(0x04,fd);
	printf("Got 0x%x\n",foo);

	//	for(i=0; i < 65536;i++){
	  write_par(0xae,fd);
	  //	}

	printf("Got identification %x\n",confbus_read(0x1f,fd));

	int memtester = 0;
	int i;
	for(memtester = 0; memtester < 40; memtester+=3) {
	  printf("%d: returns 0x%08x\n",memtester,confbus_read(memtester*32+0x1f,fd));
	  printf("  ADDOR: %08x\n",confbus_read(memtester*32+0,fd));
	  printf("  ADDAND: %08x\n",confbus_read(memtester*32+1,fd));
	  printf("  ADDRLENOR: %08x\n",confbus_read(memtester*32+2,fd));
	  printf("  ADDRLENAND: %08x\n",confbus_read(memtester*32+3,fd));
	  printf("  LFSR: %08x\n",confbus_read(memtester*32+4,fd));
	  printf("  TRANSNUM: %08x\n",confbus_read(memtester*32+5,fd));
	}

	confbus_write(0x60,0x0,fd);
	confbus_write(0x61,0x0,fd);
	confbus_write(0x62,0xf0,fd);
	confbus_write(0x63,0x0,fd);
	confbus_write(0x64,0x1,fd);
	confbus_write(0x65,0x2000,fd);
	confbus_write(0x7f,0,fd);
	  uint32_t tmp;
	do{
	  printf(".");
	  fflush(stdout);
	  tmp = confbus_read(0xffffffff,fd);
	  if(tmp & 2) {
	    printf("MEMORY TESTING ERROR, got %x!\n",tmp);
	  }
	  write_par(tmp,fd);
	}while(tmp & 0x1);
	printf("\n");

	printf("Passed\n");

	for(i=0 ; i < 120;i++){
	  memtester = (i % 12)*3;
	  confbus_write(memtester*32 + 0,0,fd);
	  confbus_write(memtester*32 + 1,0x38000000,fd);
	  confbus_write(memtester*32 + 2,0x1,fd);
	  confbus_write(memtester*32 + 3,0xff0,fd);
	  confbus_write(memtester*32 + 4,i+1,fd);
	  confbus_write(memtester*32 + 5,0x2000,fd);
	  confbus_write(memtester*32 + 0x1f,0,fd);
	  uint32_t tmp;

	  
	  printf("Testing memory (iteration %d, master %d ",i,memtester/3);
	  do{
	    printf(".");
	    fflush(stdout);
	    tmp = confbus_read(memtester*32 +0x1f,fd);
	    if(tmp & 2) {
	      printf("MEMORY TESTING ERROR, got %x!\n",tmp);
	    }
	    write_par(tmp,fd);
	  }while(tmp & 0x1);
	  printf("\n");
	}

	return 0;
}
