THETESTNAME=$(basename $0)
HTMLLOG=testlog/tests.html

# Various functions used by the testcases

function getlastfreq () {
    if test -f util/lastfreq
    then
	LASTFREQ=$(cat util/lastfreq)
    else
	LASTFREQ=4
    fi
}

getlastfreq



function cleartestlog () {
    rm -rf testlog/*
    createhtmlheader
}

function finishtestlog () {
    echo 'All tests finished'
    createhtmlfooter
}

function teststart () {
    echo "*** Starting test $THETESTNAME ***"     >> testlog/summary.txt
    rm -rf simulator/work simulator/log.txt simulator/transcript.txt
    STARTTIME=$(date +%s)
}


function testok () {
    ENDTIME=$(date +%s)
    SPENTTIME=$(($ENDTIME - $STARTTIME))

    echo "    Test $THETESTNAME finished with no errors" >> testlog/summary.txt
    echo "    <tr><td>$THETESTNAME<td> <font color = \"#008000\">Passed</font><td>$1<td>$SPENTTIME</tr>" >> $HTMLLOG
}

function testfail () {
    ENDTIME=$(date +%s)
    SPENTTIME=$(($ENDTIME - $STARTTIME))

    echo "FAIL: Test $THETESTNAME FAILED with errors, errors saved in testlog/$THETESTNAME    ">> testlog/summary.txt
    rm -rf testlog/$THETESTNAME
    cp -r simulator testlog/$THETESTNAME
    echo "    <tr><td>$THETESTNAME<td> <font color = \"#ff0000\">Failed</font><td>$1<td>$SPENTTIME</tr>" >> $HTMLLOG
    exit 1
}


# FIXME - what to do about temporary files?
function checklogs () {
    echo -n '['
    echo -n G
    grep -v debug: simulator/log.txt | grep Got | cut -d : -f 2 | sed 's/Got //g' > newfile.txt
    echo -n G
    grep -v debug: simulator/log.txt | grep wrote | cut -d : -f 2 | sed 's/wrote data //g' > newfile2.txt
    
    echo -n S
    sort newfile.txt > newfile3.txt
    echo -n S
    sort newfile2.txt > newfile4.txt
    echo -n D
    echo ']'
    
    if ! diff newfile3.txt newfile4.txt
	then
	rm -f newfile.txt newfile2.txt newfile3.txt newfile4.txt
        return 1
    else
	echo 'All packets ok (FIXME - check order packets arrive in...)'
	rm -f newfile.txt newfile2.txt newfile3.txt newfile4.txt
        return 0
    fi
}



# Creates a UCF file with the specified cycle period
function createucfforwrap {
    (cat <<EOF
NET "clk" TNM_NET = "clk";
TIMESPEC "TS_clk" = PERIOD "clk" $1 ns HIGH 50 %;

EOF
    ) > generated/wrap.ucf
}

function createwrapmk {
    THEFILES=$1

    (cat <<EOF
include Makefile

\$(WD)/wrap.scr: VERILOGFILES=\$($THEFILES) generated/wrap.v
\$(WD)/wrap.scr: \$($THEFILES) generated/wrap.v
\$(WD)/wrap.scr: TOPMODULE=wrap
\$(WD)/wrap.ngd: UCFFILE=generated/wrap.ucf
\$(WD)/wrap.ngd: generated/wrap.ucf
EOF
    ) > generated/wrap.mk

}

function createmakefile_noucffile {
    THETOP=$1
    shift
    THEFILES=$*

    (cat <<EOF
include Makefile
IOBUFINSERTION=NO
\$(WD)/test.scr: VERILOGFILES=\$($THEFILES)
\$(WD)/test.scr: \$($THEFILES)
\$(WD)/test.scr: TOPMODULE=$THETOP
\$(WD)/test.ngd: UCFFILE=generated/empty.ucf
EOF
    ) > generated/test.mk
}


function createmakefile_raw {
    THETOP=$1
    shift
    UCFFILE=$1
    shift
    THEFILES=$*

    (cat <<EOF
include Makefile
# Override default part to a larger one for this test
PART=xc4vlx80-10-ff1148
\$(WD)/test.scr: VERILOGFILES=$THEFILES
\$(WD)/test.scr: $THEFILES
\$(WD)/test.scr: TOPMODULE=$THETOP
\$(WD)/test.ngd: UCFFILE=$UCFFILE
\$(WD)/test.ngd: $UCFFILE
EOF
    ) > generated/test.mk

}

function checkresources {
    teststart
    make clean
    createmakefile_noucffile $* || testfail
    cp /dev/null generated/empty.ucf || testfail
    make -f generated/test.mk synthdir/test_map.ncd || testfail "Failed to synthesize design"
    slicenum=$(grep 'Number of occupied Slices:' synthdir/test_map.mrp|awk '{print $5}') || testfail "Failed during analysis"
    lutnum=$(grep 'Total Number of 4 input LUTs:' synthdir/test_map.mrp|awk '{print $7}') || testfail "Failed during analysis"
    ffnum=$(grep 'Number of Slice Flip Flops:' synthdir/test_map.mrp|awk '{print $6}') || testfail "Failed during analysis"
    testok "Slices: $slicenum,  4LUT: $lutnum, FF: $ffnum"
    
}



function scanmaxfreqforwrap {
    freq=$2
    step=0.1

    echo "*** Creating constraints file with $i cycle time ***"
    createucfforwrap $freq
    echo "*** Constraints file follows:"
    cat generated/wrap.ucf
    echo "*** Makefile follows:"
    cat generated/wrap.mk
    echo "*** Wrap file follows:"
    cat generated/wrap.v
    echo "*** Making...*"
    make -f generated/wrap.mk wrap.ncd || testfail


    if grep 'Timing errors: 0' synthdir/wrap.twr > /dev/null
    then
	echo AP
        while true
	do
  	    maxfreq=$(grep 'Maximum frequency' synthdir/wrap.twr|cut -d : -f 3|cut -d ')' -f 1)
	    echo "*** No timing errors at frequency $maxfreq, decreasing timing ***"
	    newfreq=$(echo $freq - $step | bc)
	    createucfforwrap $freq
	    make -f generated/wrap.mk wrap.ncd || testfail

	    if ! grep 'Timing errors: 0' synthdir/wrap.twr > /dev/null
	    then
		testok "$maxfreq"
		echo $newfreq > util/lastfreq
		return 
	    fi
	    freq=$newfreq
	done
    else
	echo BAR
        while true
	do
	    echo "*** Timing errors at frequency $maxfreq, increasing timing ***"
	    newfreq=$(echo $freq + $step | bc)
	    echo "Newfreq is $newfreq"
	    createucfforwrap $newfreq
	    make -f generated/wrap.mk wrap.ncd || testfail

	    if grep 'Timing errors: 0' synthdir/wrap.twr > /dev/null
	    then
		maxfreq=$(grep 'Maximum frequency' synthdir/wrap.twr|cut -d : -f 3|cut -d ')' -f 1)
		testok "$maxfreq"
		echo $newfreq > util/lastfreq
		return
	    fi
	    freq=$newfreq
	done
    fi
	  
    testfail "Could not meet timing constraints"
}

function runtestforwrap {
    teststart
    make clean 
    createwrapmk $2
    thebasename=$(basename $1 .v)
    echo "Running test for $thebasename"
    util/create_synt_wrapper.sh $1 > generated/wrap.v || testfail
    scanmaxfreqforwrap $thebasename 4.0
}


function createhtmlheader {
    (cat <<EOF

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<title>Test results</title>
<body bgcolor="#ffffff" text="#000000">
<h1>Test results</h1>
<hr>
Testrun started at $(date)
<hr>

<TABLE BORDER=1>
<TR><TD> Testname   <TD> Ok? <TD> Result <TD> Time spent (s) </TR>
EOF
    ) > $HTMLLOG

}

function createhtmlfooter {

    (cat <<EOF
</TABLE>
<hr>
Testrun ended at $(date)
</body>
</html>
EOF
    ) >> $HTMLLOG

}




function meshsimtest {
    teststart
    util/createmesh.pl $1 $2 $3 $3> $3.v || testfail
    make $3.sim || testfail
    if tail -6 simulator/transcript | grep -q 'Finished everything'
	then
	if checklogs
	    then
	    testok
	else
	    testfail
	fi
    else
	testfail
    fi

}


function createucffordevproperties {
    TIMECONSTRAINT=$1
    DISTANCE=$2


    (cat <<EOF
NET "clk" TNM_NET = "clk";
TIMESPEC "TS_clk" = PERIOD "clk" $TIMECONSTRAINT ns HIGH 50 %;


INST "tx" AREA_GROUP = "AG1" ;
AREA_GROUP "AG1" RANGE = SLICE_X0Y40:SLICE_X8Y20 ;

INST "rx" AREA_GROUP = "AG2" ;
AREA_GROUP "AG2" RANGE = SLICE_X$((8 + $DISTANCE))Y40:SLICE_X$((8+8+$DISTANCE))Y20 ;

EOF
    ) > generated/devprop.ucf

}


function sweepdistancescanmaxfreq {
    dist=$1
    for i in $CYCLEPERIODS
      do
      echo "*** Creating constraints file with $i cycle time ***"
      createucffordevproperties $i $dist
      make -f generated/test.mk test.ncd || testfail

      if grep 'Timing errors: 0' synthdir/test.twr > /dev/null
	  then
	  echo "No timing errors at a cycle period of $i for distance $dist"
#            cp -r synthdir testlog/${1}_freqtest
	  maxfreq=$(grep 'Maximum frequency' synthdir/test.twr|cut -d : -f 3|cut -d ')' -f 1)
	  return
      fi
    done

    testfail "Could not meet timing constraints"
}
