`timescale 1ns / 10ps
module lfsr(
    input wire clk,
    input wire enable,
    input wire [31:0] data_i,
    output reg [31:0] data_o);


   reg [31:0] 	      tmp;
   reg 		      fb;
   integer 	      i;
   


   // I had this LFSR available (it is the CRC32 for Ethernet) but
   // it should probably be replaced by some other means of pseudo random
   // generation since it is a bit too hard to synthesize efficiently.
   // 
   always @(posedge clk) begin
      if(enable) begin
	 tmp = data_i;

         // Implement it as a for loop of the bit serial version
         // and hope that the synthesizer optimize it well (at least ISE does)

	 // This should really be 32 by the way to have more pseudo randomness
	 // out of the LFSR, but this is a bit too hard to synthesize at
	 // high speed.
	 
         for(i = 0; i < 8; i = i + 1) begin
          fb = tmp[31];
          tmp[31] = tmp[30];
          tmp[30] = tmp[29];
          tmp[29] = tmp[28];
          tmp[28] = tmp[27];
          tmp[27] = tmp[26];
          tmp[26] =  fb ^ tmp[25];  // x^26
          tmp[25] = tmp[24];
          tmp[24] = tmp[23];
          tmp[23] =  fb ^ tmp[22];  // x^23
          tmp[22] =  fb ^ tmp[21];  // x^22
          tmp[21] = tmp[20];
          tmp[20] = tmp[19];
          tmp[19] = tmp[18];
          tmp[18] = tmp[17];
          tmp[17] = tmp[16];
          tmp[16] =  fb ^ tmp[15];  // x^16
          tmp[15] = tmp[14];
          tmp[14] = tmp[13];
          tmp[13] = tmp[12];
          tmp[12] =  fb ^ tmp[11];  // x^12
          tmp[11] =  fb ^ tmp[10];  // x^11
          tmp[10] =  fb ^ tmp[9];  // x^10
          tmp[9]  = tmp[8];
          tmp[8]  =  fb ^ tmp[7];  // x^8
          tmp[7]  =  fb ^ tmp[6];  // x^7
          tmp[6]  = tmp[5];
          tmp[5]  =  fb ^ tmp[4];  // x^5
          tmp[4]  =  fb ^ tmp[3];  // x^4
          tmp[3]  = tmp[2];
          tmp[2]  =  fb ^ tmp[1];  // x^2
          tmp[1]  =  fb ^ tmp[0];  // x^1
          tmp[0]  =  fb;  // 1
         end // for (i = 0; i < 8; i = i + 1)
	 data_o <= tmp;
      end
   end

endmodule // lfsr
