`timescale 1ns / 10ps
module wb_memtester
  (
    input wire clk_i,
    input wire rst_i,

    // Wishbone master  port
    output reg 	 wbm_stb_o,
    output reg 	 wbm_cyc_o,
    output reg 	 wbm_cab_o,
    output reg 	 wbm_we_o,
    output reg [31:0] 	 wbm_adr_o,
    output reg [31:0] 	 wbm_dat_o,
    output reg [3:0] 	 wbm_sel_o,
    output reg [4:0] 	 wbm_size_o, 		 
    input wire [31:0] 	 wbm_dat_i,
    input wire  	 wbm_ack_i,
    input wire  	 wbm_rty_i,
    input wire  	 wbm_err_i,

    input wire [31:0] 	 confbus_i,
    output wire [31:0] 	 confbus_o,
    input wire 		 confbus_we_i,
    output wire 		 confbus_we_o,
    input wire [11:0]	 confbus_addr_i,
    output wire [11:0]	 confbus_addr_o
   );


   parameter [6:0] 	 CONFADDR = 6'b0;
   defparam 		 transgen.CONFADDR = CONFADDR;

   reg [15:0] 		 transactionlength;
   reg [31:0] 		 addr;

   wire 		 we,rd;

   reg 			 nextdata,nexttransaction;
   wire [31:0] 		 address;
   wire [15:0] 		 length;
   wire [31:0] 		 transactiondata;
   reg 				 error;
   

   transactiongenerator transgen(
				 // Outputs
				 .confbus_o		(confbus_o[31:0]),
				 .confbus_we_o		(confbus_we_o),
				 .confbus_addr_o	(confbus_addr_o[11:0]),
				 .address		(address[31:0]),
				 .length		(length[15:0]),
				 .data_o		(transactiondata),
				 .we			(we),
				 .rd			(rd),
				 // Inputs
				 .clk_i			(clk_i),
				 .rst_i			(rst_i),
				 .error                 (error),
				 .confbus_i		(confbus_i[31:0]),
				 .confbus_we_i		(confbus_we_i),
				 .confbus_addr_i	(confbus_addr_i[11:0]),
				 .nextdata		(nextdata),
				 .nexttransaction	(nexttransaction));
   
   localparam 		 IDLE = 3'h0;
   localparam 		 WRITING = 3'h1;
   localparam 		 READING = 3'h2;
   localparam 		 ERROR = 3'h3;
   reg [1:0]			 wb_cs;
   

   always @* nextdata = wbm_ack_i;
   always @* wbm_dat_o = transactiondata;

   reg 				 wbm_ack_r1;
   reg 				 wbm_ack_r2;
   reg 				 match_r;
   reg [31:0] 			 wbm_dat_r;
   reg [31:0] 			 transactiondata_r;
   
   always @(posedge clk_i) begin
      if(rst_i) begin
	 wbm_ack_r1 <= 0;
	 wbm_ack_r2 <= 0;
	 wbm_dat_r <= 0;
	 match_r <= 0;
      end else begin
	 wbm_ack_r1 <= wbm_ack_i;
	 wbm_ack_r2 <= wbm_ack_r1 && (wb_cs == READING);
	 wbm_dat_r <= wbm_dat_i;
	 transactiondata_r <= transactiondata;
	 match_r <= wbm_dat_r === transactiondata_r;
	 
      end
   end
   
   always @(posedge clk_i)
     if(rst_i) begin
	wb_cs <= IDLE;
	wbm_size_o <= 5'h0;
	error <= 0;
     end else begin
	nexttransaction <= 0;
	wbm_stb_o <= 0;
	wbm_cyc_o <= 0;
	wbm_sel_o <= 4'hf;
	      wbm_we_o <= 0;
      case(wb_cs)
	IDLE: begin
	   if(wbm_ack_r2 && !match_r) begin
	      $display("%m: Data mismatch!");
	      $stop;
	      wb_cs <= ERROR;
	   end

	   if(we) begin
	      wb_cs <= WRITING;
	      transactionlength <= length;
	      wbm_adr_o <= address;
	      if(length != 16'h1) begin
		 wbm_cab_o <= 1;
	      end else begin
		 wbm_cab_o <= 0;
	      end
	      wbm_stb_o <= 1;
	      wbm_cyc_o <= 1;
	      wbm_we_o <= 1;
	      wbm_sel_o <= 4'hf;
	   end

	   if(rd) begin
	      wb_cs <= READING;
	      transactionlength <= length;
	      wbm_adr_o <= address;
	      if(length != 16'h1) begin
		 wbm_cab_o <= 1;
	      end else begin
		 wbm_cab_o <= 0;
	      end
	      wbm_stb_o <= 1;
	      wbm_cyc_o <= 1;
	      wbm_we_o <= 0;
	      wbm_sel_o <= 4'hf;
	      if(length < 5'b10000) begin
		 wbm_size_o <= length + 1;
	      end else begin
		 wbm_size_o <= 5'b10000;
	      end

	   end
	end

	WRITING: begin
	   wbm_stb_o <= 1;
	   wbm_cyc_o <= 1;
	   wbm_we_o <= 1;
	   if(wbm_ack_i) begin
	      if (transactionlength < 16'h2) begin
		 wbm_cab_o <= 0;
	      end else begin
		 wbm_cab_o <= 1;
	      end

	      if (transactionlength < 16'h1) begin
		 wbm_stb_o <= 0;
		 wbm_cyc_o <= 0;
		 wb_cs <= IDLE;
		 nexttransaction <= 1;
		 wbm_we_o <= 0;
	      end else begin
		 wbm_stb_o <= 1;
		 wbm_cyc_o <= 1;
	      end

	      wbm_adr_o <= wbm_adr_o + 4;

	      transactionlength <= transactionlength - 1;

	   end else if(wbm_rty_i) begin // if (wbm_ack_i)
	      wbm_stb_o <= 0;
	      wbm_cyc_o <= 0;
	   end
	end

	READING: begin
	   wbm_stb_o <= 1;
	   wbm_cyc_o <= 1;
	   wbm_we_o <= 0;

	   if(wbm_ack_r2 && !match_r) begin
	      $display("%m: Data mismatch!");
	      $stop;
	      wb_cs <= ERROR;
	   end
	   
	   if(wbm_ack_i) begin
	      if(transactionlength < 5'b10000) begin
		 wbm_size_o <= wbm_size_o - 1;
	      end else begin
		 wbm_size_o <= 5'b10000;
	      end

	      if (transactionlength < 16'h2) begin
		 wbm_cab_o <= 0;
	      end else begin
		 wbm_cab_o <= 1;
	      end

	      if (transactionlength < 16'h1) begin
		 wbm_stb_o <= 0;
		 wbm_cyc_o <= 0;
		 wb_cs <= IDLE;
		 nexttransaction <= 1;
	      end else begin
		 wbm_stb_o <= 1;
		 wbm_cyc_o <= 1;
	      end

	      wbm_adr_o <= wbm_adr_o + 4;

	      transactionlength <= transactionlength - 1;

	   end else if(wbm_rty_i) begin // if (wbm_ack_i)
	      wbm_stb_o <= 0;
	      wbm_cyc_o <= 0;
	   end
	   
	end // case: READING
	ERROR: begin
	   wb_cs <= ERROR;
	   error <= 1;
	end
      endcase // case(wb_cs)
     end // else: !if(rst_i)

   
endmodule // wb_memtester
