`timescale 1ns / 10ps

module wb_findreverseroute(
    input wire clk_i, rst_i,
    input wire [4:0] nodeaddr,
    input wire enable_routelookup,
    output reg [3:0]  route,
    output reg 	      noroute);


   reg 		      noroute_int;
   reg [3:0] 	      route_int;


   parameter 	 NUM_DESTS  = 2;

   parameter [(NUM_DESTS * 5) - 1: 0] NODES  = {5'd1, 5'd8};
   parameter [(NUM_DESTS * 4) - 1: 0]  DESTS = { 4'b001, 4'b001};

   
   integer 	      i;

   reg [(NUM_DESTS * 5) - 1 : 0] tempaddr;
   reg [(NUM_DESTS * 4) - 1 : 0] tempdests;
   
   always @(*) begin
      tempaddr = NODES;
      tempdests = DESTS;

      noroute_int = 1;
      route_int = 4'b0;
      for(i = 0; i < NUM_DESTS; i = i + 1) begin
	 if((nodeaddr) == tempaddr[4:0]) begin
	    route_int = tempdests[3:0];
	    noroute_int = 0;
	 end
	 tempaddr = tempaddr >> 5;
	 tempdests = tempdests >> 4;
      end
      
      
   end
   
   always @(posedge clk_i) begin
      if(rst_i) begin
	 route <= 0;
	 noroute <= 0;
      end else begin
	 if(enable_routelookup) begin
	    route   <= route_int;
	    noroute <= noroute_int;

	    if((route ^ route) !== 4'b0) begin
	       $display("%m: Route contained a z or x (route is %x), xor is %x",route, route ^ route);
	       $stop;
	       
	    end
	 end
      end
   end
   


endmodule // wb_findroute
