`timescale 1ns / 10ps

module wb_findroute(
    input wire clk_i, rst_i,
    input wire [31:0] addr,
    output reg [8:0]  route,
    input wire 	      enable_routelookup,
    output reg 	      noroute);


   reg 		      noroute_int;
   reg [8:0] 	      route_int;


   parameter 	 NUM_DESTS  = 1;

   parameter [(NUM_DESTS * 32) - 1: 0] ADDRMASK = 32'hf8000000;
   parameter [(NUM_DESTS * 32) - 1: 0] ADDRESS  = 32'h20000000;
   parameter [(NUM_DESTS * 9) - 1: 0]  DESTS = { 4'b0010, 5'd4};

   
   integer 	      i;

   reg [(NUM_DESTS *32) - 1 : 0] tempmask;
   reg [(NUM_DESTS *32) - 1 : 0] tempaddr;
   reg [(NUM_DESTS * 9) - 1 : 0] tempdests;


   // If a sane node addressing scheme is used
   // (such as using the upper 5 bits as the
   //  destination node), only 2 LUTs should be
   // spent per bit on this lookup table even if all
   // 32 addresses are used...
   always @(*) begin
      noroute_int = 0;
      route_int = 0; 

      tempmask = ADDRMASK;
      tempaddr = ADDRESS;
      tempdests = DESTS;

      noroute_int = 1;
      route_int = 9'b0;
      for(i = 0; i < NUM_DESTS; i = i + 1) begin
	 if((addr & tempmask[31:0]) == tempaddr[31:0]) begin
	    route_int = tempdests[8:0];
	    noroute_int = 0;
	 end
	 tempmask = tempmask >> 32;
	 tempaddr = tempaddr >> 32;
	 tempdests = tempdests >> 9;
      end
      
      
   end
   
   always @(posedge clk_i) begin
      if(rst_i) begin
	 route <= 0;
	 noroute <= 0;
      end else begin
	 if(enable_routelookup) begin
	    route   <= route_int;
	    noroute <= noroute_int;

	    if((route ^ route) !== 9'b0) begin
	       $display("%m: Route contained a z or x (route is %x), xor is %x",route, route ^ route);
	       $stop;
	       
	    end
	 end
      end
   end
   


endmodule // wb_findroute
