`timescale 1ns / 10ps
`default_nettype none

  // Remaining issues:
  // * wbm_rty_i and wbm_err_i are ignored
  // * The statemachines are pretty large and cumbersome right now.
  //   They should be divided into two different files.
  // * The design of this module is less than stellar
  //   unfortunately. The module itself had a clean design from start
  //   but quickly turned out to be much more trouble than initially
  //   imagined. Getting single transactions right is not a great
  //   problems. Getting burst transactions working with high performance
  //   is far from trivial.



// SoC request format:
// First word:  29:0  address
// 35:30:   00xxxx:  Write  (33:30 is reserved, set to 0)
//          010xxx:  Read
//          011xxx:  Reserved
//          1rrrxx:  Read reply (34:32 is request id of transaction)

// Second word (for writing): (to last word in transaction)
// 35:32: sel signal
// 31:0 data

// Second word  (For reading):
// 35:           Reserved (set to 0)
// 34:30: size | Number of requested words (Don't set to 0...)
// 29:26: sel  | Byte selects for non burst read (size is 1)
//             | This is a clear deficiency, it would be much
//             | better if this could be moved to the first word
//             | in a read request.
// 25:21: src  | Source node address
// 20:18: reqid| Request ID (Up to 8 requests in flight from one source)

// Read reply: First word through last word
// 35:         | 1 to indicate read reply   
// 34:32       | The request ID of this read request
// 31:0        | The data
  

module wb_if(
    input wire clk_i,rst_i,

    // Wishbone slave interface
    input wire [31:0] wbs_adr_i,
    input wire [31:0] wbs_dat_i,
    input wire 	      wbs_stb_i,
    input wire 	      wbs_cyc_i,
    input wire 	      wbs_cab_i,
    input wire 	      wbs_we_i,
    input wire [3:0]  wbs_sel_i,
    input wire [4:0]  wbs_size_i,
		   
    output wire [31:0] wbs_dat_o,
    output reg        wbs_ack_o,
    output reg        wbs_rty_o,
    output wire        wbs_err_o,


    // Wishbone master  port
    output reg 	       wbm_stb_o,
    output reg 	       wbm_cyc_o,
    output reg 	       wbm_cab_o,
    output reg 	       wbm_we_o,
    output reg [31:0] wbm_adr_o,
    output reg [31:0] wbm_dat_o,
    output reg [3:0]  wbm_sel_o,
    input wire [31:0] wbm_dat_i,
    input wire 	      wbm_ack_i,
    input wire 	      wbm_rty_i,
    input wire 	      wbm_err_i,

	     

    input wire [47:0]  soclink_i,
    output wire [47:0] soclink_o);



   
   reg [3:0]  masterstate;
   reg [3:0]  next_masterstate;
   reg [3:0]  state;
   
   wire 	       matching_readreply;

   wire 	       ready_to_send = soclink_i[47];

   parameter [4:0]     THISNODEADDRESS = 5'bx;


   parameter 				     NUM_DESTS = 1;
   parameter [(NUM_DESTS * 5) - 1 : 0] NEXTHOPNODES = {5'dx};
   parameter [(NUM_DESTS * 4) - 1 : 0] NEXTHOPDESTS = {4'bx};
   
   parameter [(NUM_DESTS*32) - 1: 0] 	     ADDRMASK = 32'hxxxxxxxx;
   parameter [(NUM_DESTS*32) - 1: 0] 	     ADDRESS = 32'hxxxxxxxx;
   parameter [(NUM_DESTS*9) - 1 : 0] 	     DESTS = {4'bxxxx,5'bxxxxx};
   
   
   defparam 				     findreverseroute.NUM_DESTS = NUM_DESTS;
   defparam 				     findreverseroute.NODES = NEXTHOPNODES;
   defparam 				     findreverseroute.DESTS = NEXTHOPDESTS;

   defparam 				     findroute.NUM_DESTS = NUM_DESTS;
   defparam 				     findroute.ADDRMASK = ADDRMASK;
   defparam 				     findroute.ADDRESS = ADDRESS;
   defparam 				     findroute.DESTS = DESTS;
   
   

   assign 	      wbs_err_o = 0;

   reg [46:0] 	      soclink_slave_o;
   reg [45:0] 	      masterlink;
   reg [45:0] 	      next_masterlink;
   reg 		      masterlink_send;
   assign 	      soclink_o[46:0] = masterlink_send ? {masterlink_send, masterlink} : soclink_slave_o[46:0];

   // Receiver from the SoCLINK

   wire 	      avail;
   wire [41:0] 	      incoming_req;
   reg 		      read_override;
   reg 		      read_allow;
   reg 		      last_ack;

   reg 		      next_stb;
   reg 		      next_cyc;
   reg 		      next_we;

   // A future version of wb_if should probably have flip flops here as well since
   // the output from an SRL16 based FIFO is not very fast.
   assign 	      wbs_dat_o = incoming_req[31:0];

   // Note that the fifo must have read enable activated when it is
   // empty. (A long time ago this gave better implementation results
   // than a more intuitive interface. I'm not sure if that is till
   // true with the current implementation though.)
   
   wb_srlfifo fifo(
		   // Outputs
		   .sendok		(soclink_o[47]),
		   .avail		(avail),
		   .q			(incoming_req[41:0]),
		   // Inputs
		   .clk			(clk_i),
		   .rst			(rst_i),
		   .d			(soclink_i[41:0]),
		   .we			(soclink_i[46]),
		   .rd			(read_override || (read_allow && last_ack)));

   wire 	      readreq_notfull;
   wire 	      readreq_avail;
   reg 		      readreq_rd;
   reg 		      readreq_we;
   wire [41:0] 	      readreq;
   wb_srlfifo32 readreqfifo(
		   // Outputs
		   .sendok		(readreq_notfull),
		   .avail		(readreq_avail),
		   .q			(readreq[41:0]),
		   // Inputs
		   .clk			(clk_i),
		   .rst			(rst_i),
		   .d			(incoming_req[41:0]),
		   .we			(readreq_we),
		   .rd			(readreq_rd));


   wire [3:0] 	      reverseroute;
   wire 	      reversenoroute;
   reg 		      enable_reverseroutelookup;
   
   wb_findreverseroute findreverseroute(
				    // Outputs
				    .route		(reverseroute),
				    .noroute		(reversenoroute),
				    // Inputs
				    .clk_i		(clk_i),
				    .rst_i		(rst_i),
				    .nodeaddr		(readreq[25:21]),
				    .enable_routelookup	(enable_reverseroutelookup));
   
   reg        isreading;
   
   wire [31:0] wbm_dat_mux;
   reg [3:0]  wbm_sel_mux;

   reg [31:0] wbm_dat_buf;
   reg [3:0]  wbm_sel_buf;
   reg 	      wbm_lastdat;
   reg 	      forcebufwrite;
   reg 	      force_datselload,force_addrload,force_stbload;
   reg [1:0]  addrsel;
   reg 	       load_read_parameters;
   reg [5:0]   read_count;
   reg [4:0]   read_source;
   reg [2:0]   readreqid;
   reg 	       read_in_progress;
   reg 	       next_read_in_progress;

   assign     wbm_dat_mux = last_ack ? incoming_req[31:0]  : wbm_dat_buf;

   always @* begin
      if(isreading) begin
	 wbm_sel_mux = force_datselload ? readreq[29:26] : wbm_sel_buf;
      end else begin
	 wbm_sel_mux = last_ack ? incoming_req[35:32] : wbm_sel_buf;
      end
   end
   

   wire [31:0] wbm_adr_mux;
   wire [31:0] next_wbm_adr;
   assign      next_wbm_adr = wbm_adr_o + 32'd4;
   assign      wbm_adr_mux = (addrsel  == 1) ? next_wbm_adr : (addrsel == 2) ? {readreq[29:0], 2'b00} : {incoming_req[29:0], 2'b00};

   wire [4:0]  read_count_mux;
   wire [4:0]  next_read_count;
   assign      next_read_count = read_count - 1;
   assign      read_count_mux = load_read_parameters ? readreq[34:30] : next_read_count;

   reg 	       next_cab;
   wire        write_cab_mux =  last_ack ? !incoming_req[41] : !wbm_lastdat;
   wire        read_cab_mux = next_cab;

   wire      cab_mux = isreading ? read_cab_mux : write_cab_mux;


   reg 	       new_request,del_transaction;
   reg 	     is_readreply;
   
   
   reg 			 stb, cyc, cab, we;
   reg [31:0] 		 addr;
   reg [31:0] 		 dat_i;
   reg [3:0] 		 sel_i;
   reg 			 inhibit_load;

   reg 			 force_read_count_to_2;
   
 			 
   
   always @(posedge clk_i) begin
      if(rst_i) begin
	 wbm_dat_o <= 0;
	 wbm_dat_buf <= 0;
	 last_ack <= 0;
	 wbm_adr_o <= 0;
	 wbm_sel_o <= 0;
	 wbm_stb_o   <= 0;
	 wbm_cyc_o   <= 0;
	 wbm_cab_o   <= 0;
	 wbm_we_o    <= 0;
	 wbm_lastdat <= 0;
	 read_count <= 0;
	 read_source <= 0;
	 readreqid <= 0;
	 
      end else begin
	 last_ack <= wbm_ack_i;

         // I originally hoped to connect wbm_ack_i directly to the CE port of the FD, but
         // that makes it rather hard to load the initial value :)
         // Perhaps an FPGA optimized bus architecture could have a protocol where the
         // ack signal could go directly to the CE port of all FD:s connected to the bus.
         // The synthesizer may now opt to connect wbm_ack_i || force_datselload directly to
         // the CE port instead for a relatively short path.
	 if(wbm_ack_i || force_datselload) begin
	    wbm_dat_o <= wbm_dat_mux;
	    wbm_sel_o <= wbm_sel_mux;
	 end

	 if(wbm_ack_i || force_stbload) begin
	    wbm_stb_o   <= next_stb;
	    wbm_cyc_o   <= next_cyc;
	    wbm_cab_o   <= cab_mux;
	 end
	 
	 wbm_we_o    <= next_we;

	 if(last_ack || forcebufwrite) begin
	    wbm_dat_buf <= incoming_req[31:0];
	    wbm_lastdat <= incoming_req[41];
	 end

	 if(!isreading && (last_ack || forcebufwrite)) begin
	    wbm_sel_buf <= incoming_req[35:32];
	 end
	 
	 if(load_read_parameters) begin
	    wbm_sel_buf  <= readreq[29:26];
	    read_source  <= readreq[25:21];
	    readreqid    <= readreq[20:18];
	 end



	 if(wbm_ack_i || force_addrload) begin
	    wbm_adr_o <= wbm_adr_mux;
	 end

	 if(force_read_count_to_2) begin
	    read_count <= 5'd2;
	 end else if(wbm_ack_i || load_read_parameters) begin
	    read_count <= read_count_mux;
	 end

	 
      end
   end // always @ (posedge clk_i)


   reg [1023:0]        errormessage = "";
   reg 		       error = 0;
   
   // It would probably look better to keep this kind of
   // check inside the combinatorial block below. Unfortunately that
   // could lead to spurious errors because there was a hazard on
   // one of the inputs to the combinatorial block. Therefore the
   // combinatorial block only sets an error flag
   always @(posedge clk_i) begin
      if(error) begin
	 $display("%m: ERROR: %s",errormessage);
	 $stop;
      end
   end
   

   // For master state machine
   localparam [3:0] MASTER_IDLE       = 4'd0;
   localparam [3:0] MASTER_WR_ADDR    = 4'd1;
   localparam [3:0] MASTER_WR_DAT0    = 4'd2;
   localparam [3:0] MASTER_WR_WRITE   = 4'd3;
   localparam [3:0] MASTER_WR_LAST    = 4'd4;
   localparam [3:0] MASTER_RD_QUEUE   = 4'd5;
   localparam [3:0] MASTER_RD_ADDR    = 4'd6;
   localparam [3:0] MASTER_RD_DATA    = 4'd7;
   localparam [3:0] MASTER_WAITSLAVE  = 4'd8;
   localparam [3:0] MASTER_READREPLY  = 4'd9;

   // For slave state machine
   localparam [3:0]    IDLE           = 4'd0;
   localparam [3:0]    ACCEPTADDR     = 4'd1;
   localparam [3:0]    ACCEPTWR       = 4'd2;
   localparam [3:0]    ACCEPTBURSTWR  = 4'd3;
   localparam [3:0]    WRITEREADMODE  = 4'd4;
   localparam [3:0]    BURSTWRPAUSE   = 4'd5;
   localparam [3:0]    BURSTWRLAST    = 4'd6;
   localparam [3:0]    READREPLY      = 4'd7;
   localparam [3:0]    GOTOIDLE       = 4'd8;
   localparam [3:0]    FLUSHREADREPLY = 4'd9;

   
   always @(*) begin
      error = 0;
      
      force_addrload = 0;
      force_datselload = 0;
      next_masterstate = masterstate;
      read_allow = 0;
      forcebufwrite = 0;
      addrsel = 0;
      read_override = 0;
      isreading = 0;

      next_masterlink = masterlink;
      masterlink_send = 0;

      next_stb = 0;
      next_cyc = 0;
      next_cab = 0;
      
      next_we = 0;
      force_stbload = 0;
      is_readreply = 0;

      readreq_we = 0;
      readreq_rd = 0;
      next_read_in_progress = read_in_progress;
      
      load_read_parameters = 0;
      masterlink_send = 0;

      force_read_count_to_2 = 0;
      enable_reverseroutelookup = 0;

      // Always try to read if the FIFO signals that it is empty
      // Otherwise we would not get any new data due to the way the
      // FIFO works.
      if(!avail) begin
	 read_override = 1;
      end

      // Always try to read if the FIFO signals that it is empty
      // for the same reasons as above.
      if(!readreq_avail) begin
	 readreq_rd = 1;
      end

      
      case(masterstate)
	MASTER_IDLE: begin
	   if(read_in_progress) begin
	      errormessage = "Cannot handle partial read (yet)!";
	      error = 1;
	   end else if(readreq_avail && ready_to_send && (state == IDLE)) begin
	      // Don't go to read state unless the slave state machine
	      // is idle (to avoid dual use of the soclink output)
	      readreq_rd = 1;
	      next_masterstate = MASTER_RD_ADDR;
	      force_addrload = 1;
	      addrsel = 2;
	      next_read_in_progress = 1;
	      isreading = 1;
	   end else if(avail) begin
	      if(incoming_req[41] && incoming_req[35] && incoming_req[34:32]) begin
		 errormessage = "Sanity check failed, incoming requid != 0";
		 error = 1;
	      end
	      
//	      $display("Request is %x",incoming_req[35:33]);
	      
	      if(incoming_req[35:34] == 2'b00) begin
		 next_masterstate = MASTER_WR_ADDR;
		 force_addrload = 1;
		 read_override = 1;
	      end else if(incoming_req[35:33] == 3'b010) begin
		 // There is no place left in the read request fifo.
		 // This should never happen in a well designed network.
		 // If the packet is not dropped a deadlock could occur.
		 // If the packet is dropped, the sender would hang forever.
		 // There is basically no way to handle this gracefully although
		 // the best solution would be to extend this bus wrapper so that
		 // an error of some sort is flagged to a master processor and let
		 // that one clean up the mess.
		 if ( !readreq_notfull ) begin
		    errormessage = "Read FIFO full! Aborting...";
		    error = 1;
		 end else begin
//		    $display("Starting read");
		 end

		 readreq_we = 1;
		 read_override = 1;
		 
		 next_masterstate = MASTER_RD_QUEUE;
	      end else if(incoming_req[35] == 1'b1) begin // if (incoming_req[35:33] == 3'b010)
		 is_readreply = 1;
		 next_masterstate = MASTER_WAITSLAVE;
	      end else begin // if (incoming_req[35:34] == 2'b01)
//		 $display("%m: Got unknown request, aborting");
//		 $stop;
	      end // else: !if(incoming_req[35:34] == 2'b01)
	   end
	end

	//////////////////////////////////////////////////
	// Read stuff
	//////////////////////////////////////////////////

	MASTER_RD_QUEUE: begin
	   if(avail) begin
	      if ( !readreq_notfull ) begin
		 // This should not happen, the lowwatermark in the
		 // readreq fifo should guarantee that but the sanity check is here nevertheless.
		 errormessage ="Read FIFO full in middle of transaction! Aborting...";
		 error = 1;
	      end
	      next_masterstate = MASTER_IDLE;
	      readreq_we = 1;
	      read_override = 1;
	   end // if (avail)
	end // case: MASTER_RD_QUEUE

	MASTER_RD_ADDR: begin
	   isreading = 1;
	   if(readreq_avail) begin
	      addrsel = 1;
	      readreq_rd = 1;
	      load_read_parameters = 1;
	      enable_reverseroutelookup = 1;
	      force_datselload = 1; // Need to load sel signals...
	      force_stbload = 1;
 	      next_masterstate = MASTER_RD_DATA;
	      next_stb = 1;
	      next_cyc = 1;
	      next_cab = readreq[4:0] != 1;
	   end else begin // if (readreq_avail)
	      // This could be handled more sanely.
	      errormessage = "Cannot handle a broken up read-transaction";
	      error = 1;
	   end
	end // case: MASTER_RD_ADDR

	MASTER_RD_DATA: begin
	   isreading = 1;
	   addrsel = 1;
	   next_stb = 1;
	   next_cyc = 1;
	   next_masterlink[41] = 0;
	   next_masterlink[45:42] = reverseroute;
	   next_masterlink[40:36] = read_source;
	   next_masterlink[35] = 1; // read reply
	   next_masterlink[34:32] = readreqid;
	   next_masterlink[31:0] = wbm_dat_i; // This might have to be put into a flip-flop later on for performance reasons.

	   if(reversenoroute) begin
	      errormessage = "No reverse route found! Aborting";
	      error = 1;
	   end
	   

	   // Handle next cab is zero
	   if(read_count[4:0] == 5'd2) begin
	      next_cab = 0;
	   end else begin
	      next_cab = 1;
	   end

	   if(read_count[4:0] == 5'd1) begin
	      next_stb = 0;
	      next_cyc = 0;
	      next_cab = 0;
	      next_masterlink[41] = 1; // last part of packet!
	   end

	   if(read_count[4:0] == 5'd0) begin
	      next_stb = 0;
	      next_cyc = 0;
	      next_cab = 0;
	   end

	   if(last_ack && !read_count) begin
	      next_read_in_progress = 0;
	      next_masterstate = MASTER_IDLE;
	      next_masterlink[41] = 1; // last part of packet!
	   end

	   // Read counter decreased elsewhere if wbm_ack_i
	   
	   if(last_ack) begin
	      if(ready_to_send) begin
		 masterlink_send = 1;
	      end else begin
		 // Just end the transaction too early in this case
		 // It doesn't matter because we will resend the
		 // transaction later on.
		 masterlink_send = 1;
		 if (read_count[4:0] > 2) begin 
		    force_read_count_to_2 = 1;
		 end
	      end
	   end // if (last_ack)
	end

	//////////////////////////////////////////////////
	// Write stuff
	//////////////////////////////////////////////////
	
	MASTER_WR_ADDR: begin
	   if(avail) begin
	      next_masterstate = MASTER_WR_DAT0;
	      read_override = 1;
	      forcebufwrite = 1;
	   end
	end

	MASTER_WR_DAT0: begin
	   force_datselload = 1;
	   forcebufwrite = 1;
	   force_stbload = 1;
	   next_stb = 1;
	   next_cyc = 1;
	   next_we = 1;
	   
	   if(wbm_lastdat) begin
	      next_masterstate = MASTER_WR_LAST;
	      read_override = 0;
	   end else begin
	      next_masterstate = MASTER_WR_WRITE;
	      read_override = 1;
	   end
	end

	MASTER_WR_WRITE: begin
	   addrsel = 1;
	   read_allow = 1;
	   next_we = 1;
	   next_stb = 1;
	   next_cyc = 1;

	   if(wbm_lastdat) begin
	      if(last_ack) begin
		 next_we = 0;
		 next_stb = 0;
		 next_cyc = 0;
		 next_masterstate = MASTER_IDLE;
		 read_allow = 0;
	      end
	   end else if(incoming_req[41]) begin
	      if(last_ack) begin
		 next_masterstate = MASTER_WR_LAST;
	      end
	   end

	   
	end // case: MASTER_WR_WRITE


	MASTER_WR_LAST: begin
	   next_we = 1;
	   next_stb = 0;
	   next_cyc = 0;
	   if(last_ack) begin
	      next_masterstate = MASTER_IDLE;
	   end
	end

	MASTER_WAITSLAVE: begin
	   is_readreply = 1;
	   if(incoming_req[41] && del_transaction) begin
	      next_masterstate = MASTER_IDLE;
	      read_override = 1;
	   end else if(state == READREPLY) begin
	      next_masterstate = MASTER_READREPLY;
	   end
	end

	MASTER_READREPLY: begin
	   is_readreply = 1;
	   if(state == FLUSHREADREPLY) begin
	      is_readreply = 0;
	      read_override = 1;
	      if(avail && incoming_req[41]) begin
		 next_masterstate = MASTER_IDLE;
	      end
	   end if(stb && cyc && !we && (state == READREPLY)) begin
	      if(avail) begin
		 if(incoming_req[41]) begin
		    next_masterstate = MASTER_IDLE;
		 end
		 read_override = 1;
	      end
	   end
	end

	default: begin
	   if(!rst_i) begin
	      errormessage = "Default case unexpectedly encountered";
	      error = 1;
	   end
	end


	//////////////////////////////////////////////////////////////////////
	//// Read reply handling
	//////////////////////////////////////////////////////////////////////

	

      endcase // case(masterstate)
   end // always @ (*)
   
   always @(posedge clk_i) begin
      if(rst_i) begin
	 masterstate      <= MASTER_IDLE;
	 read_in_progress <= 0;
	 masterlink <= 0;
      end else begin
	 masterstate      <= next_masterstate;
	 read_in_progress <= next_read_in_progress;
	 masterlink <= next_masterlink;
      end
   end // always @ (posedge clk_i)


   always @(posedge clk_i) begin
       if(masterstate == MASTER_IDLE) begin
       if(!read_in_progress && !(readreq_avail && ready_to_send) && avail) begin
	if(incoming_req[41] && !incoming_req[35]) begin
		$display("%m: Sanity check failed, write request must be at least 2 entries!");
		$stop;
	end
       
       end
       end
   end

   


   reg clear_slavesigs;

   reg [4:0] size;
   
   always @(posedge clk_i) begin
      if(rst_i) begin
	 stb   <= 0;
	 cyc   <= 0;
	 addr   <= 0;
	 cab   <= 0;
	 we    <= 0;
	 dat_i <= 0;
	 sel_i <= 0;
	 size <= 0;

	 
      end else begin
	 addr <= wbs_adr_i;
	 we <= wbs_we_i;
	 size <= wbs_size_i;

	 if(clear_slavesigs) begin
	    stb <= 0;
	    cyc <= 0;
	    cab <= 0;
	 end else if(!inhibit_load) begin
	    stb <= wbs_stb_i;
	    cyc <= wbs_cyc_i;
	    cab <= wbs_cab_i;
	    dat_i <= wbs_dat_i;
	    sel_i <= wbs_sel_i;
	 end
      end
   end // always @ (posedge clk_i)

   wire [8:0] route;
   wire       noroute;

   reg 	      enable_routelookup;
   
   wb_findroute findroute(
			  // Outputs
			  .route		(route),
			  .noroute		(noroute),
			  // Inputs
			  .enable_routelookup   (enable_routelookup),
			  .clk_i		(clk_i),
			  .rst_i		(rst_i),
			  .addr			(addr));

   wire [2:0] new_reqid;
   wire       transaction_ok;
   wire [2:0] matching_req;
   wire       matching_transaction;

   wire [35:0] key = {sel_i,addr};


   // The transaction memory is designed to be able to handle more than
   // one request "in flight" at the same time. This has not yet been
   // tested however and support for this feature is not yet completed.

   wb_transactioncam cam(
			 .clk			(clk_i),
			 .rst			(rst_i),

			 .first_req		(new_reqid),
			 .avail			(transaction_ok),
			 .match_req		(matching_req[2:0]),
			 .match			(matching_transaction),

			 .key			(key[35:0]),
			 .we			(new_request),
			 .del_req		(3'd0), // Request ID is hardcoded for now
			 .del			(del_transaction));
   
   assign matching_readreply = matching_transaction && 
	(matching_req == incoming_req[34:32]) && is_readreply;

   parameter [7:0] TIMEOUT = 8'h8;
   reg 	  timeout_enable;
   reg 	  timeout_occured;
   reg [7:0] timeoutcount;
   always @(posedge clk_i) begin
      if(rst_i) begin
	 timeoutcount <= 0;
      end else begin
	 if(timeout_enable) begin
	    timeoutcount <= timeoutcount + 1;
	    if(timeoutcount == TIMEOUT) begin
	       timeout_occured <= 1;
	    end 
	 end else begin
	    timeoutcount <= 0;
	    timeout_occured <= 0;
	 end
      end
   end

   // This FSM could be optimized. Right now it is conservative about going
   // quickly to IDLE. It could be changed in some places to assert
   // clear_slavesigs and go directly to IDLE instead of via GOTOIDLE.
   //
   // If I remember correctly, this FSM might also send an ACK if
   // stb/cyc is not asserted. This is actually legal per the wishbone
   // spec but should probably be corrected nevertheless.
   
   always @(posedge clk_i) begin
      if(rst_i) begin
	 wbs_rty_o <= 0;
	 wbs_ack_o <= 0;
	 state <= IDLE;
	 soclink_slave_o <= 0;
	 inhibit_load <= 0;
	 enable_routelookup <= 0;
	 clear_slavesigs <= 0;
	 timeout_enable <= 0;
	 
      end else begin
	 timeout_enable <= 0;
	 
	 soclink_slave_o[46] <= 0;   // strobe
	 soclink_slave_o[45:42] <= route[8:5]; // Next hop pins
	 soclink_slave_o[41] <= 0;   // Last packet
	 soclink_slave_o[40:36] <= route[4:0]; // destaddr
	 wbs_rty_o <= 0;
	 wbs_ack_o <= 0;
	 enable_routelookup <= 0;
	 inhibit_load <= 0;
	 del_transaction <= 0;
	 new_request <= 0;
	 clear_slavesigs <= 0;

	 

	 
	 case(state)
	   IDLE: begin
	      enable_routelookup <= 1;
	      if(isreading) begin
		 if(stb && cyc) begin
		    wbs_rty_o <= 1;
		    state <= GOTOIDLE;
		    clear_slavesigs <= 1;
		 end
	      end else begin
		 if(stb && cyc && is_readreply && !we) begin
		    
		    if(matching_readreply && incoming_req[41]) begin
		       state <= GOTOIDLE; // Immediate reply for this read request, only one word long though
		       del_transaction <= 1; // Remove it...
		       wbs_ack_o <= 1;
		       clear_slavesigs <= 1; // Flush strobe signals
		    end else if(matching_readreply) begin
		       state <= READREPLY; // We have a reply for this request!
		       del_transaction <= 1; // Remove it...
		    end else begin
		       wbs_rty_o <= 1; // We don't allow any read access when we
		       // are trying to read...
		       clear_slavesigs <= 1;
		       state <= GOTOIDLE;
		    end
		 end else if(we && stb && cyc && ready_to_send) begin
		    wbs_ack_o <= 1;
		    state <= ACCEPTADDR;
		 end else if(!we && stb && cyc && transaction_ok && ready_to_send) begin
		    state <= ACCEPTADDR;
		    new_request <= 1;
		 end else if(stb && cyc) begin
		    wbs_rty_o <= 1;
		    state <= GOTOIDLE;
		    clear_slavesigs <= 1;
		 end
	      end // if (!isreading)
	   end // case: IDLE

	   // Route lookup finished here
	   ACCEPTADDR: begin
	      if(noroute) begin
		 $display("%m: No valid route found when doing route lookup!");
		 $stop;
	      end

	      if(!route) begin
		 $display("%m: noroute is not zero but no sel signals are active anyway!");
		 $stop;
	      end
	      
	      
	      if(we) begin
		 if(cab) begin
		    wbs_ack_o <= 1;
		    state <= ACCEPTBURSTWR;
		 end else begin
		    // checkme: Not sure if this should be included on the previous line as well
		    inhibit_load <= !ready_to_send;
		    state <= ACCEPTWR;
		 end
		 soclink_slave_o[46] <= 1;
		 soclink_slave_o[35:34] <= 2'b00; // Write command
		 soclink_slave_o[33:30] <= 4'b0000;
		 soclink_slave_o[29:0] <= addr[31:2];
	      end else begin // if (we)
		 // We shouldn't get here unless we are ready_to_send!
		 state <= WRITEREADMODE;
		 soclink_slave_o[46] <= 1;
		 soclink_slave_o[35:30] <= 6'b010000;
		 soclink_slave_o[29:0] <= addr[31:2];
	      end // else: !if(we)
	   end // case: ACCEPTADDR

	   WRITEREADMODE: begin
	      // We might not have ready_to_send as true here but the
	      // generation of the ready_to_send signal should take
	      // this into account so that we always can write an extra
	      // word when ready_to_send goes from 1 to 0.

	      state <= GOTOIDLE;
	      soclink_slave_o[46] <= 1;
              soclink_slave_o[41] <= 1;
	      soclink_slave_o[35] <= 0; // reserved
	      soclink_slave_o[34:30] <= size; 
	      soclink_slave_o[29:26] <= cab ? 4'b1111 : sel_i;
	      soclink_slave_o[25:21] <= THISNODEADDRESS;
	      soclink_slave_o[20:18] <= 3'd0; // Default to req id 0
	      soclink_slave_o[17:0] <= 0;
	      wbs_rty_o <= 1;
	      clear_slavesigs <= 1;
	   end

	   // Address written, write one single entry
	   ACCEPTWR: begin
	      soclink_slave_o[46] <= 1;
	      inhibit_load <= !ready_to_send;

	      
	      // Unfortunately this line checks two incoming signals
	      // directly, it would be best if that could be avoided
	      // somehow.

	      // I'm not sure if cab has to be checked here or not.
	      if(cab && cyc && stb && wbs_stb_i && wbs_cyc_i) begin
		 $display("%m: WARNING WARNING, cab active in ACCEPTWR!");
		 $stop;
		 soclink_slave_o[41] <= 0;
		 wbs_ack_o <= 1;
	      end else begin
		 wbs_ack_o <= 0;
		 soclink_slave_o[41] <= 1;
	      end

	      if(!ready_to_send) begin
		 // No space left in outgoing FIFO.
		 // (It would be better if we issued a retry here I think.)
		 wbs_ack_o <= 0;
		 soclink_slave_o[46] <= 0;
		 state <= ACCEPTWR;
	      end else begin
		 clear_slavesigs <= 1;
		 state <= GOTOIDLE;
	      end

	      soclink_slave_o[35:32] <= sel_i;
	      soclink_slave_o[31:0] <= dat_i;

	      
	   end

	   // Address written, do burst write
	   ACCEPTBURSTWR: begin
	      soclink_slave_o[46] <= 1;
	      if(cab && cyc && stb && wbs_stb_i && wbs_cyc_i) begin
		 soclink_slave_o[41] <= 0;
		 wbs_ack_o <= 1;
	      end else begin
		 wbs_ack_o <= 0;
		 soclink_slave_o[41] <= 1;
	      end

	      if(cab && cyc && stb) begin
		 state <= ACCEPTBURSTWR;
	      end else begin
		 enable_routelookup <= 1;
		 state <= IDLE;
	      end

	      if(!ready_to_send) begin
		 // We should probably add some sort of timer to handle
		 // a retry here after a little while
//		 $display("%m: %t FIFO almost full (or full)!",$time);
		 wbs_ack_o <= 0;
		 soclink_slave_o[46] <= 1;  // Write the last entry...
		 if(cab) begin
		    state <= BURSTWRPAUSE;
		 end else begin
		    state <= GOTOIDLE;
		 end
		 inhibit_load <= 1;
	      end

	      soclink_slave_o[35:32] <= sel_i;
	      soclink_slave_o[31:0] <= dat_i;
	   end // case: ACCEPTBURSTWR

	   BURSTWRPAUSE: begin
	      timeout_enable <= 1;
	      inhibit_load <= !ready_to_send;

	      if(ready_to_send) begin
		 if(!cab && stb && cyc) begin
		    // This is the last transaction
		    // Handle it as we handle a single transaction
		    wbs_ack_o <= 0; // already acked...
		    state <= ACCEPTWR;
		 end else if(cab && stb && cyc) begin
		    wbs_ack_o <= 1;
		    state <= ACCEPTBURSTWR;
		    soclink_slave_o[46] <= 0;
		 end else begin
		    $display("%m: This should never happen!");
		    $stop;
		    wbs_ack_o <= 1;
		    state <= GOTOIDLE;
		    soclink_slave_o[46] <= 1;
		    soclink_slave_o[41] <= 1;
		    soclink_slave_o[35:32] <= sel_i;
		    soclink_slave_o[31:0] <= dat_i;
		 end // else: !if(cab && stb && cyc)
	      end else if(timeout_occured) begin // if (ready_to_send)
		 state <= GOTOIDLE;
		 clear_slavesigs <= 1;
		 wbs_ack_o <= 0;
		 soclink_slave_o[46] <= 1;
		 soclink_slave_o[41] <= 1;
		 soclink_slave_o[35:32] <= sel_i;
		 soclink_slave_o[31:0] <= dat_i;
	      end else begin
		 state <= BURSTWRPAUSE;
		 wbs_ack_o <= 0;
		 soclink_slave_o[46] <= 0;
	      end


	      if(!cab && stb && cyc) begin
	      end
	      
	   end // case: BURSTWRPAUSE


	   // This should be optimized to avoid using wbs_stb_i/cyc_i/we_i directly
	   // for performance reasons.
	   READREPLY: begin
	      if(wbs_stb_i && wbs_cyc_i && !wbs_we_i) begin
		 if(avail) begin
		    if(incoming_req[41]) begin
		       wbs_ack_o <= 0;
		       state <= GOTOIDLE;
		    end else begin
		       wbs_ack_o <= 1;
		    end
		 end
	      end else begin // if (wbs_stb_i && wbs_cyc_i && !wbs_we_i)
		 if(avail && incoming_req[41]) begin
		    state <= GOTOIDLE;
		 end else begin
		    state <= FLUSHREADREPLY;
		 end
	      end // else: !if(wbs_stb_i && wbs_cyc_i && !wbs_we_i)
	   end // case: READREPLY



	   FLUSHREADREPLY: begin
	      $display("%m: In flush readreply");
	      $stop;
	      if(avail && incoming_req[41]) begin
		 state <= GOTOIDLE;
		 $display("   All flushed, going to idle");
	      end
	   end


	   GOTOIDLE: begin
	      state <= IDLE;
	      enable_routelookup <= 1;
	   end // else: !if(rst_i)
      
	 endcase // case(state)
      end
   end // always @ (posedge clk_i)






//////////////////////////////////////////////////////////////////////
// SANITY CHECKS
//////////////////////////////////////////////////////////////////////
// Below this line is no synthesizable constructs, the remaining file
// consists of sanity checks used in simulation
//////////////////////////////////////////////////////////////////////
   

   always @(posedge clk_i) begin
      if(masterstate == MASTER_WR_WRITE) begin
	 if(!avail && !wbm_lastdat) begin
	    $display("%m: No more data! Unhandled situation...");
	    $stop;
	 end
      end

      if(masterstate == MASTER_RD_DATA) begin
	 if(last_ack && !ready_to_send) begin
	    // Enable this for some debugging output
	    //	    $display("%m: Ending transaction early if necessary, read_count is %d",read_count[4:0]);
	 end
      end

      if(masterstate == MASTER_WR_ADDR) begin
	 if(!avail) begin
	    $display("%m: MASTER_WR_ADDR, nothing available");
	    $stop;
	 end
      end

   end // always @ (posedge clk_i)


   // State machine to monitor transactions
   // (This could be moved to a monitoring module)
   reg [4:0] lastdest;
   reg 	     transmitting;
   always @(posedge clk_i) begin
      if(rst_i) begin
	 transmitting <= 0;
	 lastdest <= 0;
      end else begin
	 if(soclink_o[46] && transmitting) begin
	    if(soclink_o[40:36] !== lastdest) begin
	       $display("%m: ERROR: Changing the destination register without terminating the transaction!");
	       $stop;
	    end
	    if(soclink_o[41]) begin
	       transmitting <= 0;
	    end
	 end else if(soclink_o[46]) begin
	    transmitting <= 1;
	    lastdest <= soclink_o[40:36];

	    if(soclink_o[41]) begin
	       transmitting <= 0;
	    end
	 end

      end
   end // always @ (posedge clk_i)

   // Wishbone transaction monitoring (of burst handling)
   reg inburst;
   always @(posedge clk_i) begin
      if(rst_i) begin
	 inburst <= 0;
      end else begin
	 if(wbs_rty_o) begin
	    inburst <= 0;
	 end else if(wbs_stb_i && wbs_cyc_i && wbs_cab_i) begin
	    inburst <= 1;
	 end else if(inburst && !(wbs_stb_i && wbs_cyc_i)) begin
	    $display("%m: Burst was aborted without lowering cab first!");
	    $stop;
	 end else if(!wbs_cab_i) begin
	    inburst <= 0;
	 end
      end
   end // always @ (posedge clk_i)

   // Some sanity check, this should be moved into the state machine just like
   // the other asynchronous error checks are done now.
   always @(posedge clk_i) begin
      if(masterstate == MASTER_IDLE) begin
	 if(read_in_progress) begin
	 end else if(readreq_avail && ready_to_send && (state == IDLE)) begin
	 end else if(avail) begin
	    if(incoming_req[35:34] == 2'b00) begin
	    end else if(incoming_req[35:33] == 3'b010) begin
	    end else if(incoming_req[35] == 1'b1) begin // if (incoming_req[35:33] == 3'b010)
	    end else begin // if (incoming_req[35:34] == 2'b01)
	       $display("%m: Got unknown request, aborting");
	       $stop;
	    end // else: !if(incoming_req[35:34] == 2'b01)
	 end
      end else if(masterstate == MASTER_RD_DATA) begin // if (masterstate == MASTER_IDLE)
	 if(!reverseroute) begin
	    $display("%m: No sel signals for reverse route, aborting");
	    $stop;
	 end
      end
   end // always @ (posedge clk_i)


   

`ifdef VERBOSE

   // I used this for statistics gathering during a benchmark, and it
   // is still here in case it turns up to be useful again.
   
   integer acks_1;
   integer acks_2;
   integer clkcount;

   initial begin
      clkcount = 0;
      acks_1 = 0;
      acks_2 = 0;
   end
   
   always @(posedge clk_i) begin
      if(wbs_ack_o) begin
	 acks_1 = acks_1 + 1;
      end

      if(wbm_ack_i) begin
	 acks_2 = acks_2 + 1;
      end
      
      if(clkcount == 1) begin
	 clkcount = 0;
	 $display("%m: WBS_ACK COUNT is %d",acks_1);
	 $display("%m: WBM_ACK COUNT is %d",acks_2);
	 acks_1 = 0;
	 acks_2 = 0;
      end else begin
	 clkcount = clkcount + 1;
      end
      
   end
`endif //  `ifdef VERBOSE
   
endmodule // wishbone_if


