`timescale 1ns / 10ps
`default_nettype none
  module wb_srlfifo(input wire clk,
    input wire rst,
     
    input wire [41:0] d,
    input wire 	      we,
    output reg 	      sendok,
  
    output wire       avail,
    output wire [41:0] q,

    input wire 	       rd
		    );


   wire [3:0] 	       addrptr;
   reg [3:0] 	       new_addrptr;
   wire [44:0] 	       srlq;
   wire 	       avail_in_ff;
   wire 	       avail_int;
   
   wire 	       fdce_sel;
   

   assign 	       avail_int = addrptr != 4'hf;

   parameter 	       LOWWATERMARK = 4'hb;
   
   always @(posedge clk) begin
      sendok <= (addrptr == 4'hf) || (addrptr < LOWWATERMARK);
   end



   wire 	      extread;
   assign 	      extread = rd;

   FDS addrptr0 (.C(clk),.S(rst),.D(new_addrptr[0]),.Q(addrptr[0]));
   FDS addrptr1 (.C(clk),.S(rst),.D(new_addrptr[1]),.Q(addrptr[1]));
   FDS addrptr2 (.C(clk),.S(rst),.D(new_addrptr[2]),.Q(addrptr[2]));
   FDS addrptr3 (.C(clk),.S(rst),.D(new_addrptr[3]),.Q(addrptr[3]));
   

   reg [3:0] 	      maxval = 0;

   always @(posedge clk) begin
      if(avail_int) 
	if(maxval < addrptr) begin
	   $display("%m: New maxval: %d",addrptr);
	   maxval <= addrptr;
	end
   end
   

   always @(*) begin
      new_addrptr = addrptr;
      if(we) begin
	 //	 $display("%m: Got input %x",d);
	 if(!((extread) && avail_int))  new_addrptr = addrptr + 1;
      end else if((extread) && avail_int) begin
	 new_addrptr = addrptr - 1;
      end

      if(addrptr == 4'he) begin
	 if(new_addrptr == 4'hf) begin
	    $display("%m: Fifo overflow!");
	    $stop;
	 end 
	 //else begin
	 //	    $display("%m: Fifo near overflow");
	 //	 end
      end 
   end // always @ (posedge clk)

   assign fdce_sel = extread;

   reg 	  avail_flag;
   always @(*) begin
      if(extread) begin
	 avail_flag = avail_int;
      end else begin
	 avail_flag = avail_in_ff;
      end
   end

   always @(posedge clk) begin
      if(extread && avail_in_ff) begin
	 //	 $display("%m: reading from FIFO: read value %x (destinations are %x)",q,{Asel,Bsel,Csel});
      end
   end
   

   //   always @(posedge clk) begin
   //      if(!avail_in_ff && (extread)) begin
   //	 $display("Trying to read from the fifo even though it is empty!");
   //	 $stop;
   //      end
   //   end

   FDR availfd (.C(clk),.R(rst),.D(avail_flag),.Q(avail_in_ff));
   assign avail = avail_in_ff;

   FDE fd0(.C(clk),.CE(fdce_sel),.D(srlq[0]),.Q(q[0]));
   FDE fd1(.C(clk),.CE(fdce_sel),.D(srlq[1]),.Q(q[1]));
   FDE fd2(.C(clk),.CE(fdce_sel),.D(srlq[2]),.Q(q[2]));
   FDE fd3(.C(clk),.CE(fdce_sel),.D(srlq[3]),.Q(q[3]));
   FDE fd4(.C(clk),.CE(fdce_sel),.D(srlq[4]),.Q(q[4]));
   FDE fd5(.C(clk),.CE(fdce_sel),.D(srlq[5]),.Q(q[5]));
   FDE fd6(.C(clk),.CE(fdce_sel),.D(srlq[6]),.Q(q[6]));
   FDE fd7(.C(clk),.CE(fdce_sel),.D(srlq[7]),.Q(q[7]));
   FDE fd8(.C(clk),.CE(fdce_sel),.D(srlq[8]),.Q(q[8]));
   FDE fd9(.C(clk),.CE(fdce_sel),.D(srlq[9]),.Q(q[9]));
   FDE fd10(.C(clk),.CE(fdce_sel),.D(srlq[10]),.Q(q[10]));
   FDE fd11(.C(clk),.CE(fdce_sel),.D(srlq[11]),.Q(q[11]));
   FDE fd12(.C(clk),.CE(fdce_sel),.D(srlq[12]),.Q(q[12]));
   FDE fd13(.C(clk),.CE(fdce_sel),.D(srlq[13]),.Q(q[13]));
   FDE fd14(.C(clk),.CE(fdce_sel),.D(srlq[14]),.Q(q[14]));
   FDE fd15(.C(clk),.CE(fdce_sel),.D(srlq[15]),.Q(q[15]));
   FDE fd16(.C(clk),.CE(fdce_sel),.D(srlq[16]),.Q(q[16]));
   FDE fd17(.C(clk),.CE(fdce_sel),.D(srlq[17]),.Q(q[17]));
   FDE fd18(.C(clk),.CE(fdce_sel),.D(srlq[18]),.Q(q[18]));
   FDE fd19(.C(clk),.CE(fdce_sel),.D(srlq[19]),.Q(q[19]));
   FDE fd20(.C(clk),.CE(fdce_sel),.D(srlq[20]),.Q(q[20]));
   FDE fd21(.C(clk),.CE(fdce_sel),.D(srlq[21]),.Q(q[21]));
   FDE fd22(.C(clk),.CE(fdce_sel),.D(srlq[22]),.Q(q[22]));
   FDE fd23(.C(clk),.CE(fdce_sel),.D(srlq[23]),.Q(q[23]));
   FDE fd24(.C(clk),.CE(fdce_sel),.D(srlq[24]),.Q(q[24]));
   FDE fd25(.C(clk),.CE(fdce_sel),.D(srlq[25]),.Q(q[25]));
   FDE fd26(.C(clk),.CE(fdce_sel),.D(srlq[26]),.Q(q[26]));
   FDE fd27(.C(clk),.CE(fdce_sel),.D(srlq[27]),.Q(q[27]));
   FDE fd28(.C(clk),.CE(fdce_sel),.D(srlq[28]),.Q(q[28]));
   FDE fd29(.C(clk),.CE(fdce_sel),.D(srlq[29]),.Q(q[29]));
   FDE fd30(.C(clk),.CE(fdce_sel),.D(srlq[30]),.Q(q[30]));
   FDE fd31(.C(clk),.CE(fdce_sel),.D(srlq[31]),.Q(q[31]));
   FDE fd32(.C(clk),.CE(fdce_sel),.D(srlq[32]),.Q(q[32]));
   FDE fd33(.C(clk),.CE(fdce_sel),.D(srlq[33]),.Q(q[33]));
   FDE fd34(.C(clk),.CE(fdce_sel),.D(srlq[34]),.Q(q[34]));
   FDE fd35(.C(clk),.CE(fdce_sel),.D(srlq[35]),.Q(q[35]));
   FDE fd36(.C(clk),.CE(fdce_sel),.D(srlq[36]),.Q(q[36]));
   FDE fd37(.C(clk),.CE(fdce_sel),.D(srlq[37]),.Q(q[37]));
   FDE fd38(.C(clk),.CE(fdce_sel),.D(srlq[38]),.Q(q[38]));
   FDE fd39(.C(clk),.CE(fdce_sel),.D(srlq[39]),.Q(q[39]));
   FDE fd40(.C(clk),.CE(fdce_sel),.D(srlq[40]),.Q(q[40]));
   FDE fd41(.C(clk),.CE(fdce_sel),.D(srlq[41]),.Q(q[41]));



   SRL16E srl00(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[0]),.D(d[0]));
   SRL16E srl01(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[1]),.D(d[1]));
   SRL16E srl02(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[2]),.D(d[2]));
   SRL16E srl03(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[3]),.D(d[3]));
   SRL16E srl04(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[4]),.D(d[4]));
   SRL16E srl05(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[5]),.D(d[5]));
   SRL16E srl06(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[6]),.D(d[6]));
   SRL16E srl07(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[7]),.D(d[7]));
   SRL16E srl08(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[8]),.D(d[8]));
   SRL16E srl09(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[9]),.D(d[9]));
   SRL16E srl10(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[10]),.D(d[10]));
   SRL16E srl11(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[11]),.D(d[11]));
   SRL16E srl12(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[12]),.D(d[12]));
   SRL16E srl13(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[13]),.D(d[13]));
   SRL16E srl14(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[14]),.D(d[14]));
   SRL16E srl15(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[15]),.D(d[15]));
   SRL16E srl16(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[16]),.D(d[16]));
   SRL16E srl17(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[17]),.D(d[17]));
   SRL16E srl18(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[18]),.D(d[18]));
   SRL16E srl19(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[19]),.D(d[19]));
   SRL16E srl20(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[20]),.D(d[20]));
   SRL16E srl21(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[21]),.D(d[21]));
   SRL16E srl22(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[22]),.D(d[22]));
   SRL16E srl23(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[23]),.D(d[23]));
   SRL16E srl24(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[24]),.D(d[24]));
   SRL16E srl25(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[25]),.D(d[25]));
   SRL16E srl26(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[26]),.D(d[26]));
   SRL16E srl27(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[27]),.D(d[27]));
   SRL16E srl28(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[28]),.D(d[28]));
   SRL16E srl29(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[29]),.D(d[29]));
   SRL16E srl30(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[30]),.D(d[30]));
   SRL16E srl31(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[31]),.D(d[31]));
   SRL16E srl32(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[32]),.D(d[32]));
   SRL16E srl33(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[33]),.D(d[33]));
   SRL16E srl34(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[34]),.D(d[34]));
   SRL16E srl35(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[35]),.D(d[35]));
   SRL16E srl36(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[36]),.D(d[36]));
   SRL16E srl37(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[37]),.D(d[37]));
   SRL16E srl38(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[38]),.D(d[38]));
   SRL16E srl39(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[39]),.D(d[39]));
   SRL16E srl40(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[40]),.D(d[40]));
   SRL16E srl41(.CLK(clk), .CE(we),.A0(addrptr[0]),.A1(addrptr[1]),.A2(addrptr[2]),.A3(addrptr[3]),.Q(srlq[41]),.D(d[41]));
endmodule // srlfifo
