`timescale 1ns / 10ps
`default_nettype none
module wb_srlfifo32(input wire clk,
	       input wire rst,
	       
	       input wire [41:0] d,
	       input wire we,
	       output reg sendok,
	       
	       output wire avail,
	       output wire [41:0] q,

	       input wire rd
	       );


   wire [4:0] 	      addrptr;
   reg [4:0]		      new_addrptr;
   wire [41:0] 	      srlq;
   wire 	      avail_in_ff;
   wire 	      avail_int;
   
   wire 	      fdce_sel;
   

   assign 	      avail_int = addrptr != 5'h1f;

   parameter 	      LOWWATERMARK = 5'h1b;
   always @(posedge clk) begin
      sendok <= (addrptr == 5'h1f) || (addrptr < LOWWATERMARK);
   end



   wire 	      extread;
   assign 	      extread = rd;

   FDS addrptr0 (.C(clk),.S(rst),.D(new_addrptr[0]),.Q(addrptr[0]));
   FDS addrptr1 (.C(clk),.S(rst),.D(new_addrptr[1]),.Q(addrptr[1]));
   FDS addrptr2 (.C(clk),.S(rst),.D(new_addrptr[2]),.Q(addrptr[2]));
   FDS addrptr3 (.C(clk),.S(rst),.D(new_addrptr[3]),.Q(addrptr[3]));
    FDS addrptr4 (.C(clk),.S(rst),.D(new_addrptr[4]),.Q(addrptr[4]));
   

   reg [3:0] 	      maxval = 0;

   always @(posedge clk) begin
      if(avail_int) 
	if(maxval < addrptr) begin
	   $display("%m: New maxval: %d",addrptr);
	   maxval <= addrptr;
	end
   end
   

   // Make sure we use 1 LUT / bit! (Can't be done)
   always @(*) begin
      new_addrptr = addrptr;
      if(we) begin
//	 $display("%m: Got input %x",d);
	 if(!((extread) && avail_int))  new_addrptr = addrptr + 1;
      end else if((extread) && avail_int) begin
	 //	    if(addrptr != 4'hf) begin
	 new_addrptr = addrptr - 1;
	 //	    end
      end

      if(addrptr == 5'h1e) begin
	 if(new_addrptr == 5'h1f) begin
	    $display("%m: Fifo overflow!");
	    $stop;
	 end 
//else begin
//	    $display("%m: Fifo near overflow");
//	 end
      end 
   end // always @ (posedge clk)

   assign fdce_sel = extread;

   reg 	  avail_flag;
   always @(*) begin
      if(extread) begin
	 avail_flag = avail_int;
      end else begin
	 avail_flag = avail_in_ff;
      end
   end

   always @(posedge clk) begin
      if(extread && avail_in_ff) begin
//	 $display("%m: reading from FIFO: read value %x (destinations are %x)",q,{Asel,Bsel,Csel});
      end
   end
   

   FDR availfd (.C(clk),.R(rst),.D(avail_flag),.Q(avail_in_ff));
   assign avail = avail_in_ff;

FDE fd0(.C(clk),.CE(fdce_sel),.D(srlq[0]),.Q(q[0]));
   FDE fd1(.C(clk),.CE(fdce_sel),.D(srlq[1]),.Q(q[1]));
   FDE fd2(.C(clk),.CE(fdce_sel),.D(srlq[2]),.Q(q[2]));
   FDE fd3(.C(clk),.CE(fdce_sel),.D(srlq[3]),.Q(q[3]));
   FDE fd4(.C(clk),.CE(fdce_sel),.D(srlq[4]),.Q(q[4]));
   FDE fd5(.C(clk),.CE(fdce_sel),.D(srlq[5]),.Q(q[5]));
   FDE fd6(.C(clk),.CE(fdce_sel),.D(srlq[6]),.Q(q[6]));
   FDE fd7(.C(clk),.CE(fdce_sel),.D(srlq[7]),.Q(q[7]));
   FDE fd8(.C(clk),.CE(fdce_sel),.D(srlq[8]),.Q(q[8]));
   FDE fd9(.C(clk),.CE(fdce_sel),.D(srlq[9]),.Q(q[9]));
   FDE fd10(.C(clk),.CE(fdce_sel),.D(srlq[10]),.Q(q[10]));
   FDE fd11(.C(clk),.CE(fdce_sel),.D(srlq[11]),.Q(q[11]));
   FDE fd12(.C(clk),.CE(fdce_sel),.D(srlq[12]),.Q(q[12]));
   FDE fd13(.C(clk),.CE(fdce_sel),.D(srlq[13]),.Q(q[13]));
   FDE fd14(.C(clk),.CE(fdce_sel),.D(srlq[14]),.Q(q[14]));
   FDE fd15(.C(clk),.CE(fdce_sel),.D(srlq[15]),.Q(q[15]));
   FDE fd16(.C(clk),.CE(fdce_sel),.D(srlq[16]),.Q(q[16]));
   FDE fd17(.C(clk),.CE(fdce_sel),.D(srlq[17]),.Q(q[17]));
   FDE fd18(.C(clk),.CE(fdce_sel),.D(srlq[18]),.Q(q[18]));
   FDE fd19(.C(clk),.CE(fdce_sel),.D(srlq[19]),.Q(q[19]));
    FDE fd20(.C(clk),.CE(fdce_sel),.D(srlq[20]),.Q(q[20]));
    FDE fd21(.C(clk),.CE(fdce_sel),.D(srlq[21]),.Q(q[21]));
    FDE fd22(.C(clk),.CE(fdce_sel),.D(srlq[22]),.Q(q[22]));
    FDE fd23(.C(clk),.CE(fdce_sel),.D(srlq[23]),.Q(q[23]));
    FDE fd24(.C(clk),.CE(fdce_sel),.D(srlq[24]),.Q(q[24]));
    FDE fd25(.C(clk),.CE(fdce_sel),.D(srlq[25]),.Q(q[25]));
    FDE fd26(.C(clk),.CE(fdce_sel),.D(srlq[26]),.Q(q[26]));
    FDE fd27(.C(clk),.CE(fdce_sel),.D(srlq[27]),.Q(q[27]));
    FDE fd28(.C(clk),.CE(fdce_sel),.D(srlq[28]),.Q(q[28]));
    FDE fd29(.C(clk),.CE(fdce_sel),.D(srlq[29]),.Q(q[29]));
    FDE fd30(.C(clk),.CE(fdce_sel),.D(srlq[30]),.Q(q[30]));
    FDE fd31(.C(clk),.CE(fdce_sel),.D(srlq[31]),.Q(q[31]));
    FDE fd32(.C(clk),.CE(fdce_sel),.D(srlq[32]),.Q(q[32]));
    FDE fd33(.C(clk),.CE(fdce_sel),.D(srlq[33]),.Q(q[33]));
    FDE fd34(.C(clk),.CE(fdce_sel),.D(srlq[34]),.Q(q[34]));
    FDE fd35(.C(clk),.CE(fdce_sel),.D(srlq[35]),.Q(q[35]));
    FDE fd36(.C(clk),.CE(fdce_sel),.D(srlq[36]),.Q(q[36]));
    FDE fd37(.C(clk),.CE(fdce_sel),.D(srlq[37]),.Q(q[37]));
    FDE fd38(.C(clk),.CE(fdce_sel),.D(srlq[38]),.Q(q[38]));
    FDE fd39(.C(clk),.CE(fdce_sel),.D(srlq[39]),.Q(q[39]));
    FDE fd40(.C(clk),.CE(fdce_sel),.D(srlq[40]),.Q(q[40]));
    FDE fd41(.C(clk),.CE(fdce_sel),.D(srlq[41]),.Q(q[41]));



      srl32e srl00(.clk(clk), .we(we),.addr(addrptr),.q(srlq[0]),.d(d[0]));
      srl32e srl01(.clk(clk), .we(we),.addr(addrptr),.q(srlq[1]),.d(d[1]));
      srl32e srl02(.clk(clk), .we(we),.addr(addrptr),.q(srlq[2]),.d(d[2]));
      srl32e srl03(.clk(clk), .we(we),.addr(addrptr),.q(srlq[3]),.d(d[3]));
      srl32e srl04(.clk(clk), .we(we),.addr(addrptr),.q(srlq[4]),.d(d[4]));
      srl32e srl05(.clk(clk), .we(we),.addr(addrptr),.q(srlq[5]),.d(d[5]));
      srl32e srl06(.clk(clk), .we(we),.addr(addrptr),.q(srlq[6]),.d(d[6]));
      srl32e srl07(.clk(clk), .we(we),.addr(addrptr),.q(srlq[7]),.d(d[7]));
      srl32e srl08(.clk(clk), .we(we),.addr(addrptr),.q(srlq[8]),.d(d[8]));
      srl32e srl09(.clk(clk), .we(we),.addr(addrptr),.q(srlq[9]),.d(d[9]));
      srl32e srl10(.clk(clk), .we(we),.addr(addrptr),.q(srlq[10]),.d(d[10]));
      srl32e srl11(.clk(clk), .we(we),.addr(addrptr),.q(srlq[11]),.d(d[11]));
      srl32e srl12(.clk(clk), .we(we),.addr(addrptr),.q(srlq[12]),.d(d[12]));
      srl32e srl13(.clk(clk), .we(we),.addr(addrptr),.q(srlq[13]),.d(d[13]));
      srl32e srl14(.clk(clk), .we(we),.addr(addrptr),.q(srlq[14]),.d(d[14]));
      srl32e srl15(.clk(clk), .we(we),.addr(addrptr),.q(srlq[15]),.d(d[15]));
      srl32e srl16(.clk(clk), .we(we),.addr(addrptr),.q(srlq[16]),.d(d[16]));
      srl32e srl17(.clk(clk), .we(we),.addr(addrptr),.q(srlq[17]),.d(d[17]));
      srl32e srl18(.clk(clk), .we(we),.addr(addrptr),.q(srlq[18]),.d(d[18]));
      srl32e srl19(.clk(clk), .we(we),.addr(addrptr),.q(srlq[19]),.d(d[19]));
      srl32e srl20(.clk(clk), .we(we),.addr(addrptr),.q(srlq[20]),.d(d[20]));
      srl32e srl21(.clk(clk), .we(we),.addr(addrptr),.q(srlq[21]),.d(d[21]));
      srl32e srl22(.clk(clk), .we(we),.addr(addrptr),.q(srlq[22]),.d(d[22]));
      srl32e srl23(.clk(clk), .we(we),.addr(addrptr),.q(srlq[23]),.d(d[23]));
      srl32e srl24(.clk(clk), .we(we),.addr(addrptr),.q(srlq[24]),.d(d[24]));
      srl32e srl25(.clk(clk), .we(we),.addr(addrptr),.q(srlq[25]),.d(d[25]));
      srl32e srl26(.clk(clk), .we(we),.addr(addrptr),.q(srlq[26]),.d(d[26]));
      srl32e srl27(.clk(clk), .we(we),.addr(addrptr),.q(srlq[27]),.d(d[27]));
      srl32e srl28(.clk(clk), .we(we),.addr(addrptr),.q(srlq[28]),.d(d[28]));
      srl32e srl29(.clk(clk), .we(we),.addr(addrptr),.q(srlq[29]),.d(d[29]));
      srl32e srl30(.clk(clk), .we(we),.addr(addrptr),.q(srlq[30]),.d(d[30]));
      srl32e srl31(.clk(clk), .we(we),.addr(addrptr),.q(srlq[31]),.d(d[31]));
      srl32e srl32(.clk(clk), .we(we),.addr(addrptr),.q(srlq[32]),.d(d[32]));
      srl32e srl33(.clk(clk), .we(we),.addr(addrptr),.q(srlq[33]),.d(d[33]));
      srl32e srl34(.clk(clk), .we(we),.addr(addrptr),.q(srlq[34]),.d(d[34]));
      srl32e srl35(.clk(clk), .we(we),.addr(addrptr),.q(srlq[35]),.d(d[35]));
      srl32e srl36(.clk(clk), .we(we),.addr(addrptr),.q(srlq[36]),.d(d[36]));
      srl32e srl37(.clk(clk), .we(we),.addr(addrptr),.q(srlq[37]),.d(d[37]));
      srl32e srl38(.clk(clk), .we(we),.addr(addrptr),.q(srlq[38]),.d(d[38]));
      srl32e srl39(.clk(clk), .we(we),.addr(addrptr),.q(srlq[39]),.d(d[39]));
      srl32e srl40(.clk(clk), .we(we),.addr(addrptr),.q(srlq[40]),.d(d[40]));
      srl32e srl41(.clk(clk), .we(we),.addr(addrptr),.q(srlq[41]),.d(d[41]));
endmodule // srlfifo
