`timescale 1ns / 10ps


// This was intended to be a cam memory so that more than one transaction
// could be kept in flight at the same time. Changing this file to support
// that would be trivial but changing wb_if.v is more troublesome even if
// some of the framework is already there.

module wb_transactioncam(
    input wire clk,rst,
    input wire [35:0] key,
    output wire [2:0]  first_req,
    output wire avail, // available space
	      
    input wire we,
    output wire [2:0] match_req,
    output reg       match,

    input wire [2:0]  del_req,
    input wire 	      del);

   
   parameter [2:0]    num_entries = 3'd1; // Only one entry for now...


   reg [35:0] 	      current[num_entries - 1:0];
   reg [0:0]	      inuse;

   assign 	      avail = ~inuse ? 1 : 0;

   assign 	      first_req = 3'd0;
   

   always @(posedge clk) begin
      if(rst) begin
	 current[0] <= 35'b0;
	 inuse[0] <= 0;
      end else begin
	 if(we) begin
	    current[0] <= key;
	    inuse[0] <= 1;
	 end

	 if(del) begin
	    inuse[0] <= 0;
	 end
      end
   end

   always @(posedge clk) begin
      match <= (key == current[0]) && inuse[0];
   end

   assign match_req = 3'd0;
   


endmodule // wb_transactioncam
